@@ -1,7 +1,7 @@
 /*
  * rasqal_datetime.c - Rasqal XSD dateTime and XSD date
  *
- * Copyright (C) 2007-2010, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2011, David Beckett http://www.dajobe.org/
  *
  * Contributions:
  *   Copyright (C) 2007, Lauri Aalto <laalto@iki.fi>
@@ -34,6 +34,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -45,33 +48,11 @@
 
 /* Local definitions */
  
-/**
- * rasqal_xsd_datetime:
- *
- * INTERNAL - XML schema dateTime datatype
- *
- * Signed types are required for normalization process where a value
- * can be negative temporarily.
- */
-typedef struct {
-  signed int year;
-  /* the following fields are integer values not characters */
-  unsigned char month;
-  unsigned char day;
-  signed char hour;
-  signed char minute;
-  signed char second;
-  /* second_frac is a string of 1-3 length (+1 for NUL)
-   * supports only up to milliseconds
-   */
-  char second_frac[3+1];
-  /* have_tz is an integer flag: non-0 if 'Z'ulu timezone is present */
-  char have_tz;
-}  rasqal_xsd_datetime;
-
-
 /**
  * rasqal_xsd_date:
+ * @year: year
+ * @month: month 1-12
+ * @day: 0-31
  *
  * INTERNAL - XML schema date datatype
  *
@@ -84,8 +65,7 @@ typedef struct {
 } rasqal_xsd_date;
 
 
-static int rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_string, rasqal_xsd_datetime *result, int is_dateTime);
-static unsigned char *rasqal_xsd_datetime_to_string(const rasqal_xsd_datetime *dt);
+static int rasqal_xsd_datetime_parse(const char *datetime_string, rasqal_xsd_datetime *result, int is_dateTime);
 static unsigned int days_per_month(int month, int year);
 
 
@@ -106,6 +86,16 @@ static int
 rasqal_xsd_datetime_normalize(rasqal_xsd_datetime *datetime)
 {
   int t;
+
+  if(datetime->timezone_minutes &&
+     (datetime->timezone_minutes  != RASQAL_XSD_DATETIME_NO_TZ)) {
+    /* Normalize to Zulu if there was a timezone offset */
+
+    datetime->hour   += (datetime->timezone_minutes / 60);
+    datetime->minute += (datetime->timezone_minutes % 60);
+
+    datetime->timezone_minutes = 0;
+  }
   
   /* second & second parts: no need to normalize as they are not
    * touched after range check
@@ -166,13 +156,16 @@ rasqal_xsd_datetime_normalize(rasqal_xsd_datetime *datetime)
 
 
 /**
- * rasqal_xsd_datetime_parse_and_normalize_common:
+ * rasqal_xsd_datetime_parse:
  * @datetime_string: xsd:dateTime as lexical form string
  * @result: target struct for holding dateTime components
  * @is_dateTime: is xsd:dateTime and should look for time (hour, mins, secs)
  *   otherwise is xsd:date and should skip to looking for timezone
  *
- * INTERNAL - Parse a xsd:dateTime string to a normalized #rasqal_xsd_datetime struct.
+ * INTERNAL - Parse a xsd:dateTime string into a #rasqal_xsd_datetime struct.
+ *
+ * Does NOT normalize the structure.  Call
+ * rasqal_xsd_datetime_normalize() to do that.
  *
  * http://www.w3.org/TR/xmlschema-2/#dt-dateTime
  *
@@ -207,14 +200,15 @@ rasqal_xsd_datetime_normalize(rasqal_xsd_datetime *datetime)
  * Return value: zero on success, non zero on failure.
  */
 static int
-rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_string,
-                                               rasqal_xsd_datetime *result,
-                                               int is_dateTime)
+rasqal_xsd_datetime_parse(const char *datetime_string,
+                          rasqal_xsd_datetime *result,
+                          int is_dateTime)
 {
   const char *p, *q; 
   char b[16];
   unsigned int l, t, t2, is_neg;
   unsigned long u;
+#define MICROSECONDS_MAX_DIGITS 6
 
   if(!datetime_string || !result)
     return -1;
@@ -381,7 +375,7 @@ rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_str
       return -7;
 
     /* parse fraction seconds if any */
-    result->second_frac[0] = 0;
+    result->microseconds = 0;
     if(*p == '.') {
       for(q = ++p; ISNUM(*p); p++)
         ;
@@ -394,17 +388,22 @@ rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_str
       if(!(*q == '0' && q == p)) {
         /* allow ".0" */
         l = p - q;
-        /* support only to milliseconds with truncation */
-        if(l > sizeof(result->second_frac)-1)
-          l=sizeof(result->second_frac)-1;
 
-        if(l<1) /* need at least 1 num */
+        if(l < 1) /* need at least 1 num */
           return -8;
 
-        for(t2 = 0; t2 < l; ++t2)
-          result->second_frac[t2] = *q++;
+        /* support only to microseconds with truncation */
+        if(l > MICROSECONDS_MAX_DIGITS)
+          l = MICROSECONDS_MAX_DIGITS;
+        
+        result->microseconds = 0;
+        for(t2 = 0; t2 < MICROSECONDS_MAX_DIGITS; ++t2) {
+          if(t2 < l)
+            result->microseconds += (*q++ - '0');
+          if(t2 != MICROSECONDS_MAX_DIGITS - 1)
+            result->microseconds *= 10;
+        }
 
-        result->second_frac[l] = 0;
       }
 
       /* skip ignored trailing zeros */
@@ -417,19 +416,21 @@ rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_str
     result->hour = 12;
     result->minute = 0;
     result->second = 0;
-    result->second_frac[0] = 0;
+    result->microseconds = 0;
   }
 
   
   /* parse & adjust timezone offset */
   /* result is normalized later */
-  result->have_tz = 0;
+  result->timezone_minutes = RASQAL_XSD_DATETIME_NO_TZ;
   if(*p) {
     if(*p == 'Z') {
       /* utc timezone - no need to adjust */
+      result->timezone_minutes = 0;
       p++;
-      result->have_tz = 1;
     } else if(*p == '+' || *p == '-') {
+      result->timezone_minutes = 0;
+
       /* work out timezone offsets */
       is_neg = *p == '-';
      
@@ -449,7 +450,7 @@ rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_str
         return -9;
     
       /* negative tz offset adds to the result */
-      result->hour += is_neg ? t2 : -t2;
+      result->timezone_minutes += (is_neg ? t2 : -t2) * 60;
 
       /* timezone minutes */    
       for(q = ++p; ISNUM(*p); p++)
@@ -469,8 +470,7 @@ rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_str
       }
     
       /* negative tz offset adds to the result */
-      result->minute += is_neg ? t : -t;
-      result->have_tz = 1;
+      result->timezone_minutes += is_neg ? t : -t;
     }
     
     /* failure if extra chars after the timezone part */
@@ -479,27 +479,30 @@ rasqal_xsd_datetime_parse_and_normalize_common(const unsigned char *datetime_str
 
   }
 
+  return 0;
+}
+
+
+static int
+rasqal_xsd_datetime_parse_and_normalize(const char *datetime_string,
+                                        rasqal_xsd_datetime *result)
+{
+  if(rasqal_xsd_datetime_parse(datetime_string, result, 1))
+    return 1;
+  
   return rasqal_xsd_datetime_normalize(result);
 }
 
-
 static int
-rasqal_xsd_datetime_parse_and_normalize(const unsigned char *datetime_string,
-                                        rasqal_xsd_datetime *result)
-{
-  return rasqal_xsd_datetime_parse_and_normalize_common(datetime_string,
-                                                        result, 1);
-}
-
-static int
-rasqal_xsd_date_parse_and_normalize(const unsigned char *date_string,
+rasqal_xsd_date_parse_and_normalize(const char *date_string,
                                     rasqal_xsd_date *result)
 {
   rasqal_xsd_datetime dt_result; /* on stack */
   int rc;
 
-  rc = rasqal_xsd_datetime_parse_and_normalize_common(date_string,
-                                                      &dt_result, 0);
+  rc = rasqal_xsd_datetime_parse(date_string, &dt_result, 0);
+  if(!rc)
+    rc = rasqal_xsd_datetime_normalize(&dt_result);
 
   if(!rc) {
     result->year = dt_result.year;
@@ -510,47 +513,216 @@ rasqal_xsd_date_parse_and_normalize(const unsigned char *date_string,
   return rc;
 }
 
+
 /**
- * rasqal_xsd_datetime_to_string:
+ * rasqal_new_xsd_datetime:
+ * @world: world object
+ * @datetime_string: XSD Datetime string
+ *
+ * Constructor - make a new XSD datetime object from a string
+ *
+ * Return value: new datetime or NULL on failure
+ */
+rasqal_xsd_datetime*
+rasqal_new_xsd_datetime(rasqal_world* world, const char *datetime_string)
+{
+  rasqal_xsd_datetime* dt;
+  int rc = 0;
+  
+  dt = (rasqal_xsd_datetime*)RASQAL_MALLOC(datetime, sizeof(*dt));
+  if(!dt)
+    return NULL;
+  
+  rc = rasqal_xsd_datetime_parse(datetime_string, dt, 1);
+  if(!rc)
+    rc = rasqal_xsd_datetime_normalize(dt);
+
+  if(rc) {
+    rasqal_free_xsd_datetime(dt); dt = NULL;
+  }
+
+  return dt;
+}
+
+
+/**
+ * rasqal_new_xsd_datetime_from_unixtime:
+ * @world: world object
+ * @secs: unixtime
+ *
+ * Constructor - make a new XSD datetime object from unixtime seconds
+ *
+ * Return value: new datetime or NULL on failure
+ */
+rasqal_xsd_datetime*
+rasqal_new_xsd_datetime_from_unixtime(rasqal_world* world, time_t secs)
+{
+  rasqal_xsd_datetime* dt;
+  int rc = 0;
+  
+  dt = (rasqal_xsd_datetime*)RASQAL_MALLOC(datetime, sizeof(*dt));
+  if(!dt)
+    return NULL;
+
+  rc = rasqal_xsd_datetime_set_from_unixtime(dt, secs);
+
+  if(rc) {
+    rasqal_free_xsd_datetime(dt); dt = NULL;
+  }
+
+  return dt;
+}
+
+
+/**
+ * rasqal_new_xsd_datetime_from_timeval:
+ * @world: world object
+ * @tv: pointer to struct timeval
+ *
+ * Constructor - make a new XSD datetime object from a timeval
+ *
+ * Return value: new datetime or NULL on failure
+ */
+rasqal_xsd_datetime*
+rasqal_new_xsd_datetime_from_timeval(rasqal_world* world, struct timeval *tv)
+{
+  rasqal_xsd_datetime* dt;
+  int rc = 0;
+  
+  dt = (rasqal_xsd_datetime*)RASQAL_MALLOC(datetime, sizeof(*dt));
+  if(!dt)
+    return NULL;
+
+  rc = rasqal_xsd_datetime_set_from_timeval(dt, tv);
+
+  if(rc) {
+    rasqal_free_xsd_datetime(dt); dt = NULL;
+  }
+
+  return dt;
+}
+
+
+/**
+ * rasqal_free_xsd_datetime:
+ * @dt: datetime object
+ * 
+ * Destroy XSD datetime object.
+ **/
+void
+rasqal_free_xsd_datetime(rasqal_xsd_datetime* dt)
+{
+  if(!dt)
+    return;
+  
+  RASQAL_FREE(datetime, dt);
+}
+
+
+#define TIMEZONE_BUFFER_LEN 7
+
+/*
+ * rasqal_xsd_datetime_timezone_format:
+ * @dt: datetime
+ * @buffer: buffer to write the formatted timezone
+ * @len: length of buffer; must be 7 or larger
+ *
+ * INTERNAL - format a timezone into the passed in buffer
+ *
+ * Reutnr value: non-0 on failure
+ */
+static int
+rasqal_xsd_datetime_timezone_format(const rasqal_xsd_datetime *dt,
+                                    char* buffer, size_t len)
+{
+  int mins;
+  
+  if(!buffer || len < TIMEZONE_BUFFER_LEN)
+    return 1;
+  
+  mins = abs(dt->timezone_minutes);
+  if(mins == RASQAL_XSD_DATETIME_NO_TZ)
+    buffer[0] = '\0';
+  else if(!mins)
+    memcpy(buffer, "Z", 2);
+  else {
+    int hrs = (mins / 60);
+    snprintf(buffer, len, "%c%02d:%02d", 
+             (mins != dt->timezone_minutes ? '-' : '+'),
+             hrs, mins);
+  }
+
+  return 0;
+}
+
+
+/**
+ * rasqal_xsd_datetime_to_counted_string:
  * @dt: datetime struct
+ * @len_p: output length (or NULL)
  *
- * INTERNAL - Convert a #rasqal_xsd_datetime struct to a xsd:dateTime lexical form string.
+ * Convert a #rasqal_xsd_datetime struct to a xsd:dateTime lexical form counted string.
  *
- * Caller should RASQAL_FREE() the returned string.
+ * Caller should rasqal_free_memory() the returned string.
  *
  * Return value: lexical form string or NULL on failure.
  */
-static unsigned char*
-rasqal_xsd_datetime_to_string(const rasqal_xsd_datetime *dt)
+char*
+rasqal_xsd_datetime_to_counted_string(const rasqal_xsd_datetime *dt,
+                                      size_t *len_p)
 {
-  unsigned char *ret=0;
+  char *ret = 0;
   int is_neg;
-  int r=0;
+  int r = 0;
   int i;
-   
+  /* "[+-]HH:MM\0" */
+  char timezone_string[TIMEZONE_BUFFER_LEN];
+  
   if(!dt)
     return NULL;
     
-  is_neg=dt->year<0;
+  is_neg = dt->year < 0;
+
+  if(rasqal_xsd_datetime_timezone_format(dt, timezone_string,
+                                         TIMEZONE_BUFFER_LEN))
+    return NULL;
 
   /* format twice: first with null buffer of zero size to get the
    * required buffer size second time to the allocated buffer
    */
-  for(i=0; i < 2; i++) {
-    r=snprintf((char*)ret, r, "%s%04d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d%s%s%s",
-      is_neg ? "-" : "",
-      is_neg ? -dt->year : dt->year,
-      dt->month,
-      dt->day,
-      dt->hour,
-      dt->minute,
-      dt->second,
-      *dt->second_frac ? "." : "",
-      dt->second_frac,
-      dt->have_tz ? "Z" : "");
-
+  for(i = 0; i < 2; i++) { 
+    if(dt->microseconds) {
+      char microsecs[9];
+      int j;
+      
+      snprintf(microsecs, 9, "%.6f", ((double)dt->microseconds) / 1000000);
+      for(j = strlen(microsecs) - 1; j > 2 && microsecs[j] == '0'; j--)
+        microsecs[j] = '\0';
+      
+      r = snprintf(ret, r, "%s%04d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d%s%s",
+                   is_neg ? "-" : "",
+                   is_neg ? -dt->year : dt->year,
+                   dt->month,
+                   dt->day,
+                   dt->hour,
+                   dt->minute,
+                   dt->second,
+                   microsecs+1,
+                   timezone_string);
+    } else {
+      r = snprintf((char*)ret, r, "%s%04d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d%s",
+                   is_neg ? "-" : "",
+                   is_neg ? -dt->year : dt->year,
+                   dt->month,
+                   dt->day,
+                   dt->hour,
+                   dt->minute,
+                   dt->second,
+                   timezone_string);
+    }
+    
     /* error? */
-    if(r<0) {
+    if(r < 0) {
       if(ret)
         RASQAL_FREE(cstring, ret);
       return NULL;
@@ -558,7 +730,10 @@ rasqal_xsd_datetime_to_string(const rasqal_xsd_datetime *dt)
 
     /* alloc return buffer on first pass */
     if(!i) {
-      ret=(unsigned char *)RASQAL_MALLOC(cstring, ++r);
+      if(len_p)
+        *len_p = r;
+
+      ret = (char*)RASQAL_MALLOC(cstring, ++r);
       if(!ret)
         return NULL;
     }
@@ -567,6 +742,23 @@ rasqal_xsd_datetime_to_string(const rasqal_xsd_datetime *dt)
 }
 
 
+/**
+ * rasqal_xsd_datetime_to_string:
+ * @dt: datetime struct
+ *
+ * Convert a #rasqal_xsd_datetime struct to a xsd:dateTime lexical form string.
+ *
+ * Caller should rasqal_free_memory() the returned string.
+ *
+ * Return value: lexical form string or NULL on failure.
+ */
+char*
+rasqal_xsd_datetime_to_string(const rasqal_xsd_datetime *dt)
+{
+  return rasqal_xsd_datetime_to_counted_string(dt, NULL);
+}
+
+
 /**
  * rasqal_xsd_datetime_string_to_canonical:
  * @datetime_string: xsd:dateTime as lexical form string
@@ -590,24 +782,141 @@ rasqal_xsd_datetime_to_string(const rasqal_xsd_datetime *dt)
  *    * for timezoned values, the timezone must be represented with 'Z'
  *      (All timezoned dateTime values are UTC.)."
  */
-const unsigned char*
-rasqal_xsd_datetime_string_to_canonical(const unsigned char* datetime_string)
+const char*
+rasqal_xsd_datetime_string_to_canonical(const char* datetime_string)
 {
   rasqal_xsd_datetime d; /* allocated on stack */
 
   /* parse_and_normalize makes the rasqal_xsd_datetime canonical... */
   if(rasqal_xsd_datetime_parse_and_normalize(datetime_string, &d))
     return NULL;
+
   /* ... so return a string representation of it */
   return rasqal_xsd_datetime_to_string(&d);
 }
 
 
 
+/**
+ * rasqal_xsd_datetime_equals:
+ * @dt1: first XSD dateTime
+ * @dt2: second XSD dateTime
+ * 
+ * Compare two XSD dateTimes for equality.
+ * 
+ * Return value: non-0 if equal.
+ **/
+int
+rasqal_xsd_datetime_equals(const rasqal_xsd_datetime *dt1,
+                           const rasqal_xsd_datetime *dt2)
+{
+  /* Handle NULLs */
+  if(!dt1 || !dt2) {
+    /* equal only if both are NULL */
+    return (dt1 && dt2);
+  }
+  
+  return ((dt1->year == dt2->year) &&
+          (dt1->month == dt2->month) &&
+          (dt1->day == dt2->day) &&
+          (dt1->hour == dt2->hour) &&
+          (dt1->minute == dt2->minute) &&
+          (dt1->second == dt2->second) &&
+          (dt1->microseconds == dt2->microseconds));
+}
+
+
+/**
+ * rasqal_xsd_datetime_compare:
+ * @dt1: first XSD dateTime
+ * @dt2: second XSD dateTime
+ * 
+ * Compare two XSD dateTimes
+ * 
+ * Return value: <0 if @dt1 is less than @dt2, 0 if equal, >1 otherwise
+ **/
+int
+rasqal_xsd_datetime_compare(const rasqal_xsd_datetime *dt1,
+                            const rasqal_xsd_datetime *dt2)
+{
+  int rc;
+
+  /* Handle NULLs */
+  if(!dt1 || !dt2) {
+    /* NULLs sort earlier. equal only if both are NULL */
+    if(dt1 && dt2)
+      return 0;
+
+    return (!dt1) ? -1 : 1;
+  }
+  
+  rc = dt1->year - dt2->year;
+  if(rc)
+    return rc;
+
+  rc = dt1->month - dt2->month;
+  if(rc)
+    return rc;
+
+  rc = dt1->day - dt2->day;
+  if(rc)
+    return rc;
+
+  rc = dt1->hour - dt2->hour;
+  if(rc)
+    return rc;
+
+  rc = dt1->minute - dt2->minute;
+  if(rc)
+    return rc;
+
+  rc = dt1->second - dt2->second;
+  if(rc)
+    return rc;
+
+  rc = dt1->microseconds - dt2->microseconds;
+
+  return rc;
+}
+
+
+/**
+ * rasqal_xsd_datetime_get_seconds_as_decimal:
+ * @world: world object
+ * @dt: XSD dateTime
+ * 
+ * Get the seconds component of a dateTime as a decimal
+ * 
+ * Return value: decimal object or NULL on failure
+ **/
+rasqal_xsd_decimal*
+rasqal_xsd_datetime_get_seconds_as_decimal(rasqal_world* world,
+                                           rasqal_xsd_datetime* dt)
+{
+  rasqal_xsd_decimal* dec;
+
+  dec = rasqal_new_xsd_decimal(world);
+  if(!dec)
+    return NULL;
+  
+  if(!dt->microseconds) {
+    rasqal_xsd_decimal_set_long(dec, (long)dt->second);
+  } else {
+    /* Max len 9 "SS.UUUUUU\0" */
+    char str[9];
+
+    sprintf(str, "%d.%06d", dt->second, dt->microseconds);
+
+    rasqal_xsd_decimal_set_string(dec, str);
+  }
+  
+  return dec;
+}
+
 
 /**
  * rasqal_xsd_date_to_string:
- * @dt: date struct
+ * @d: date struct
  *
  * INTERNAL - Convert a #rasqal_xsd_date struct to a xsd:date lexical form string.
  *
@@ -615,10 +924,10 @@ rasqal_xsd_datetime_string_to_canonical(const unsigned char* datetime_string)
  *
  * Return value: lexical form string or NULL on failure.
  */
-static unsigned char*
+static char*
 rasqal_xsd_date_to_string(const rasqal_xsd_date *d)
 {
-  unsigned char *ret = 0;
+  char *ret = 0;
   int is_neg;
   int r = 0;
   int i;
@@ -647,7 +956,7 @@ rasqal_xsd_date_to_string(const rasqal_xsd_date *d)
 
     /* alloc return buffer on first pass */
     if(!i) {
-      ret = (unsigned char *)RASQAL_MALLOC(cstring, ++r);
+      ret = (char*)RASQAL_MALLOC(cstring, ++r);
       if(!ret)
         return NULL;
     }
@@ -677,14 +986,15 @@ rasqal_xsd_date_to_string(const rasqal_xsd_date *d)
  * 'T' and all following characters). For timezoned values, append
  * the canonical representation of the ·recoverable timezone·. "
  */
-const unsigned char*
-rasqal_xsd_date_string_to_canonical(const unsigned char* date_string)
+const char*
+rasqal_xsd_date_string_to_canonical(const char* date_string)
 {
   rasqal_xsd_date d; /* allocated on stack */
 
   /* parse_and_normalize makes the rasqal_xsd_date canonical... */
   if(rasqal_xsd_date_parse_and_normalize(date_string, &d))
     return NULL;
+
   /* ... so return a string representation of it */
   return rasqal_xsd_date_to_string(&d);
 }
@@ -743,7 +1053,7 @@ days_per_month(int month, int year) {
 
 
 int
-rasqal_xsd_datetime_check(const unsigned char* string)
+rasqal_xsd_datetime_check(const char* string)
 {
   rasqal_xsd_datetime d;
   
@@ -755,7 +1065,7 @@ rasqal_xsd_datetime_check(const unsigned char* string)
 
 
 int
-rasqal_xsd_date_check(const unsigned char* string)
+rasqal_xsd_date_check(const char* string)
 {
   rasqal_xsd_date d;
   
@@ -766,8 +1076,281 @@ rasqal_xsd_date_check(const unsigned char* string)
 }
 
 
+#define TM_YEAR_ORIGIN 1900
+#define TM_MONTH_ORIGIN 1
+
+/**
+ * rasqal_xsd_datetime_set_from_timeval:
+ * @dt: datetime
+ * @tv: timeval
+ * 
+ * Set an XSD dateTime from a struct timeval pointer
+ * 
+ * Returns: non-0 on failure
+ **/
+int
+rasqal_xsd_datetime_set_from_timeval(rasqal_xsd_datetime *dt,
+                                     struct timeval *tv)
+{
+  struct tm* my_time;
+#ifdef HAVE_GMTIME_R
+  struct tm time_buf;
+#endif
+  time_t sec;
+
+  if(!dt || !tv)
+    return 1;
+
+  sec = (time_t)tv->tv_sec;
+#ifdef HAVE_GMTIME_R
+  memset(&time_buf, '\0', sizeof(time_buf));
+  my_time = gmtime_r(&sec, &time_buf);
+#else
+  my_time = gmtime(&sec);
+#endif
+  if(!my_time)
+    return 1;
+
+  dt->year = my_time->tm_year + TM_YEAR_ORIGIN;
+  dt->month = my_time->tm_mon + TM_MONTH_ORIGIN;
+  dt->day = my_time->tm_mday;
+  dt->hour = my_time->tm_hour;
+  dt->minute = my_time->tm_min;
+  dt->second = my_time->tm_sec;
+  dt->microseconds = tv->tv_usec;
+  dt->timezone_minutes = 0; /* always Zulu time */
+
+  return 0;
+}
+
+
+/**
+ * rasqal_xsd_datetime_set_from_unixtime:
+ * @dt: date time
+ * @clock: unix time in seconds
+ * 
+ * Set an XSD dateTime from unixtime seconds
+ * 
+ * Returns: non-0 on failure
+ **/
+int
+rasqal_xsd_datetime_set_from_unixtime(rasqal_xsd_datetime* dt,
+                                      time_t secs)
+{
+  struct timeval tv;
+
+  if(!dt)
+    return 1;
+  
+  tv.tv_sec = secs;
+  tv.tv_usec = 0;
+
+  return rasqal_xsd_datetime_set_from_timeval(dt, &tv);
+}
+
+
+/**
+ * rasqal_xsd_datetime_get_as_unixtime:
+ * @dt: datetime
+ * 
+ * Get a datetime as unix seconds
+ *
+ * Returns: unix seconds or 0 if @dt is NULL
+ **/
+time_t
+rasqal_xsd_datetime_get_as_unixtime(rasqal_xsd_datetime* dt)
+{
+  struct tm time_buf;
+
+  if(!dt)
+    return 0;
+  
+  memset(&time_buf, '\0', sizeof(time_buf));
+
+  time_buf.tm_year = dt->year - TM_YEAR_ORIGIN;
+  time_buf.tm_mon  = dt->month - TM_MONTH_ORIGIN;
+  time_buf.tm_mday = dt->day;
+  time_buf.tm_hour = dt->hour;
+  time_buf.tm_min  = dt->minute;
+  time_buf.tm_sec  = dt->second;
+  time_buf.tm_wday = 0;
+  time_buf.tm_yday = 0;
+  time_buf.tm_isdst = -1;
+
+#ifdef HAVE_TM_GMTOFF
+  if(dt->timezone_minutes == RASQAL_XSD_DATETIME_NO_TZ)
+    time_buf.tm_gmtoff = 0;
+  else
+    time_buf.tm_gmtoff = dt->timezone_minutes * 60;
+#endif
+
+  return timegm(&time_buf);
+}
+
+
+/**
+ * rasqal_xsd_datetime_get_as_timeval:
+ * @dt: datetime
+ * 
+ * Get a datetime as struct timeval
+ *
+ * The returned timeval must be freed by the caller such as using
+ * rasqal_free_memory().
+ *
+ * Returns: pointer to a new timeval structure or NULL on failure
+ **/
+struct timeval*
+rasqal_xsd_datetime_get_as_timeval(rasqal_xsd_datetime *dt)
+{
+  struct timeval *tv;
+  
+  if(!dt)
+    return NULL;
+  
+  tv = RASQAL_CALLOC(timeval, 1, sizeof(*tv));
+  if(!tv)
+    return NULL;
+  
+  tv->tv_sec = rasqal_xsd_datetime_get_as_unixtime(dt);
+  tv->tv_usec = dt->microseconds;
+  
+  return tv;
+}
+
+
+/**
+ * rasqal_xsd_datetime_get_timezone_as_counted_string:
+ * @dt: datetime
+ * @len_p: pointer to store returned string length
+ * 
+ * Get the timezone of a datetime as a duration format string with optional length count
+ *
+ * The returned string is owned by the caller and must be freed
+ * by rasqal_free_memory().
+ *
+ * Returns: pointer to a new string or NULL on failure
+ **/
+char*
+rasqal_xsd_datetime_get_timezone_as_counted_string(rasqal_xsd_datetime *dt,
+                                                   size_t *len_p)
+{
+  /* timezone duration as implemented here is a signed integer number
+   * of seconds like +- 14 hours:minutes (no days or larger units, no
+   * seconds or smaller).
+   *
+   * When written in the canonical format, a restricted
+   * xsd:dayTimeDuration format, it is constraint to a format like -?PThhmm
+   *
+   * For example: -PT14H59M PT14H59M and PT0S for a zero timezone offset
+   */
+#define TZ_STR_SIZE 10
+  char* tz_str;
+  char* p;
+  int minutes;
+  unsigned int hours;
+  
+  if(!dt)
+    return NULL;
+  
+  tz_str = RASQAL_MALLOC(cstring, TZ_STR_SIZE + 1);
+  if(!tz_str)
+    return NULL;
+  
+  p = tz_str;
+
+  minutes = dt->timezone_minutes;
+  if(minutes < 0) {
+    *p++ = '-';
+    minutes = -minutes;
+  }
+
+  *p++ = 'P';
+  *p++ = 'T';
+    
+  hours = (minutes / 60);
+  if(hours) {
+#if 1
+    if(hours > 9) {
+      *p++ = '0' + (hours / 10);
+      hours %= 10;
+    }
+    *p++ = '0' + hours;
+    *p++ = 'H';
+#else
+    p += sprintf(p, "%dH", hours);
+#endif
+    minutes -= hours * 60;
+  }
+  
+  if(minutes) {
+#if 1
+    if(minutes > 9) {
+      *p++ = '0' + (minutes / 10);
+      minutes %= 10;
+    }
+    *p++ = '0' + minutes;
+    *p++ = 'M';
+#else
+    p += sprintf(p, "%dM", minutes);
+#endif
+  }
+
+  if(!dt->timezone_minutes) {
+    *p++ = '0';
+    *p++ = 'S';
+  }
+  
+  *p = '\0';
+
+  if(len_p)
+    *len_p = p - tz_str;
+  
+  return tz_str;
+}
+
+
+/**
+ * rasqal_xsd_datetime_get_tz_as_counted_string:
+ * @dt: datetime
+ * @len_p: pointer to store returned string length
+ * 
+ * Get the timezone of a datetime as a timezone string
+ *
+ * The returned string is owned by the caller and must be freed
+ * by rasqal_free_memory().
+ *
+ * Returns: pointer to a new string or NULL on failure
+ **/
+char*
+rasqal_xsd_datetime_get_tz_as_counted_string(rasqal_xsd_datetime* dt,
+                                             size_t *len_p)
+{
+  char* s;
+  
+  s = (char*)RASQAL_MALLOC(cstring, TIMEZONE_BUFFER_LEN);
+  if(!s)
+    return NULL;
+
+  if(rasqal_xsd_datetime_timezone_format(dt, s, TIMEZONE_BUFFER_LEN))
+    goto failed;
+
+  if(len_p)
+    *len_p = (TIMEZONE_BUFFER_LEN - 1); /* \0 is not in the length */
+  
+  return s;
+
+  failed:
+  RASQAL_FREE(cstring, s);
+  return NULL;
+}
+
+
+
 #ifdef STANDALONE
 #include <stdio.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 
 int main(int argc, char *argv[]);
 
@@ -780,9 +1363,9 @@ int main(int argc, char *argv[]);
 
 static int test_datetime_parser_tostring(const char *in_str, const char *out_expected)
 {
-  unsigned char const *s;
+  char const *s;
   int r = 1;
-  s = rasqal_xsd_datetime_string_to_canonical((const unsigned char *)in_str);
+  s = rasqal_xsd_datetime_string_to_canonical((const char*)in_str);
   if(s) {
     r = strcmp((char*)s, out_expected);
     if(r)
@@ -796,9 +1379,9 @@ static int test_datetime_parser_tostring(const char *in_str, const char *out_exp
 
 static int test_date_parser_tostring(const char *in_str, const char *out_expected)
 {
-  unsigned char const *s;
+  char const *s;
   int r = 1;
-  s = rasqal_xsd_date_string_to_canonical((const unsigned char *)in_str);
+  s = rasqal_xsd_date_string_to_canonical((const char*)in_str);
   if(s) {
     r = strcmp((char*)s, out_expected);
     if(r)
@@ -847,7 +1430,7 @@ main(int argc, char *argv[])
      rasqal_xsd_datetime_string_to_canonical */
   
   #define PARSE_AND_NORMALIZE_DATETIME(_s,_d) \
-    rasqal_xsd_datetime_parse_and_normalize((const unsigned char*)_s, _d)
+    rasqal_xsd_datetime_parse_and_normalize((const char*)_s, _d)
   
   /* generic */
 
@@ -951,10 +1534,10 @@ main(int argc, char *argv[])
   MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.01Z", "2006-05-18T18:36:03.01Z") == 0);
   MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.10Z", "2006-05-18T18:36:03.1Z") == 0);
   MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.010Z", "2006-05-18T18:36:03.01Z") == 0);
-  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1234Z", "2006-05-18T18:36:03.123Z") == 0);
-  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1234", "2006-05-18T18:36:03.123") == 0);
-  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1239Z", "2006-05-18T18:36:03.123Z") == 0);
-  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1239", "2006-05-18T18:36:03.123") == 0);
+  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1234Z", "2006-05-18T18:36:03.1234Z") == 0);
+  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1234", "2006-05-18T18:36:03.1234") == 0);
+  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1239Z", "2006-05-18T18:36:03.1239Z") == 0);
+  MYASSERT(test_datetime_parser_tostring("2006-05-18T18:36:03.1239", "2006-05-18T18:36:03.1239") == 0);
 
   /* timezones + normalization */
 
@@ -987,7 +1570,7 @@ main(int argc, char *argv[])
   /* DATE */
 
   #define PARSE_AND_NORMALIZE_DATE(_s,_d) \
-    rasqal_xsd_date_parse_and_normalize((const unsigned char*)_s, _d)
+    rasqal_xsd_date_parse_and_normalize((const char*)_s, _d)
   
   /* generic */
 
@@ -1065,6 +1648,36 @@ main(int argc, char *argv[])
   MYASSERT(test_date_parser_tostring("2004-12-31-11:59", "2004-12-31") == 0);
   MYASSERT(test_date_parser_tostring("2005-01-01+11:59", "2005-01-01") == 0);
 
+
+  if(1) {
+    struct timeval my_tv;
+    time_t secs;
+    time_t new_secs;
+    struct timeval* new_tv;
+
+    /* 2010-12-14T06:22:36.868099Z or 2010-12-13T22:22:36.868099+0800 
+     * when I was testing this
+     */
+    my_tv.tv_sec = 1292307756;
+    my_tv.tv_usec = 868099;
+
+    secs = my_tv.tv_sec;
+    
+    MYASSERT(rasqal_xsd_datetime_set_from_timeval(&dt, &my_tv) == 0);
+
+    MYASSERT((new_tv = rasqal_xsd_datetime_get_as_timeval(&dt)));
+    MYASSERT(new_tv->tv_sec == my_tv.tv_sec);
+    MYASSERT(new_tv->tv_usec == my_tv.tv_usec);
+
+    RASQAL_FREE(timeval, new_tv);
+    
+    MYASSERT(rasqal_xsd_datetime_set_from_unixtime(&dt, secs) == 0);
+    
+    MYASSERT((new_secs = rasqal_xsd_datetime_get_as_unixtime(&dt)));
+    MYASSERT(new_secs == secs);
+  }
+  
+
   return 0;
 }
 
