@@ -34,6 +34,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_MATH_H
+#include <math.h>
+#endif
 #include <stdarg.h>
 
 #include "rasqal.h"
@@ -82,6 +85,7 @@ static void rasqal_xsd_decimal_clear(rasqal_xsd_decimal* dec);
 /* No implementation - use double. */
 #define RASQAL_DECIMAL_RAW double
 #define RASQAL_DECIMAL_ROUNDING int
+#include <float.h>
 
 #endif
 #endif
@@ -139,28 +143,34 @@ rasqal_xsd_decimal_init(rasqal_xsd_decimal* dec)
   /* XSD wants min of 18 decimal (base 10) digits 
    * http://www.w3.org/TR/2004/REC-xmlschema-2-20041028/#decimal
    */
-  dec->precision_digits= 32;
+#ifdef RASQAL_DECIMAL_NONE
+  /* double has a restricted range */
+  dec->precision_digits = DBL_DIG;
+#else
+  dec->precision_digits = 18;
+#endif
+
   /* over-estimate bits since log(10)/log(2) = 3.32192809488736234789 < 4 */
-  dec->precision_bits= dec->precision_digits*4;
+  dec->precision_bits = dec->precision_digits*4;
   
 #ifdef RASQAL_DECIMAL_C99
-  dec->raw= 0DD;
+  dec->raw = 0DD;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
   mpfr_init2(dec->raw, dec->precision_bits);
 
   /* GMP_RNDD, GMP_RNDU, GMP_RNDN, GMP_RNDZ */
-  dec->rounding=mpfr_get_default_rounding_mode();
+  dec->rounding = mpfr_get_default_rounding_mode();
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_init2(dec->raw, dec->precision_bits);
 #endif
 #ifdef RASQAL_DECIMAL_NONE
-  dec->raw= 0e0;
+  dec->raw = 0e0;
 #endif
 
-  dec->string=NULL;
-  dec->string_len=0;
+  dec->string = NULL;
+  dec->string_len = 0;
 }
 
 
@@ -213,21 +223,26 @@ rasqal_xsd_decimal_set_string(rasqal_xsd_decimal* dec, const char* string)
 
   rasqal_xsd_decimal_clear_string(dec);
 
-  len=strlen(string);
-  dec->string=(char*)RASQAL_MALLOC(cstring, len+1);
+  len = strlen(string);
+  dec->string = (char*)RASQAL_MALLOC(cstring, len+1);
   if(!dec->string)
     return 1;
+
   memcpy(dec->string, string, len + 1);
-  dec->string_len=len;
+  dec->string_len = len;
   
 #if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
-  dec->raw=strtod(string, NULL);
+  dec->raw = strtod(string, NULL);
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_set_str(dec->raw, string, 10, dec->rounding);
+  if(*string == '+')
+    string++;
+  rc = mpfr_set_str(dec->raw, string, 10, dec->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
-  rc=mpf_set_str(dec->raw, string, 10);
+  if(*string == '+')
+    string++;
+  rc = mpf_set_str(dec->raw, string, 10);
 #endif
 
   return rc;
@@ -254,7 +269,7 @@ rasqal_xsd_decimal_set_long(rasqal_xsd_decimal* dec, long l)
   dec->raw=l;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_set_si(dec->raw, l, dec->rounding);
+  rc = mpfr_set_si(dec->raw, l, dec->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_set_si(dec->raw, l);
@@ -309,7 +324,7 @@ rasqal_xsd_decimal_get_double(rasqal_xsd_decimal* dec)
   result=(double)dec->raw;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  result=mpfr_get_d(dec->raw, dec->rounding);
+  result = mpfr_get_d(dec->raw, dec->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   result=mpf_get_d(dec->raw);
@@ -319,6 +334,24 @@ rasqal_xsd_decimal_get_double(rasqal_xsd_decimal* dec)
 }
 
 
+#ifdef RASQAL_DECIMAL_C99
+#define DECIMAL_SNPRINTF snprintf
+#define DECIMAL_FMT "%DDf"
+#endif
+#ifdef RASQAL_DECIMAL_MPFR
+#define DECIMAL_SNPRINTF mpfr_snprintf
+#define DECIMAL_FMT "%%.%dRf"
+#endif
+#ifdef RASQAL_DECIMAL_GMP
+#define DECIMAL_SNPRINTF gmp_snprintf
+#define DECIMAL_FMT "%%.%dFf"
+#endif
+#ifdef RASQAL_DECIMAL_NONE
+#define DECIMAL_SNPRINTF snprintf
+#define DECIMAL_FMT "%%.%df"
+#endif
+
+
 /**
  * rasqal_xsd_decimal_as_string:
  * @dec: XSD Decimal
@@ -335,165 +368,63 @@ rasqal_xsd_decimal_as_string(rasqal_xsd_decimal* dec)
 {
   char *s=NULL;
   size_t len=0;
-#if defined(RASQAL_DECIMAL_MPFR) || defined(RASQAL_DECIMAL_GMP)
-  mp_exp_t expo;
-  char *mpf_s;
+#ifdef RASQAL_DECIMAL_C99
+#else
+  char fmt[16];
+  size_t e_index = 0;
+  size_t trailing_zero_start = 0;
 #endif
   
   if(dec->string)
     return dec->string;
   
 #ifdef RASQAL_DECIMAL_C99
-  len=dec->precision_digits;
-  s=RASQAL_MALLOC(cstring, len+1);
+  len = dec->precision_digits;
+  s = RASQAL_MALLOC(cstring, len + 1);
   if(!s)
     return NULL;
+
   /* NOTE: Never seen a sprintf that supports _Decimal yet */
   snprintf(s, len, "%DDf", dec->raw);
-  len=strlen(s);
-#endif
-#ifdef RASQAL_DECIMAL_MPFR
-  mpf_s=mpfr_get_str(NULL, &expo, 10, 0, dec->raw, dec->rounding);
-  if(mpf_s) {
-    size_t from_len=strlen(mpf_s);
-    char *from_p=mpf_s;
-    size_t to_len;
-
-    /* 7=strlen("0.0e0")+1 for sign */
-    to_len=!from_len ? 6 : (from_len*2);
-
-    s=(char*)RASQAL_MALLOC(cstring, to_len);
-    if(!s) {
-      mpfr_free_str((char*)mpf_s);
-      return NULL;
-    }
-    /* first digit of mantissa */
-    if(!*from_p || *from_p == '0') {
-      len=5;
-      memcpy(s, "0.0e0", len + 1);
-    } else {
-      char* to_p=s;
-      int n;
-      
-      if(*from_p == '-') {
-        *to_p++ = *from_p++;
-        from_len--;
-      }
-
-      *to_p++ = *from_p++;
-      from_len--;
-      *to_p++ = '.';
-      /* rest of mantissa */
-      /* remove trailing 0s */
-      while(from_len > 1 && from_p[from_len-1]=='0')
-        from_len--;
-      memcpy(to_p, from_p, from_len);
-      to_p += from_len;
-      /* exp */
-      n=sprintf(to_p, "e%ld", expo-1);
-      len=to_p+n-s;
-    }
-    mpfr_free_str((char*)mpf_s);
-  }
-#endif
-#ifdef RASQAL_DECIMAL_GMP
-  mpf_s=mpf_get_str(NULL, &expo, 10, 0, dec->raw);
-  if(mpf_s) {
-    size_t from_len=strlen(mpf_s);
-    char *from_p=mpf_s;
-    size_t to_len;
-
-    /* 7=strlen("0.0e0")+1 for sign */
-    to_len=!from_len ? 6 : (from_len*2);
-
-    s=RASQAL_MALLOC(cstring, to_len+1);
-    if(!s) {
-      free(mpf_s);
-      return NULL;
-    }
-    /* first digit of mantissa */
-    if(!*from_p || *from_p == '0') {
-      len=5;
-      memcpy(s, "0.0e0", len + 1);
-    } else {
-      char *to_p=s;
-      int n;
-      
-      if(*from_p == '-') {
-        *to_p++ = *from_p++;
-        from_len--;
-      }
-
-      *to_p++ = *from_p++;
-      from_len--;
-      *to_p++ = '.';
-      /* rest of mantissa */
-      memcpy(to_p, from_p, from_len);
-      to_p+= from_len;
-      /* exp */
-      n=sprintf(to_p, "e%ld", expo-1);
-      len=to_p+n-s;
-    }
-    free(mpf_s);
-  }
-#endif
-#ifdef RASQAL_DECIMAL_NONE
-  {
-    char *p, *p2;
-    char fmt[16];
-    /* construct a format string */
-    snprintf(fmt, sizeof(fmt), "%%.%de", dec->precision_digits);
+  len = strlen(s);
+#else
+  /* construct a format string */
+  snprintf(fmt, sizeof(fmt), DECIMAL_FMT, dec->precision_digits);
     
-    /* snprintf with no buffer to get buffer length */
-    len=snprintf(NULL, 0, fmt, dec->raw)+1;
-    s=(char*)RASQAL_MALLOC(cstring, len);
-    if(!s)
-      return NULL;
+  /* decimal snprintf with no buffer to get buffer length */
+  len = DECIMAL_SNPRINTF(NULL, 0, fmt, dec->raw);
 
-    /* format with snprintf */
-    snprintf(s, len, fmt, dec->raw);
+  s = (char*)RASQAL_MALLOC(cstring, len + 1);
+  if(!s)
+    return NULL;
+  
+  /* format into allocated buffer */
+  DECIMAL_SNPRINTF(s, len, fmt, dec->raw);
 
-    /* "1.2000e+02"
-     * - remove zeros before 'e', leave one for ".0"
-     * - remove '+' after 'e' (leave '-')
-     * - remove leading zero after 'e' */
+  len = strlen(s);
 
-    /* find 'e' */
-    p=strchr(s, 'e');
-
-    /* move 'e' and everything that follows on top of the trailing zeros */
-    for(p2=p; *--p2 == '0'; ) ;
-    if(*p2=='.') p2++; /* leave ".0" */
-    p2++;
-    
-    /* move string tail if required */
-    if(p != p2)
-      while((*p2++=*p++)) ;
-
-    /* adjust p to the first char after 'e' */
-    p=strchr(s, 'e');
-    p++;
-
-    /* leave '-' but remove '+' */
-    if(*p=='-') {
-      p++;
-      p2=p;
-    } else {
-      p2=p+1;
-    }
-    /* skip leading zero afer "e[+-]" */
-    if(*p2=='0') p2++;
-
-    /* move string tail if required */
-    if(p != p2)
-      while((*p++=*p2++)) ;
-
-    len=strlen(s);
+  /* find trailing zeros */
+  for(e_index = len - 1;
+      e_index > 0 && s[e_index] == '0';
+      e_index--) {
+    trailing_zero_start = e_index;
   }
+
+  /* Stop string where the trailing zeros started */
+  if(trailing_zero_start) {
+    s[trailing_zero_start] = '\0';
+    len = trailing_zero_start;
+  }
+  
+  if(s[len - 1] == '.') {
+    s[len-- - 1] = '\0';
+  }
+  
 #endif
 
-  dec->string=s;
-  dec->string_len=len;
+  dec->string = s;
+  dec->string_len = len;
+
   return s;
 }
 
@@ -523,10 +454,10 @@ rasqal_xsd_decimal_as_counted_string(rasqal_xsd_decimal* dec, size_t* len_p)
 
 /**
  * rasqal_xsd_decimal_print:
- * @dec: XSD Decimal
- * @stream: FILE* stream
+ * @dec: XSD Decimal object
+ * @stream: FILE* handle to print to
  * 
- * Write an XSD Decimal to a FILE* stream
+ * Print an XSD Decimal to a stream
  * 
  * Return value: non-0 on failure
  **/
@@ -567,7 +498,7 @@ rasqal_xsd_decimal_add(rasqal_xsd_decimal* result,
   result->raw = a->raw + b->raw;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_add(result->raw, a->raw, b->raw, result->rounding);
+  mpfr_add(result->raw, a->raw, b->raw, result->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_add(result->raw, a->raw, b->raw);
@@ -599,7 +530,7 @@ rasqal_xsd_decimal_subtract(rasqal_xsd_decimal* result,
   result->raw = a->raw - b->raw;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_sub(result->raw, a->raw, b->raw, result->rounding);
+  mpfr_sub(result->raw, a->raw, b->raw, result->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_sub(result->raw, a->raw, b->raw);
@@ -631,7 +562,7 @@ rasqal_xsd_decimal_multiply(rasqal_xsd_decimal* result,
   result->raw = a->raw * b->raw;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_mul(result->raw, a->raw, b->raw, result->rounding);
+  mpfr_mul(result->raw, a->raw, b->raw, result->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_mul(result->raw, a->raw, b->raw);
@@ -696,7 +627,7 @@ rasqal_xsd_decimal_divide(rasqal_xsd_decimal* result,
   result->raw = a->raw / b->raw;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_div(result->raw, a->raw, b->raw, result->rounding);
+  mpfr_div(result->raw, a->raw, b->raw, result->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_div(result->raw, a->raw, b->raw);
@@ -726,7 +657,7 @@ rasqal_xsd_decimal_negate(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
   result->raw = -a->raw;
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_neg(result->raw, a->raw, result->rounding);
+  mpfr_neg(result->raw, a->raw, result->rounding);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   mpf_neg(result->raw, a->raw);
@@ -736,6 +667,126 @@ rasqal_xsd_decimal_negate(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
 }
 
 
+/**
+ * rasqal_xsd_decimal_abs:
+ * @result: result variable
+ * @a: argment decimal
+ * 
+ * Return the absolute value of an XSD Decimal
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_xsd_decimal_abs(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
+{
+  int rc = 0;
+  
+  rasqal_xsd_decimal_clear_string(result);
+  
+#if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
+  result->raw = abs(a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_MPFR
+  mpfr_abs(result->raw, a->raw, result->rounding);
+#endif
+#ifdef RASQAL_DECIMAL_GMP
+  mpf_abs(result->raw, a->raw);
+#endif
+
+  return rc;
+}
+
+
+/**
+ * rasqal_xsd_decimal_round:
+ * @result: result variable
+ * @a: argment decimal
+ * 
+ * Return the number with no fractional part closes to argument for an XSD Decimal
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_xsd_decimal_round(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
+{
+  int rc = 0;
+  
+  rasqal_xsd_decimal_clear_string(result);
+  
+#if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
+  result->raw = round(a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_MPFR
+  mpfr_round(result->raw, a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_GMP
+  mpf_round(result->raw, a->raw);
+#endif
+
+  return rc;
+}
+
+
+/**
+ * rasqal_xsd_decimal_ceil:
+ * @result: result variable
+ * @a: argment decimal
+ * 
+ * Return the number with no fractional part closes to argument for an XSD Decimal
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_xsd_decimal_ceil(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
+{
+  int rc = 0;
+  
+  rasqal_xsd_decimal_clear_string(result);
+  
+#if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
+  result->raw = ceil(a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_MPFR
+  mpfr_ceil(result->raw, a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_GMP
+  mpf_ceil(result->raw, a->raw);
+#endif
+
+  return rc;
+}
+
+
+/**
+ * rasqal_xsd_decimal_floor:
+ * @result: result variable
+ * @a: argment decimal
+ * 
+ * Return the number with no fractional part closes to argument for an XSD Decimal
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_xsd_decimal_floor(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
+{
+  int rc = 0;
+  
+  rasqal_xsd_decimal_clear_string(result);
+  
+#if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
+  result->raw = floor(a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_MPFR
+  mpfr_floor(result->raw, a->raw);
+#endif
+#ifdef RASQAL_DECIMAL_GMP
+  mpf_floor(result->raw, a->raw);
+#endif
+
+  return rc;
+}
+
+
 /**
  * rasqal_xsd_decimal_compare:
  * @a: first XSD decimal
@@ -763,7 +814,7 @@ rasqal_xsd_decimal_compare(rasqal_xsd_decimal* a, rasqal_xsd_decimal* b)
   /* else rc is 0 set above */
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
-  rc=mpfr_cmp(a->raw, b->raw);
+  rc = mpfr_cmp(a->raw, b->raw);
 #endif
 #ifdef RASQAL_DECIMAL_GMP
   rc=mpf_cmp(a->raw, b->raw);
@@ -790,7 +841,7 @@ rasqal_xsd_decimal_equals(rasqal_xsd_decimal* a, rasqal_xsd_decimal* b)
 #if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
   rc= (b->raw == a->raw);
 #elif defined(RASQAL_DECIMAL_MPFR)
-  rc=mpfr_equal_p(a->raw, b->raw);
+  rc = mpfr_equal_p(a->raw, b->raw);
 #elif defined(RASQAL_DECIMAL_GMP)
   /* NOTE: Not using mpf_eq() but could do, with sufficient bits */
   rc=!mpf_cmp(a->raw, b->raw);
@@ -818,13 +869,13 @@ main(int argc, char *argv[]) {
   double result_d;
   char *result_s;
   int result_i;
-  const long a_long=1234567890L;
-  const double a_double=1234567890e0;
-  const char* b_string="123456789012345678e0";
-  const char* expected_a_plus_b="1.23456790246913568e17";
-  const char* expected_a_plus_b_minus_b="1.23456789e9";
-  const char* expected_a_plus_b_minus_b_minus_a="0.0e0";
-  const char* expected_negative_b="-1.23456789012345678e17";
+  const long a_long = 1234567890L;
+  const double a_double = 1234567890e0;
+  const char* b_string = "123456789012345678";
+  const char* expected_a_plus_b = "123456790246913568";
+  const char* expected_a_plus_b_minus_b = "1234567890";
+  const char* expected_a_plus_b_minus_b_minus_a = "0";
+  const char* expected_negative_b = "-123456789012345678";
   int expected_a_compare_b= -1;
   int expected_a_equals_b= 0;
   rasqal_world *world;
