@@ -217,7 +217,8 @@ rasqal_free_gp_data(rasqal_engine_gp_data* gp_data)
 {
   rasqal_graph_pattern* gp;
 
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(gp_data, rasqal_engine_gp_data);
+  if(!gp_data)
+    return;
   
   gp = gp_data->gp;
   if(gp_data->triple_meta) {
@@ -624,11 +625,7 @@ rasqal_engine_remove_filter_graph_patterns(rasqal_query* query,
   }
   
   
-#ifdef HAVE_RAPTOR2_API
   seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
-#else
-  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-#endif
   if(!seq) {
     RASQAL_DEBUG1("Cannot create new gp sequence\n");
     *modified_p = -1;
@@ -736,18 +733,17 @@ rasqal_engine_check_constraint(rasqal_engine_execution_data* execution_data,
   fputc('\n', DEBUG_FH);
 #endif
     
-  result = rasqal_expression_evaluate(query->world, &query->locator,
-                                      gp->filter_expression, 
-                                      query->compare_flags);
+  result = rasqal_expression_evaluate2(gp->filter_expression, 
+                                       query->eval_context, &error);
 #ifdef RASQAL_DEBUG
   RASQAL_DEBUG1("filter expression result: ");
-  if(!result)
+  if(error)
     fputs("type error", DEBUG_FH);
   else
     rasqal_literal_print(result, DEBUG_FH);
   fputc('\n', DEBUG_FH);
 #endif
-  if(!result) {
+  if(error) {
     bresult = 0;
     step = STEP_ERROR;
   } else {
@@ -1428,24 +1424,23 @@ rasqal_engine_make_rowsource(rasqal_query* query,
   con->need_store_results = need_store_results;
 
   if(con->need_store_results) {
+    raptor_sequence *order_seq;
+    order_seq = rasqal_query_get_order_conditions_sequence(query);
+    
     /* make a row:NULL map in order to sort or do distinct */
     con->map = rasqal_engine_new_rowsort_map(query->distinct,
                                              query->compare_flags,
-                                             query->order_conditions_sequence);
+                                             order_seq);
     if(!con->map) {
       rasqal_rowsource_engine_finish(NULL, con);
       return NULL;
     }
 
-    if(query->order_conditions_sequence)
-      con->order_size = raptor_sequence_size(query->order_conditions_sequence);
+    if(order_seq)
+      con->order_size = raptor_sequence_size(order_seq);
   }
   
-#ifdef HAVE_RAPTOR2_API
   con->seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
-#else
-  con->seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
-#endif
   if(!con->seq) {
     rasqal_rowsource_engine_finish(NULL, con);
     return NULL;
@@ -1484,6 +1479,7 @@ rasqal_query_engine_1_get_row(void* ex_data, rasqal_engine_error *error_p)
 
   while(1) {
     int rc;
+    int check;
     
     /* rc<0 error rc=0 end of results,  rc>0 got a result */
     rc = rasqal_engine_get_next_result(execution_data);
@@ -1501,17 +1497,20 @@ rasqal_query_engine_1_get_row(void* ex_data, rasqal_engine_error *error_p)
     /* otherwise is >0 match */
     execution_data->result_count++;
 
+    check = rasqal_query_check_limit_offset(execution_data->query,
+                                            execution_data->result_count);
+    
     /* finished if beyond result range */
-    if(rasqal_query_results_check_limit_offset(query_results) > 0) {
+    if(check > 0) {
       execution_data->result_count--;
       break;
     }
 
     /* continue if before start of result range */
-    if(rasqal_query_results_check_limit_offset(query_results) < 0)
+    if(check < 0)
       continue;
 
-    /* else got result or finished */
+    /* else got result */
     break;
 
   } /* while */
@@ -1806,11 +1805,7 @@ rasqal_query_engine_1_execute_init(void* ex_data,
     }
   }
 
-#ifdef HAVE_RAPTOR2_API
   execution_data->seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_gp_data, NULL);
-#else
-  execution_data->seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_gp_data, NULL);
-#endif
   if(!execution_data->seq) {
     *error_p = RASQAL_ENGINE_FAILED;
     return 1;
