@@ -44,8 +44,8 @@
 #include <rasqal_internal.h>
 
 
-static int rasqal_query_results_write_sparql_xml(raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
-static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri);
+static int rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter, raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
+static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter, rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri);
 
 
 #if RASQAL_DEBUG > 1
@@ -79,16 +79,13 @@ static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_wo
  * Return value: non-0 on failure
  **/
 static int
-rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
+rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
+                                      raptor_iostream *iostr,
                                       rasqal_query_results* results,
                                       raptor_uri *base_uri)
 {
   int rc=1;
-  rasqal_query* query = rasqal_query_results_get_query(results);
-#ifndef HAVE_RAPTOR2_API
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-#endif
+  rasqal_world* world = rasqal_query_results_get_world(results);
   raptor_xml_writer* xml_writer=NULL;
   raptor_namespace *res_ns=NULL;
   raptor_namespace_stack *nstack=NULL;
@@ -103,35 +100,20 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
   if(!rasqal_query_results_is_bindings(results) &&
      !rasqal_query_results_is_boolean(results)) {
-    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                            &query->locator,
+    rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
+                            NULL,
                             "Can only write XML format v3 for variable binding and boolean results");
     return 1;
   }
   
 
-#ifdef HAVE_RAPTOR2_API  
-  nstack = raptor_new_namespaces(query->world->raptor_world_ptr, 1);
-#else
-  raptor_uri_get_handler(&uri_handler, &uri_context);
-  nstack = raptor_new_namespaces(uri_handler, uri_context,
-                                 (raptor_simple_message_handler)rasqal_query_simple_error, query,
-                                 1);
-#endif
+  nstack = raptor_new_namespaces(world->raptor_world_ptr, 1);
   if(!nstack)
     return 1;
 
-#ifdef HAVE_RAPTOR2_API
-  xml_writer = raptor_new_xml_writer(query->world->raptor_world_ptr,
+  xml_writer = raptor_new_xml_writer(world->raptor_world_ptr,
                                      nstack,
                                      iostr);
-#else
-  xml_writer = raptor_new_xml_writer(nstack,
-                                     uri_handler, uri_context,
-                                     iostr,
-                                     (raptor_simple_message_handler)rasqal_query_simple_error, query,
-                                     1);
-#endif
   if(!xml_writer)
     goto tidy;
 
@@ -179,16 +161,10 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef HAVE_RAPTOR2_API
-      attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+      attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
                                                             (const unsigned char*)"name",
                                                             (const unsigned char*)name); /* attribute value */
-#else
-      attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
-                                                            (const unsigned char*)"name",
-                                                            (const unsigned char*)name); /* attribute value */
-#endif
       if(!attrs[0]) {
         raptor_free_memory((void*)attrs);
         goto tidy;
@@ -279,17 +255,11 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef HAVE_RAPTOR2_API
-      attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+      attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
                                                             (const unsigned char*)"name",
                                                             name);
 
-#else
-      attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
-                                                            (const unsigned char*)"name",
-                                                            name);
-#endif
       if(!attrs[0]) {
         raptor_free_memory((void*)attrs);
         goto tidy;
@@ -336,6 +306,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
           break;
 
         case RASQAL_LITERAL_STRING:
+        case RASQAL_LITERAL_UDT:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
                                                                     (const unsigned char*)"literal",
                                                                     NULL, base_uri);
@@ -350,22 +321,12 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
             if(l->language)
               attrs[0]=raptor_new_qname(nstack,
                                         (const unsigned char*)"xml:lang",
-                                        (const unsigned char*)l->language
-#ifndef HAVE_RAPTOR2_API
-                                        , (raptor_simple_message_handler)rasqal_query_simple_error, query
-#endif
-                                        );
+                                        (const unsigned char*)l->language);
             else
-#ifdef HAVE_RAPTOR2_API
-              attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+              attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                                     res_ns,
                                                                     (const unsigned char*)"datatype",
                                                                     (const unsigned char*)raptor_uri_as_string(l->datatype));
-#else
-              attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
-                                                                    (const unsigned char*)"datatype",
-                                                                    (const unsigned char*)raptor_uri_as_string(l->datatype));
-#endif
             if(!attrs[0]) {
               raptor_free_memory((void*)attrs);
               goto tidy;
@@ -395,13 +356,12 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
         case RASQAL_LITERAL_VARIABLE:
         case RASQAL_LITERAL_DECIMAL:
         case RASQAL_LITERAL_DATETIME:
-        case RASQAL_LITERAL_UDT:
         case RASQAL_LITERAL_INTEGER_SUBTYPE:
 
         case RASQAL_LITERAL_UNKNOWN:
         default:
-          rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                                  &query->locator,
+          rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
+                                  NULL,
                                   "Cannot turn literal type %d into XML", 
                                   l->type);
           goto tidy;
@@ -515,9 +475,6 @@ typedef struct
   raptor_sax2* sax2;
   raptor_locator locator;
   int depth; /* element depth */
-#ifndef HAVE_RAPTOR2_API
-  raptor_error_handlers error_handlers; /* SAX2 error handler */
-#endif
 
   /* SPARQL XML Results parsing */
   rasqal_sparql_xml_read_state state; /* state */
@@ -624,7 +581,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
 
   switch(state) {
     case STATE_variable:
-      if(1) {
+      if(con->name) {
         unsigned char* var_name;
         rasqal_variable *v;
         
@@ -641,7 +598,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
       
     case STATE_result:
       if(1) {
-        con->row=rasqal_new_row(con->rowsource);
+        con->row = rasqal_new_row(con->rowsource);
         RASQAL_DEBUG2("Made new row %d\n", con->offset);
         con->offset++;
       }
@@ -748,8 +705,9 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         if(con->datatype)
           datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
         if(con->language) {
-          language_str=(char*)RASQAL_MALLOC(cstring, strlen(con->language)+1);
-          strcpy(language_str, con->language);
+          size_t language_len = strlen(con->language);
+          language_str=(char*)RASQAL_MALLOC(cstring, language_len + 1);
+          memcpy(language_str, con->language, language_len + 1);
         }
         l = rasqal_new_string_literal_node(con->world, lvalue, language_str,
                                            datatype_uri);
@@ -954,7 +912,8 @@ static const rasqal_rowsource_handler rasqal_rowsource_sparql_xml_handler={
  * Return value: a new rasqal_rowsource or NULL on failure
  **/
 static rasqal_rowsource*
-rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
+rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter,
+                                              rasqal_world *world,
                                               rasqal_variables_table* vars_table,
                                               raptor_iostream *iostr,
                                               raptor_uri *base_uri)
@@ -971,14 +930,7 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
 
   con->locator.uri=base_uri;
 
-#ifdef HAVE_RAPTOR2_API
   con->sax2 = raptor_new_sax2(world->raptor_world_ptr, &con->locator, con);
-#else
-  con->error_handlers.locator = &con->locator;
-  raptor_error_handlers_init(&con->error_handlers);
-
-  con->sax2 = raptor_new_sax2(con, &con->error_handlers);
-#endif
   if(!con->sax2)
     return NULL;
   
@@ -990,11 +942,7 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
   raptor_sax2_set_end_element_handler(con->sax2,
                                       rasqal_sparql_xml_sax2_end_element_handler);
 
-#ifdef HAVE_RAPTOR2_API
   con->results_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
-#else
-  con->results_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
-#endif
 
   con->vars_table = rasqal_new_variables_table_from_variables_table(vars_table);
   
@@ -1007,35 +955,60 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
 
 
 
-int
-rasqal_init_result_format_sparql_xml(rasqal_world* world)
+static int
+rasqal_query_results_xml_recognise_syntax(rasqal_query_results_format_factory* factory, 
+                                          const unsigned char *buffer, 
+                                          size_t len,
+                                          const unsigned char *identifier,
+                                          const unsigned char *suffix,
+const char *mime_type)
 {
-  rasqal_query_results_formatter_func writer_fn=NULL;
-  rasqal_query_results_formatter_func reader_fn=NULL;
-  rasqal_query_results_get_rowsource_func get_rowsource_fn=NULL;
-  int rc=0;
 
-  /*
-   * SPARQL XML Results 2008-01-15
-   * http://www.w3.org/TR/2008/REC-rdf-sparql-XMLres-20080115/
-   */
-  writer_fn=&rasqal_query_results_write_sparql_xml;
-  reader_fn=NULL,
-  get_rowsource_fn=&rasqal_query_results_get_rowsource_sparql_xml;
-  rc+= rasqal_query_results_format_register_factory(world,
-                                                    "xml",
-                                                    "SPARQL XML Query Results",
-                                                    (unsigned char*)"http://www.w3.org/2005/sparql-results#",
-                                                    writer_fn, reader_fn, get_rowsource_fn,
-                                                    "application/sparql-results+xml")
-                                                    != 0;
-  rc+= rasqal_query_results_format_register_factory(world,
-                                                    NULL,
-                                                    NULL,
-                                                    (unsigned char*)"http://www.w3.org/TR/2008/REC-rdf-sparql-XMLres-20080115/",
-                                                    writer_fn, reader_fn, get_rowsource_fn,
-                                                    "application/sparql-results+xml")
-                                                    != 0;
+  if(suffix && !strcmp((const char*)suffix, "srx"))
+    return 8;
+  
+  return 0;
+}
+
+
+static const char* const sparql_xml_names[] = { "xml", NULL};
+
+static const char* const sparql_xml_uri_strings[] = {
+  "http://www.w3.org/TR/rdf-sparql-XMLres/",
+  "http://www.w3.org/TR/2008/REC-rdf-sparql-XMLres-20080115/",
+  "http://www.w3.org/2005/sparql-results#",
+  NULL
+};
+
+static const raptor_type_q sparql_xml_types[] = {
+  { "application/sparql-results+xml", 30, 10}, 
+  { NULL, 0, 0}
+};
+
+static int
+rasqal_query_results_sparql_xml_register_factory(rasqal_query_results_format_factory *factory) 
+{
+  int rc = 0;
+
+  factory->desc.names = sparql_xml_names;
+  factory->desc.mime_types = sparql_xml_types;
+
+  factory->desc.label = "SPARQL XML Query Results";
+  factory->desc.uri_strings = sparql_xml_uri_strings;
+
+  factory->desc.flags = 0;
+  
+  factory->write         = rasqal_query_results_write_sparql_xml;
+  factory->get_rowsource = rasqal_query_results_get_rowsource_sparql_xml;
+  factory->recognise_syntax = rasqal_query_results_xml_recognise_syntax;
+
   return rc;
 }
 
+
+int
+rasqal_init_result_format_sparql_xml(rasqal_world* world)
+{
+  return !rasqal_world_register_query_results_format_factory(world,
+                                                             &rasqal_query_results_sparql_xml_register_factory);
+}
