@@ -47,8 +47,13 @@
 static void
 rasqal_free_chararray(void* object)
 {
-  char** values = (char**)object;
+  char** values;
   int i;
+
+  if(!object)
+    return;
+  
+  values = (char**)object;
   for(i = 0; values[i] != (char*)-1; i++) {
     if(values[i])
       free(values[i]);
@@ -56,14 +61,6 @@ rasqal_free_chararray(void* object)
   free(values);
 }
 
-#ifdef HAVE_RAPTOR2_API
-static void
-rasqal_free_chararray_v2(void* context, void* object)
-{
-  rasqal_free_chararray(object);
-}
-#endif
-
 
 static int
 rasqal_iostream_write_counted_string_padded(raptor_iostream *iostr,
@@ -90,6 +87,7 @@ rasqal_query_results_write_table_bindings(raptor_iostream *iostr,
                                           rasqal_query_results* results,
                                           raptor_uri *base_uri)
 {
+  rasqal_world* world = rasqal_query_results_get_world(results);
   raptor_sequence *seq = NULL;
   int *widths = NULL;
   int bindings_count = -1;
@@ -122,11 +120,7 @@ rasqal_query_results_write_table_bindings(raptor_iostream *iostr,
       widths[i] = w;
   }
 
-#ifdef HAVE_RAPTOR2_API
-  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_chararray_v2, NULL);
-#else
-  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_chararray, NULL);
-#endif
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_chararray, NULL);
   if(!seq) {
     rc = 1;
     goto tidy;
@@ -148,14 +142,9 @@ rasqal_query_results_write_table_bindings(raptor_iostream *iostr,
       if(!l)
         continue;
 
-#ifdef HAVE_RAPTOR2_API
-      str_iostr = raptor_new_iostream_to_string(rasqal_query_results_get_query(results)->world->raptor_world_ptr,
+      str_iostr = raptor_new_iostream_to_string(world->raptor_world_ptr,
                                                 (void**)&values[i], &v_len,
                                                 rasqal_alloc_memory);
-#else
-      str_iostr = raptor_new_iostream_to_string((void**)&values[i], &v_len,
-                                                rasqal_alloc_memory);
-#endif
       if(!str_iostr) {
         rc = 1;
         goto tidy;
@@ -240,7 +229,7 @@ rasqal_query_results_write_table_bindings(raptor_iostream *iostr,
       raptor_iostream_counted_string_write(VSEP, VSEP_LEN, iostr);
       for(i = 0; i < bindings_count; i++) {
         char *value = values[i];
-        int w = strlen((const char*)value);
+        int w = value ? strlen((const char*)value) : 0;
     
         raptor_iostream_counted_string_write(PAD, PAD_LEN, iostr);
         rasqal_iostream_write_counted_string_padded(iostr, value, w,
@@ -287,7 +276,8 @@ rasqal_query_results_write_table_boolean(raptor_iostream *iostr,
 }
 
 static int
-rasqal_query_results_write_table(raptor_iostream *iostr,
+rasqal_query_results_write_table(rasqal_query_results_formatter* formatter,
+                                 raptor_iostream *iostr,
                                  rasqal_query_results* results,
                                  raptor_uri *base_uri)
 {
@@ -305,15 +295,37 @@ rasqal_query_results_write_table(raptor_iostream *iostr,
   }
 }
 
+
+static const char* const table_names[] = { "table", NULL};
+
+static const raptor_type_q table_types[] = {
+  { "text/plain", 10, 10}, 
+  { NULL, 0, 0}
+};
+
+static int
+rasqal_query_results_table_register_factory(rasqal_query_results_format_factory *factory) 
+{
+  int rc = 0;
+
+  factory->desc.names = table_names;
+  factory->desc.mime_types = table_types;
+
+  factory->desc.label = "Table";
+  factory->desc.uri_strings = NULL;
+
+  factory->desc.flags = 0;
+  
+  factory->write         = rasqal_query_results_write_table;
+  factory->get_rowsource = NULL;
+
+  return rc;
+}
+
+
 int
 rasqal_init_result_format_table(rasqal_world* world)
 {
-  rasqal_query_results_formatter_func writer_fn=NULL;
-  writer_fn = &rasqal_query_results_write_table;
-
-  return rasqal_query_results_format_register_factory(world,
-                                                      "table", "Table",
-                                                      NULL,
-                                                      writer_fn, NULL, NULL,
-                                                      "text/plain");
+  return !rasqal_world_register_query_results_format_factory(world,
+                                                             &rasqal_query_results_table_register_factory);
 }
