@@ -43,13 +43,14 @@
 
 /* prototypes for helper functions */
 static void rasqal_delete_query_language_factories(rasqal_world*);
+static void rasqal_free_query_language_factory(rasqal_query_language_factory *factory);
 
 
 /* statics */
 
-const char * const rasqal_short_copyright_string = "Copyright 2003-2010 David Beckett.  Copyright 2003-2005 University of Bristol";
+const char * const rasqal_short_copyright_string = "Copyright 2003-2011 David Beckett.  Copyright 2003-2005 University of Bristol";
 
-const char * const rasqal_copyright_string = "Copyright (C) 2003-2010 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
+const char * const rasqal_copyright_string = "Copyright (C) 2003-2011 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
 
 const char * const rasqal_license_string = "LGPL 2.1 or newer, GPL 2 or newer, Apache 2.0 or newer.\nSee http://librdf.org/rasqal/LICENSE.html for full terms.";
 
@@ -140,7 +141,6 @@ rasqal_world_open(rasqal_world *world)
   if(world->opened++)
     return 0; /* not an error */
 
-#ifdef HAVE_RAPTOR2_API
   /* Create and init a raptor_world unless one is provided externally with rasqal_world_set_raptor() */
   if(!world->raptor_world_ptr) {
     world->raptor_world_ptr = raptor_new_world();
@@ -151,9 +151,6 @@ rasqal_world_open(rasqal_world *world)
     if(rc)
       return rc;
   }
-#else  
-  raptor_init();
-#endif
 
   rc = rasqal_uri_init(world);
   if(rc)
@@ -163,18 +160,18 @@ rasqal_world_open(rasqal_world *world)
   if(rc)
     return rc;
 
-#ifndef HAVE_RAPTOR2_API
-/* FIXME */
-#ifndef RAPTOR_ERROR_HANDLER_MAGIC
-#define RAPTOR_ERROR_HANDLER_MAGIC 0xD00DB1FF
-#endif
-  world->error_handlers.magic = RAPTOR_ERROR_HANDLER_MAGIC;
-#endif
+  world->query_languages = raptor_new_sequence((raptor_data_free_handler)rasqal_free_query_language_factory, NULL);
+  if(!world->query_languages)
+    return 1;
+  
+  /* first query language declared is the default */
 
-  /* last one declared is the default - RDQL */
+#ifdef RASQAL_QUERY_SPARQL  
+  rc = rasqal_init_query_language_sparql(world);
+  if(rc)
+    return rc;
 
-#ifdef RASQAL_QUERY_RDQL
-  rc = rasqal_init_query_language_rdql(world);
+  rc = rasqal_init_query_language_sparql11(world);
   if(rc)
     return rc;
 #endif
@@ -185,12 +182,13 @@ rasqal_world_open(rasqal_world *world)
     return rc;
 #endif
 
-#ifdef RASQAL_QUERY_SPARQL  
-  rc = rasqal_init_query_language_sparql(world);
+#ifdef RASQAL_QUERY_RDQL
+  rc = rasqal_init_query_language_rdql(world);
   if(rc)
     return rc;
 #endif
 
+
 #ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
   rc = rasqal_raptor_init(world);
   if(rc)
@@ -242,12 +240,8 @@ rasqal_free_world(rasqal_world* world)
 
   rasqal_uri_finish(world);
 
-#ifdef HAVE_RAPTOR2_API
   if(world->raptor_world_ptr && world->raptor_world_allocated_here)
     raptor_free_world(world->raptor_world_ptr);
-#else
-  raptor_finish();
-#endif
 
   RASQAL_FREE(rasqal_world, world);
 }
@@ -292,7 +286,6 @@ rasqal_world_get_raptor(rasqal_world* world)
 }
 
 
-#ifdef HAVE_RAPTOR2_API
 /**
  * rasqal_world_set_log_handler:
  * @world: rasqal_world object
@@ -310,7 +303,6 @@ rasqal_world_set_log_handler(rasqal_world* world, void *user_data,
   world->log_handler = handler;
   world->log_handler_user_data = user_data;
 }
-#endif
 
 
 /* helper functions */
@@ -321,23 +313,13 @@ rasqal_world_set_log_handler(rasqal_world* world, void *user_data,
 static void
 rasqal_free_query_language_factory(rasqal_query_language_factory *factory)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(factory, rasqal_query_language_factory);
+  if(!factory)
+    return;
   
-  if(factory) {
-    if(factory->finish_factory)
-      factory->finish_factory(factory);
+  if(factory->finish_factory)
+    factory->finish_factory(factory);
 
-    if(factory->name)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->name);
-    if(factory->label)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->label);
-    if(factory->alias)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->alias);
-    if(factory->uri_string)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->uri_string);
-
-    RASQAL_FREE(rasqal_query_language_factory, factory);
-  }
+  RASQAL_FREE(rasqal_query_language_factory, factory);
 }
 
 
@@ -347,105 +329,63 @@ rasqal_free_query_language_factory(rasqal_query_language_factory *factory)
 static void
 rasqal_delete_query_language_factories(rasqal_world *world)
 {
-  rasqal_query_language_factory *factory, *next;
-  
-  for(factory=world->query_languages; factory; factory=next) {
-    next=factory->next;
-    rasqal_free_query_language_factory(factory);
+  if(world->query_languages) {
+    raptor_free_sequence(world->query_languages);
+    world->query_languages = NULL;
   }
-  world->query_languages=NULL;
 }
 
 
 /* class methods */
 
 /*
- * rasqal_query_language_register_factory - Register a syntax handled by a query factory
- * @name: the short syntax name
- * @label: readable label for syntax
- * @uri_string: URI string of the syntax (or NULL)
+ * rasqal_query_language_register_factory:
  * @factory: pointer to function to call to register the factory
  * 
- * INTERNAL
+ * INTERNAL - Register a query language syntax handled by a query factory
  *
- * Return value: non-0 on failure
+ * Return value: new factory or NULL on failure
  **/
 RASQAL_EXTERN_C
-int
+rasqal_query_language_factory*
 rasqal_query_language_register_factory(rasqal_world *world,
-                                     const char *name, const char *label,
-                                     const char *alias,
-                                     const unsigned char *uri_string,
-                                     void (*factory) (rasqal_query_language_factory*)) 
+                                       int (*factory) (rasqal_query_language_factory*))
 {
-  rasqal_query_language_factory *query, *h;
-  char *name_copy, *label_copy, *alias_copy;
-  unsigned char *uri_string_copy;
+  rasqal_query_language_factory *query = NULL;
   
-#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
-  RASQAL_DEBUG4("Received registration for syntax %s '%s' with alias '%s'\n", 
-                name, label, (alias ? alias : "none"));
-  RASQAL_DEBUG2("URI %s\n", (uri_string ? (const char*)uri_string : (const char*)"none"));
-#endif
-  
-  query=(rasqal_query_language_factory*)RASQAL_CALLOC(rasqal_query_language_factory, 1,
-                                                    sizeof(rasqal_query_language_factory));
+  query = (rasqal_query_language_factory*)RASQAL_CALLOC(rasqal_query_language_factory, 1,
+                                                        sizeof(*query));
   if(!query)
-    goto tidy_noquery;
+    goto tidy;
 
-  for(h = world->query_languages; h; h = h->next ) {
-    if(!strcmp(h->name, name) ||
-       (alias && !strcmp(h->name, alias))) {
-      RASQAL_FATAL2("query %s already registered\n", h->name);
-    }
-  }
+  query->world = world;
+  
+  if(raptor_sequence_push(world->query_languages, query))
+    return NULL; /* on error, query is already freed by the sequence */
   
-  name_copy=(char*)RASQAL_CALLOC(cstring, strlen(name)+1, 1);
-  if(!name_copy)
-    goto tidy;
-  strcpy(name_copy, name);
-  query->name=name_copy;
-        
-  label_copy=(char*)RASQAL_CALLOC(cstring, strlen(label)+1, 1);
-  if(!label_copy)
-    goto tidy;
-  strcpy(label_copy, label);
-  query->label=label_copy;
-
-  if(uri_string) {
-    uri_string_copy=(unsigned char*)RASQAL_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
-    if(!uri_string_copy)
-      goto tidy;
-    strcpy((char*)uri_string_copy, (const char*)uri_string);
-    query->uri_string=uri_string_copy;
-  }
-        
-  if(alias) {
-    alias_copy=(char*)RASQAL_CALLOC(cstring, strlen(alias)+1, 1);
-    if(!alias_copy)
-      goto tidy;
-    strcpy(alias_copy, alias);
-    query->alias=alias_copy;
-  }
-
   /* Call the query registration function on the new object */
-  (*factory)(query);
+  if(factory(query))
+    return NULL; /* query is owned and freed by the query_languages sequence */
   
+  if(raptor_syntax_description_validate(&query->desc)) {
+    rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR, NULL,
+                            "Query language format description failed to validate\n");
+    goto tidy;
+  }
+
 #if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
-  RASQAL_DEBUG3("%s has context size %d\n", name, (int)query->context_length);
+  RASQAL_DEBUG3("Registered query language %s with context size %d\n",
+                query->names[0], query->context_length);
 #endif
-  
-  query->next = world->query_languages;
-  world->query_languages = query;
 
-  return 0;
+  return query;
 
+  /* Clean up on failure */
   tidy:
-  rasqal_free_query_language_factory(query);
-  tidy_noquery:
-  rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_FATAL, NULL,
-                          "Out of memory in rasqal_query_language_register_factory()");
-  return 1;
+  if(query)
+    rasqal_free_query_language_factory(query);
+
+  return NULL;
 }
 
 
@@ -462,32 +402,30 @@ rasqal_query_language_factory*
 rasqal_get_query_language_factory(rasqal_world *world, const char *name,
                                   const unsigned char *uri)
 {
-  rasqal_query_language_factory *factory;
+  rasqal_query_language_factory *factory = NULL;
 
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
-
-  /* for compatibility with old API that does not call this - FIXME Remove V2 */
-  rasqal_world_open(world);
-  
-  /* return 1st query if no particular one wanted - why? */
-  if(!name && !uri) {
-    factory=world->query_languages;
+  /* return 1st query language if no particular one wanted - why? */
+  if(!name) {
+    factory = (rasqal_query_language_factory*)raptor_sequence_get_at(world->query_languages, 0);
     if(!factory) {
-      RASQAL_DEBUG1("No (default) query_languages registered\n");
+      RASQAL_DEBUG1("No (default) query languages registered\n");
       return NULL;
     }
   } else {
-    for(factory=world->query_languages; factory; factory=factory->next) {
-      if((name && !strcmp(factory->name, name)) ||
-         (factory->alias && !strcmp(factory->alias, name)))
+    int i;
+    
+    for(i = 0;
+        (factory = (rasqal_query_language_factory*)raptor_sequence_get_at(world->query_languages, i));
+        i++) {
+      int namei;
+      const char* fname;
+      
+      for(namei = 0; (fname = factory->desc.names[namei]); namei++) {
+        if(!strcmp(fname, name))
+          break;
+      }
+      if(fname)
         break;
-      if(uri && !strcmp((const char*)factory->uri_string, (const char*)uri))
-        break;
-    }
-    /* else FACTORY name not found */
-    if(!factory) {
-      RASQAL_DEBUG2("No query language with name %s found\n", name);
-      return NULL;
     }
   }
         
@@ -495,6 +433,33 @@ rasqal_get_query_language_factory(rasqal_world *world, const char *name,
 }
 
 
+/**
+ * rasqal_world_get_query_language_description:
+ * @world: world object
+ * @counter: index into the list of query languages
+ *
+ * Get query language descriptive information
+ *
+ * Return value: description or NULL if counter is out of range
+ **/
+const raptor_syntax_description*
+rasqal_world_get_query_language_description(rasqal_world* world,
+                                            unsigned int counter)
+{
+  rasqal_query_language_factory *factory;
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
+
+  rasqal_world_open(world);
+
+  factory = raptor_sequence_get_at(world->query_languages, counter);
+  if(!factory)
+    return NULL;
+
+  return &factory->desc;
+}
+
+
 /**
  * rasqal_languages_enumerate:
  * @world: rasqal_world object
@@ -503,6 +468,8 @@ rasqal_get_query_language_factory(rasqal_world *world, const char *name,
  * @label: pointer to store syntax readable label (or NULL)
  * @uri_string: pointer to store syntax URI string (or NULL)
  *
+ * @deprecated: Use rasqal_world_get_query_language_description() instead.
+ *
  * Get information on query languages.
  * 
  * Return value: non 0 on failure of if counter is out of range
@@ -513,7 +480,6 @@ rasqal_languages_enumerate(rasqal_world *world,
                            const char **name, const char **label,
                            const unsigned char **uri_string)
 {
-  unsigned int i;
   rasqal_query_language_factory *factory;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
@@ -523,23 +489,20 @@ rasqal_languages_enumerate(rasqal_world *world,
   /* for compatibility with old API that does not call this - FIXME Remove V2 */
   rasqal_world_open(world);
   
-  factory = world->query_languages;
+  factory = raptor_sequence_get_at(world->query_languages, counter);
   if(!factory)
     return 1;
 
-  for(i=0; factory && i<=counter ; i++, factory=factory->next) {
-    if(i == counter) {
-      if(name)
-        *name=factory->name;
-      if(label)
-        *label=factory->label;
-      if(uri_string)
-        *uri_string=factory->uri_string;
-      return 0;
-    }
-  }
-        
-  return 1;
+  if(name)
+    *name = factory->desc.names[0];
+
+  if(label)
+    *label = factory->desc.label;
+
+  if(uri_string && factory->desc.uri_strings)
+    *uri_string = (const unsigned char*)factory->desc.uri_strings[0];
+
+  return 0;
 }
 
 
@@ -562,8 +525,6 @@ rasqal_language_name_check(rasqal_world* world, const char *name)
 
 
 static const char* const rasqal_log_level_labels[RAPTOR_LOG_LEVEL_LAST+1]={
-#ifdef HAVE_RAPTOR2_API
-  /* raptor2 levels */
  "none",
  "trace",
  "debug",
@@ -571,13 +532,6 @@ static const char* const rasqal_log_level_labels[RAPTOR_LOG_LEVEL_LAST+1]={
  "warn",
  "error",
  "fatal"
-#else
-  /* raptor1 levels */
-  "none",
-  "fatal error",
-  "error",
-  "warning"
-#endif
 };
 
 
@@ -605,14 +559,9 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
 {
   char *buffer;
   size_t length;
-#ifdef HAVE_RAPTOR2_API
   raptor_log_message logmsg;
   raptor_log_handler handler = world->log_handler;
   void* handler_data = world->log_handler_user_data;
-#else
-  raptor_message_handler handler = world->error_handlers.handlers[level].handler;
-  void* handler_data = world->error_handlers.handlers[level].user_data;
-#endif
   
   if(level == RAPTOR_LOG_LEVEL_NONE)
     return;
@@ -620,11 +569,7 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   buffer=raptor_vsnprintf(message, arguments);
   if(!buffer) {
     if(locator) {
-#ifdef HAVE_RAPTOR2_API
       raptor_locator_print(locator, stderr);
-#else
-      raptor_print_locator(stderr, locator);
-#endif
       fputc(' ', stderr);
     }
     fputs("rasqal ", stderr);
@@ -643,24 +588,15 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
     /* This is the single place in rasqal that the user error handler
      * functions are called.
      */
-#ifdef HAVE_RAPTOR2_API
     /* raptor2 raptor_log_handler */
     logmsg.code = -1; /* no information to put as code */
     logmsg.level = level;
     logmsg.locator = locator;
     logmsg.text = buffer;
     handler(handler_data, &logmsg);
-#else
-    /* raptor1 raptor_message_handler */
-    handler(handler_data, locator, buffer);
-#endif
   } else {
     if(locator) {
-#ifdef HAVE_RAPTOR2_API
       raptor_locator_print(locator, stderr);
-#else
-      raptor_print_locator(stderr, locator);
-#endif
       fputc(' ', stderr);
     }
     fputs("rasqal ", stderr);
@@ -698,6 +634,30 @@ rasqal_query_simple_error(void* user_data, const char *message, ...)
 }
 
 
+/*
+ * rasqal_world_simple_error:
+ *
+ * INTERNAL - Handle a simple error
+ *
+ * Matches the raptor_simple_message_handler API but with a world object
+ */
+void
+rasqal_world_simple_error(void* user_data, const char *message, ...)
+{
+  rasqal_world* world = (rasqal_world*)user_data;
+
+  va_list arguments;
+
+  va_start(arguments, message);
+
+  rasqal_log_error_varargs(world,
+                           RAPTOR_LOG_LEVEL_ERROR, NULL,
+                           message, arguments);
+  
+  va_end(arguments);
+}
+
+
 /* wrapper */
 const char*
 rasqal_basename(const char *name)
@@ -921,10 +881,10 @@ rasqal_world_set_default_generate_bnodeid_parameters(rasqal_world* world,
   if(prefix) {
     length = strlen(prefix);
     
-    prefix_copy = (char*)RASQAL_MALLOC(cstring, length+1);
+    prefix_copy = (char*)RASQAL_MALLOC(cstring, length + 1);
     if(!prefix_copy)
       return 1;
-    strcpy(prefix_copy, prefix);
+    memcpy(prefix_copy, prefix, length + 1);
   }
   
   if(world->default_generate_bnodeid_handler_prefix)
@@ -955,15 +915,12 @@ rasqal_world_set_default_generate_bnodeid_parameters(rasqal_world* world,
  *
  * If handler is NULL, the default method is used
  *
- * If set, this overrides any bnodeid handler set via
- * rasqal_query_set_generate_bnodeid_handler().
- *  
  * Return value: non-0 on failure
  **/
 int
 rasqal_world_set_generate_bnodeid_handler(rasqal_world* world,
                                           void *user_data,
-                                          rasqal_generate_bnodeid_handler2 handler)
+                                          rasqal_generate_bnodeid_handler handler)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
 
@@ -1031,6 +988,52 @@ rasqal_world_generate_bnodeid(rasqal_world* world,
 }
 
 
+/**
+ * rasqal_world_reset_now:
+ * @world: world
+ *
+ * Internal - Mark current now value as invalid
+ *
+ * Intended to be run before starting a query so that the
+ * value is recalculated.
+ *
+ * Return value: non-0 on failure
+ */
+int
+rasqal_world_reset_now(rasqal_world* world)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
+
+  world->now_set = 0;
+
+  return 0;
+}
+
+
+/**
+ * rasqal_world_get_now_timeval:
+ * @world: world
+ *
+ * Internal - Get current now timeval
+ *
+ * Return value: pointer to timeval or NULL on failure
+ */
+struct timeval*
+rasqal_world_get_now_timeval(rasqal_world* world)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
+
+  if(!world->now_set) {
+    if(gettimeofday(&world->now, NULL))
+      return NULL;
+    
+    world->now_set = 1;
+  }
+
+  return &world->now;
+}
+
+
 /**
  * rasqal_free_memory:
  * @ptr: memory pointer
