@@ -2,7 +2,7 @@
  *
  * rasqal_graph_pattern.c - Rasqal graph pattern class
  *
- * Copyright (C) 2004-2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2004-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -62,20 +62,23 @@ rasqal_new_graph_pattern(rasqal_query* query,
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
   
-  gp = (rasqal_graph_pattern*)RASQAL_CALLOC(rasqal_graph_pattern, 1, sizeof(rasqal_graph_pattern));
+  gp = (rasqal_graph_pattern*)RASQAL_CALLOC(rasqal_graph_pattern, 1, 
+                                            sizeof(*gp));
   if(!gp)
     return NULL;
 
   gp->op = op;
   
-  gp->query=query;
-  gp->triples= NULL;
-  gp->start_column= -1;
-  gp->end_column= -1;
-  /* This is initialised by rasqal_query_prepare_count_graph_patterns() inside
+  gp->query = query;
+  gp->triples = NULL;
+  gp->start_column = -1;
+  gp->end_column = -1;
+
+  /* This is initialised by
+   * rasqal_query_prepare_count_graph_patterns() inside
    * rasqal_query_prepare()
    */
-  gp->gp_index= -1;
+  gp->gp_index = -1;
 
   return gp;
 }
@@ -106,9 +109,9 @@ rasqal_new_basic_graph_pattern(rasqal_query* query,
   if(!gp)
     return NULL;
 
-  gp->triples=triples;
-  gp->start_column=start_column;
-  gp->end_column=end_column;
+  gp->triples = triples;
+  gp->start_column = start_column;
+  gp->end_column = end_column;
 
   return gp;
 }
@@ -141,7 +144,7 @@ rasqal_new_graph_pattern_from_sequence(rasqal_query* query,
     return NULL;
   }
   
-  gp->graph_patterns=graph_patterns;
+  gp->graph_patterns = graph_patterns;
   return gp;
 }
 
@@ -172,7 +175,7 @@ rasqal_new_filter_graph_pattern(rasqal_query* query,
   
   if(rasqal_graph_pattern_set_filter_expression(gp, expr)) {
     rasqal_free_graph_pattern(gp);
-    gp=NULL;
+    gp = NULL;
   }
 
   return gp;
@@ -213,6 +216,87 @@ rasqal_new_let_graph_pattern(rasqal_query *query,
 }
 
 
+/*
+ * rasqal_new_select_graph_pattern:
+ * @query: #rasqal_graph_pattern query object
+ * @projection: projection object
+ * @where: WHERE graph pattern
+ * @modifiers: solution modifier
+ *
+ * INTERNAL - Create a new SELECT graph pattern
+ * 
+ * Return value: a new #rasqal_graph_pattern object or NULL on failure
+ **/
+rasqal_graph_pattern*
+rasqal_new_select_graph_pattern(rasqal_query *query,
+                                rasqal_projection* projection,
+                                rasqal_graph_pattern* where,
+                                rasqal_solution_modifier* modifier)
+{
+  rasqal_graph_pattern* gp;
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(projection, rasqal_projeciton, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(where, rasqal_graph_pattern, NULL);
+  
+  gp = rasqal_new_graph_pattern(query, RASQAL_GRAPH_PATTERN_OPERATOR_SELECT);
+  if(!gp) {
+    rasqal_free_projection(projection);
+    if(where)
+      rasqal_free_graph_pattern(where);
+
+    if(modifier)
+      rasqal_free_solution_modifier(modifier);
+
+    return NULL;
+  }
+
+  gp->projection = projection;
+  gp->modifier = modifier;
+  
+  if(rasqal_graph_pattern_add_sub_graph_pattern(gp, where)) {
+    rasqal_free_graph_pattern(gp);
+    return NULL;
+  }
+
+  return gp;
+}
+
+
+/*
+ * rasqal_new_single_graph_pattern:
+ * @query: #rasqal_graph_pattern query object
+ * @op: enum #rasqal_graph_pattern_operator operator
+ * @single: single inner graph grattern
+ *
+ * INTERNAL - Create a new graph pattern object over a single graph pattern.
+ * 
+ * Return value: a new #rasqal_graph_pattern object or NULL on failure
+ **/
+rasqal_graph_pattern*
+rasqal_new_single_graph_pattern(rasqal_query* query,
+                                rasqal_graph_pattern_operator op,
+                                rasqal_graph_pattern* single)
+{
+  rasqal_graph_pattern* gp;
+
+  gp = rasqal_new_graph_pattern(query, op);
+  if(!gp) {
+    if(single)
+      rasqal_free_graph_pattern(single);
+
+    return NULL;
+  }
+
+  if(rasqal_graph_pattern_add_sub_graph_pattern(gp, single)) {
+    rasqal_free_graph_pattern(gp);
+    return NULL;
+  }
+
+  return gp;
+}
+
+
 /*
  * rasqal_free_graph_pattern:
  * @gp: #rasqal_graph_pattern object
@@ -223,7 +307,8 @@ rasqal_new_let_graph_pattern(rasqal_query *query,
 void
 rasqal_free_graph_pattern(rasqal_graph_pattern* gp)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(gp, rasqal_graph_pattern);
+  if(!gp)
+    return;
   
   if(gp->graph_patterns)
     raptor_free_sequence(gp->graph_patterns);
@@ -234,6 +319,12 @@ rasqal_free_graph_pattern(rasqal_graph_pattern* gp)
   if(gp->origin)
     rasqal_free_literal(gp->origin);
 
+  if(gp->projection)
+    rasqal_free_projection(gp->projection);
+
+  if(gp->modifier)
+    rasqal_free_solution_modifier(gp->modifier);
+
   RASQAL_FREE(rasqal_graph_pattern, gp);
 }
 
@@ -272,7 +363,9 @@ rasqal_graph_pattern_set_filter_expression(rasqal_graph_pattern* gp,
   
   if(gp->filter_expression)
     rasqal_free_expression(gp->filter_expression);
+
   gp->filter_expression = expr;
+
   return 0;
 }
 
@@ -314,7 +407,7 @@ rasqal_graph_pattern_get_operator(rasqal_graph_pattern* graph_pattern)
 }
 
 
-static const char* const rasqal_graph_pattern_operator_labels[RASQAL_GRAPH_PATTERN_OPERATOR_LAST+1]={
+static const char* const rasqal_graph_pattern_operator_labels[RASQAL_GRAPH_PATTERN_OPERATOR_LAST + 1] = {
   "UNKNOWN",
   "Basic",
   "Optional",
@@ -322,7 +415,10 @@ static const char* const rasqal_graph_pattern_operator_labels[RASQAL_GRAPH_PATTE
   "Group",
   "Graph",
   "Filter",
-  "Let"
+  "Let",
+  "Select",
+  "Service",
+  "Minus"
 };
 
 
@@ -339,7 +435,7 @@ rasqal_graph_pattern_operator_as_string(rasqal_graph_pattern_operator op)
 {
   if(op <= RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN || 
      op > RASQAL_GRAPH_PATTERN_OPERATOR_LAST)
-    op=RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN;
+    op = RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN;
 
   return rasqal_graph_pattern_operator_labels[(int)op];
 }
@@ -352,13 +448,13 @@ rasqal_graph_pattern_operator_as_string(rasqal_graph_pattern_operator op)
 #endif
 
 #define SPACES_LENGTH 80
-static const char spaces[SPACES_LENGTH+1]="                                                                                ";
+static const char spaces[SPACES_LENGTH + 1] = "                                                                                ";
 
 static void
 rasqal_graph_pattern_write_indent(raptor_iostream *iostr, int indent) 
 {
   while(indent > 0) {
-    int sp=(indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
+    int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
     raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
@@ -372,6 +468,7 @@ rasqal_graph_pattern_write_plurals(raptor_iostream *iostr,
   raptor_iostream_decimal_write(value, iostr);
   raptor_iostream_write_byte(' ', iostr);
   raptor_iostream_string_write(label, iostr);
+
   if(value != 1)
     raptor_iostream_write_byte('s', iostr);
 }
@@ -390,37 +487,40 @@ static int
 rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
                                     raptor_iostream* iostr, int indent)
 {
-  int pending_nl=0;
+  int pending_nl = 0;
   
   raptor_iostream_counted_string_write("graph pattern", 13, iostr);
+
   if(gp->gp_index >= 0) {
     raptor_iostream_write_byte('[', iostr);
     raptor_iostream_decimal_write(gp->gp_index, iostr);
     raptor_iostream_write_byte(']', iostr);
   }
+
   raptor_iostream_write_byte(' ', iostr);
   raptor_iostream_string_write(rasqal_graph_pattern_operator_as_string(gp->op),
                                iostr);
   raptor_iostream_write_byte('(', iostr);
 
   if(indent >= 0)
-    indent+= 2;
+    indent += 2;
 
   if(gp->triples) {
-    int size=gp->end_column - gp->start_column +1;
+    int size = gp->end_column - gp->start_column + 1;
     int i;
+
     raptor_iostream_counted_string_write("over ", 5, iostr);
     rasqal_graph_pattern_write_plurals(iostr, "triple", size);
     raptor_iostream_write_byte('[', iostr);
 
     if(indent >= 0) {
       raptor_iostream_write_byte('\n', iostr);
-      indent+= 2;
+      indent += 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
     
-    for(i=gp->start_column; i <= gp->end_column; i++) {
-      rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(gp->triples, i);
+    for(i = gp->start_column; i <= gp->end_column; i++) {
+      rasqal_triple *t = (rasqal_triple*)raptor_sequence_get_at(gp->triples, i);
       if(i > gp->start_column) {
         raptor_iostream_counted_string_write(" ,", 2, iostr);
 
@@ -431,14 +531,16 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
       }
       rasqal_triple_write(t, iostr);
     }
+
     if(indent >= 0) {
       raptor_iostream_write_byte('\n', iostr);
-      indent-= 2;
+      indent -= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
+
     raptor_iostream_write_byte(']', iostr);
 
-    pending_nl=1;
+    pending_nl = 1;
   }
 
   if(gp->origin) {
@@ -455,15 +557,16 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
 
     rasqal_literal_write(gp->origin, iostr);
 
-    pending_nl=1;
+    pending_nl = 1;
   }
 
   if(gp->graph_patterns) {
-    int size=raptor_sequence_size(gp->graph_patterns);
+    int size = raptor_sequence_size(gp->graph_patterns);
     int i;
 
     if(pending_nl) {
       raptor_iostream_counted_string_write(" ,", 2, iostr);
+
       if(indent >= 0) {
         raptor_iostream_write_byte('\n', iostr);
         rasqal_graph_pattern_write_indent(iostr, indent);
@@ -476,13 +579,13 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
 
     if(indent >= 0) {
       raptor_iostream_write_byte('\n', iostr);
-      indent+= 2;
+      indent += 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
     
-    for(i=0; i< size; i++) {
+    for(i = 0; i< size; i++) {
       rasqal_graph_pattern* sgp;
-      sgp=(rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+      sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
       if(i) {
         raptor_iostream_counted_string_write(" ,", 2, iostr);
         if(indent >= 0) {
@@ -490,19 +593,22 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
           rasqal_graph_pattern_write_indent(iostr, indent);
         }
       }
+
       if(sgp)
         rasqal_graph_pattern_write_internal(sgp, iostr, indent);
       else
         raptor_iostream_counted_string_write("(empty)", 7, iostr);
     }
+
     if(indent >= 0) {
       raptor_iostream_write_byte('\n', iostr);
-      indent-= 2;
+      indent -= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
+
     raptor_iostream_write_byte(']', iostr);
 
-    pending_nl=1;
+    pending_nl = 1;
   }
 
   if(gp->var) {
@@ -526,19 +632,84 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
   
     if(indent >= 0) {
       raptor_iostream_write_byte('\n', iostr);
-      indent+= 2;
+      indent += 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
 
     rasqal_expression_write(gp->filter_expression, iostr);
+
     if(indent >= 0)
-      indent-= 2;
+      indent -= 2;
     
-    pending_nl=1;
+    pending_nl = 1;
   }
 
+  if(gp->projection) {
+    int i;
+    raptor_sequence* vars_seq;
+    
+    if(pending_nl) {
+      raptor_iostream_counted_string_write(" ,", 2, iostr);
+
+      if(indent >= 0) {
+        raptor_iostream_write_byte('\n', iostr);
+        rasqal_graph_pattern_write_indent(iostr, indent);
+      }
+    }
+
+    if(indent >= 0) {
+      raptor_iostream_write_byte('\n', iostr);
+      indent += 2;
+      rasqal_graph_pattern_write_indent(iostr, indent);
+    }
+
+    raptor_iostream_counted_string_write("select-variables: [", 19, iostr);
+    vars_seq = rasqal_projection_get_variables_sequence(gp->projection);
+    for(i = 0; i < raptor_sequence_size(vars_seq); i++) {
+      rasqal_variable* v;
+      v = (rasqal_variable*)raptor_sequence_get_at(vars_seq, i);
+
+      if(i > 0)
+        raptor_iostream_counted_string_write(", ", 2, iostr);
+
+      rasqal_variable_write(v, iostr);
+    }
+    raptor_iostream_counted_string_write("]", 1, iostr);
+    
+    if(indent >= 0)
+      indent -= 2;
+
+    pending_nl = 1;
+  }
+
+#if 0  
+  if(gp->where) {
+    if(pending_nl) {
+      raptor_iostream_counted_string_write(" ,", 2, iostr);
+
+      if(indent >= 0) {
+        raptor_iostream_write_byte('\n', iostr);
+        rasqal_graph_pattern_write_indent(iostr, indent);
+      }
+    }
+
+    if(indent >= 0) {
+      raptor_iostream_write_byte('\n', iostr);
+      indent += 2;
+      rasqal_graph_pattern_write_indent(iostr, indent);
+    }
+
+    rasqal_graph_pattern_write_internal(gp->where, iostr, indent);
+
+    if(indent >= 0)
+      indent -= 2;
+    
+    pending_nl = 1;
+  }
+#endif
+  
   if(indent >= 0)
-    indent-= 2;
+    indent -= 2;
 
   if(pending_nl) {
     if(indent >= 0) {
@@ -610,18 +781,21 @@ rasqal_graph_pattern_visit(rasqal_query *query,
   if(result)
     return result;
   
-  seq=rasqal_graph_pattern_get_sub_graph_pattern_sequence(gp);
+  seq = rasqal_graph_pattern_get_sub_graph_pattern_sequence(gp);
+
   if(seq && raptor_sequence_size(seq) > 0) {
     int gp_index=0;
+
     while(1) {
       rasqal_graph_pattern* sgp;
-      sgp=rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+      sgp = rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
       if(!sgp)
         break;
       
-      result=rasqal_graph_pattern_visit(query, sgp, fn, user_data);
+      result = rasqal_graph_pattern_visit(query, sgp, fn, user_data);
       if(result)
         return result;
+
       gp_index++;
     }
   }
@@ -667,17 +841,15 @@ rasqal_graph_pattern_add_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(sub_graph_pattern, rasqal_graph_pattern, 1);
 
   if(!graph_pattern->graph_patterns) {
-#ifdef HAVE_RAPTOR2_API
     graph_pattern->graph_patterns = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
-#else
-    graph_pattern->graph_patterns = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-#endif
+
     if(!graph_pattern->graph_patterns) {
       if(sub_graph_pattern)
         rasqal_free_graph_pattern(sub_graph_pattern);
       return 1;
     }
   }
+
   return raptor_sequence_push(graph_pattern->graph_patterns, sub_graph_pattern);
 }
 
@@ -750,16 +922,21 @@ rasqal_graph_pattern_get_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
  * rasqal_graph_pattern_get_origin:
  * @graph_pattern: #rasqal_graph_pattern graph pattern object
  *
- * Get the graph pattern literal for #RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH graph patterns
+ * Get the graph pattern literal for
+ * #RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH graph pattern
  * 
- * Return value: graph literal parameter
+ * Return value: graph literal parameter or NULL if wrong graph pattern type or not defined
  **/
 rasqal_literal*
 rasqal_graph_pattern_get_origin(rasqal_graph_pattern* graph_pattern)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern,
+                                            rasqal_graph_pattern, NULL);
 
-  return graph_pattern->origin;
+  if(graph_pattern->op == RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH)
+    return graph_pattern->origin;
+  else
+    return NULL;
 }
   
 
@@ -795,14 +972,15 @@ rasqal_new_basic_graph_pattern_from_formula(rasqal_query* query,
                                             rasqal_formula* formula)
 {
   rasqal_graph_pattern* gp;
-  raptor_sequence *triples=query->triples;
-  raptor_sequence *formula_triples=formula->triples;
-  int offset=raptor_sequence_size(triples);
-  int triple_pattern_size=0;
+  raptor_sequence *triples = query->triples;
+  raptor_sequence *formula_triples  = formula->triples;
+  int offset = raptor_sequence_size(triples);
+  int triple_pattern_size = 0;
 
   if(formula_triples) {
     /* Move formula triples to end of main triples sequence */
-    triple_pattern_size=raptor_sequence_size(formula_triples);
+    triple_pattern_size = raptor_sequence_size(formula_triples);
+
     if(raptor_sequence_join(triples, formula_triples)) {
       rasqal_free_formula(formula);
       return NULL;
@@ -811,9 +989,10 @@ rasqal_new_basic_graph_pattern_from_formula(rasqal_query* query,
 
   rasqal_free_formula(formula);
 
-  gp=rasqal_new_basic_graph_pattern(query, triples, 
-                                    offset, 
-                                    offset+triple_pattern_size-1);
+  gp = rasqal_new_basic_graph_pattern(query, triples, 
+                                      offset, 
+                                      offset + triple_pattern_size - 1);
+
   return gp;
 }
 
@@ -840,16 +1019,13 @@ rasqal_new_2_group_graph_pattern(rasqal_query* query,
 {
   raptor_sequence *seq;
 
-#ifdef HAVE_RAPTOR2_API
   seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
-#else
-  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-#endif
   if(!seq) {
     if(first_gp)
       rasqal_free_graph_pattern(first_gp);
     if(second_gp)
       rasqal_free_graph_pattern(second_gp);
+
     return NULL;
   }
 
@@ -857,10 +1033,13 @@ rasqal_new_2_group_graph_pattern(rasqal_query* query,
     raptor_free_sequence(seq);
     if(second_gp)
       rasqal_free_graph_pattern(second_gp);
+
     return NULL;
   }
+
   if(second_gp && raptor_sequence_push(seq, second_gp)) {
     raptor_free_sequence(seq);
+
     return NULL;
   }
 
@@ -889,6 +1068,7 @@ rasqal_graph_pattern_variable_bound_in(rasqal_graph_pattern *gp,
 
   query = gp->query;
   column = query->variables_bound_in[v->offset];
+
   if(column < 0)
     return 0;
   
@@ -920,6 +1100,7 @@ rasqal_new_basic_graph_pattern_from_triples(rasqal_query* query,
   if(triples) {
     /* Move triples to end of graph triples sequence */
     triple_pattern_size = raptor_sequence_size(triples);
+
     if(raptor_sequence_join(graph_triples, triples)) {
       raptor_free_sequence(triples);
       return NULL;
@@ -931,7 +1112,142 @@ rasqal_new_basic_graph_pattern_from_triples(rasqal_query* query,
   gp = rasqal_new_basic_graph_pattern(query, graph_triples,
                                       offset, 
                                       offset + triple_pattern_size - 1);
+
   return gp;
 }
 
 
+/**
+ * rasqal_graph_pattern_get_variable:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ *
+ * Get the variable for #RASQAL_GRAPH_PATTERN_OPERATOR_LET graph pattern
+ * 
+ * Return value: graph variable or NULL if wrong graph pattern or not defined
+ **/
+rasqal_variable*
+rasqal_graph_pattern_get_variable(rasqal_graph_pattern* graph_pattern)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern,
+                                            rasqal_graph_pattern, NULL);
+
+  if(graph_pattern->op == RASQAL_GRAPH_PATTERN_OPERATOR_LET)
+    return graph_pattern->var;
+  else
+    return NULL;
+}
+
+
+/**
+ * rasqal_graph_pattern_get_service:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ *
+ * Get the literal for #RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE graph pattern
+ * 
+ * Return value: graph variable or NULL if wrong graph pattern or not defined
+ **/
+rasqal_literal*
+rasqal_graph_pattern_get_service(rasqal_graph_pattern* graph_pattern)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern,
+                                            rasqal_graph_pattern, NULL);
+
+  if(graph_pattern->op == RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE)
+    return graph_pattern->origin;
+  else
+    return NULL;
+}
+
+
+struct gpft
+{
+  /* sequence owned here */
+  raptor_sequence* triples;
+  /* shared pointer to current origin */
+  rasqal_literal* origin;
+};
+
+static int
+rasqal_graph_pattern_get_flattened_triples_visit(rasqal_query* query,
+                                                 rasqal_graph_pattern* gp,
+                                                 struct gpft* state)
+{
+  raptor_sequence* seq;
+  int size;
+
+  if(gp->op == RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH)
+    state->origin = gp->origin;
+  else if(gp->op != RASQAL_GRAPH_PATTERN_OPERATOR_BASIC)
+    return 1;
+  else {
+    /*  RASQAL_GRAPH_PATTERN_OPERATOR_BASIC */
+    rasqal_triples_sequence_set_origin(state->triples,
+                                       gp->triples, state->origin);
+  }
+  
+
+  seq = rasqal_graph_pattern_get_sub_graph_pattern_sequence(gp);  
+  if(seq) {
+    int gp_index = 0;
+    int result = 0;
+    
+    size = raptor_sequence_size(seq);
+    while(size > 0) {
+      rasqal_graph_pattern* sgp;
+      sgp = rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+      if(!sgp)
+        break;
+      
+      result = rasqal_graph_pattern_get_flattened_triples_visit(query, sgp,
+                                                                state);
+      if(result)
+        return result;
+      
+      gp_index++;
+      size--;
+    }
+  }
+    
+
+  if(gp->op == RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH)
+    state->origin = NULL;
+  
+  return 0;
+}
+
+
+/**
+ * rasqal_graph_pattern_get_flattened_triples:
+ * @query: query
+ * @graph_pattern: graph pattern
+ *
+ * Get the triples inside a tree of graph patterns (BASIC + GRAPH) as a single sequence with GRAPHs turned into triple origin.
+ *
+ * The returned sequence and all the #rasqal_triple in it are owned
+ * by the caller (hold references).
+ *
+ * Return value: new sequence of #raptor_triple or NULL on failure
+ */
+raptor_sequence*
+rasqal_graph_pattern_get_flattened_triples(rasqal_query* query,
+                                           rasqal_graph_pattern* graph_pattern)
+{
+  struct gpft state;
+  
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern,
+                                            rasqal_graph_pattern, NULL);
+
+  memset(&state, '\0', sizeof(state));
+
+  state.triples = raptor_new_sequence((raptor_data_free_handler)rasqal_free_triple,
+                                      (raptor_data_print_handler)rasqal_triple_print);
+  state.origin = NULL;
+  
+  if(rasqal_graph_pattern_get_flattened_triples_visit(query, graph_pattern,
+                                                      &state)) {
+    raptor_free_sequence(state.triples);
+    return NULL;
+  }
+
+  return state.triples;
+}
