@@ -35,6 +35,12 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <stdarg.h>
 
 #include "rasqal.h"
@@ -87,8 +93,8 @@ static int rasqal_query_add_query_result(rasqal_query* query, rasqal_query_resul
  *
  * A query language can be named or identified by a URI, either
  * of which is optional.  The default query language will be used
- * if both are NULL.  rasqal_languages_enumerate returns
- * information on the known names, labels and URIs.
+ * if both are NULL.  rasqal_world_get_query_language_description returns
+ * the description of the known names, labels, MIME types and URIs.
  *
  * Return value: a new #rasqal_query object or NULL on failure
  */
@@ -98,10 +104,6 @@ rasqal_new_query(rasqal_world *world, const char *name,
 {
   rasqal_query_language_factory* factory;
   rasqal_query* query;
-#ifndef HAVE_RAPTOR2_API
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-#endif
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
 
@@ -123,9 +125,6 @@ rasqal_new_query(rasqal_world *world, const char *name,
   
   query->factory = factory;
 
-  query->limit = -1;
-  query->offset = -1;
-
   query->genid_counter = 1;
 
   query->context = (char*)RASQAL_CALLOC(rasqal_query_context, 1,
@@ -133,15 +132,7 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!query->context)
     goto tidy;
   
-#ifdef HAVE_RAPTOR2_API
   query->namespaces = raptor_new_namespaces(world->raptor_world_ptr, 0);
-#else
-  raptor_uri_get_handler(&uri_handler, &uri_context);
-  query->namespaces = raptor_new_namespaces(uri_handler, uri_context,
-                                            (raptor_simple_message_handler)rasqal_query_simple_error,
-                                            query,
-                                            0);
-#endif
   if(!query->namespaces)
     goto tidy;
 
@@ -149,38 +140,28 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!query->vars_table)
     goto tidy;
 
-#ifdef HAVE_RAPTOR2_API
   query->triples = raptor_new_sequence((raptor_data_free_handler)rasqal_free_triple, (raptor_data_print_handler)rasqal_triple_print);
-#else
-  query->triples = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_triple, (raptor_sequence_print_handler*)rasqal_triple_print);
-#endif
   if(!query->triples)
     goto tidy;
 
-#ifdef HAVE_RAPTOR2_API
   query->prefixes = raptor_new_sequence((raptor_data_free_handler)rasqal_free_prefix, (raptor_data_print_handler)rasqal_prefix_print);
-#else
-  query->prefixes = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
-#endif
   if(!query->prefixes)
     goto tidy;
 
-#ifdef HAVE_RAPTOR2_API
   query->data_graphs = raptor_new_sequence((raptor_data_free_handler)rasqal_free_data_graph, (raptor_data_print_handler)rasqal_data_graph_print);
-#else
-  query->data_graphs = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_data_graph, (raptor_sequence_print_handler*)rasqal_data_graph_print);
-#endif
   if(!query->data_graphs)
     goto tidy;
 
-#ifdef HAVE_RAPTOR2_API
   query->results = raptor_new_sequence((raptor_data_free_handler)rasqal_query_results_remove_query_reference, NULL);
-#else
-  query->results = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_query_results_remove_query_reference, NULL);
-#endif
   if(!query->results)
     goto tidy;
 
+  query->eval_context = rasqal_new_evaluation_context(query->world,
+                                                      &query->locator,
+                                                      query->compare_flags);
+  if(!query->eval_context)
+    goto tidy;
+
   if(factory->init(query, name))
     goto tidy;
   
@@ -211,6 +192,9 @@ rasqal_free_query(rasqal_query* query)
   if(query->factory)
     query->factory->terminate(query);
 
+  if(query->eval_context)
+    rasqal_free_evaluation_context(query->eval_context);
+
   if(query->context)
     RASQAL_FREE(rasqal_query_context, query->context);
 
@@ -250,15 +234,6 @@ rasqal_free_query(rasqal_query* query)
   if(query->query_graph_pattern)
     rasqal_free_graph_pattern(query->query_graph_pattern);
 
-  if(query->order_conditions_sequence)
-    raptor_free_sequence(query->order_conditions_sequence);
-
-  if(query->group_conditions_sequence)
-    raptor_free_sequence(query->group_conditions_sequence);
-
-  if(query->having_conditions_sequence)
-    raptor_free_sequence(query->having_conditions_sequence);
-
   if(query->graph_patterns_sequence)
     raptor_free_sequence(query->graph_patterns_sequence);
 
@@ -271,6 +246,12 @@ rasqal_free_query(rasqal_query* query)
 
   if(query->updates)
     raptor_free_sequence(query->updates);
+
+  if(query->modifier)
+    rasqal_free_solution_modifier(query->modifier);
+  
+  if(query->bindings)
+    rasqal_free_bindings(query->bindings);
   
   RASQAL_FREE(rasqal_query, query);
 }
@@ -291,7 +272,7 @@ rasqal_query_get_name(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->factory->name;
+  return query->factory->desc.names[0];
 }
 
 
@@ -308,103 +289,7 @@ rasqal_query_get_label(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->factory->label;
-}
-
-
-/**
- * rasqal_query_set_fatal_error_handler:
- * @query: the query
- * @user_data: user data to pass to function
- * @handler: pointer to the function
- *
- * Set the query error handling function.
- * 
- * The function will receive callbacks when the query fails.
- * 
- **/
-void
-rasqal_query_set_fatal_error_handler(rasqal_query* query, void *user_data,
-                                     raptor_message_handler handler)
-{
-#ifdef HAVE_RAPTOR2_API
-#else
-  raptor_error_handlers* error_handlers;
-
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
-
-  if(!query->world)
-    return;
-
-  error_handlers = &query->world->error_handlers;
-  
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].handler   = handler;
-#endif
-}
-
-
-/**
- * rasqal_query_set_error_handler:
- * @query: the query
- * @user_data: user data to pass to function
- * @handler: pointer to the function
- *
- * Set the query error handling function.
- * 
- * The function will receive callbacks when the query fails.
- * 
- **/
-void
-rasqal_query_set_error_handler(rasqal_query* query, void *user_data,
-                               raptor_message_handler handler)
-{
-#ifdef HAVE_RAPTOR2_API
-#else
-  raptor_error_handlers* error_handlers;
-
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
-
-  if(!query->world)
-    return;
-
-  error_handlers = &query->world->error_handlers;
-  
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler   = handler;
-#endif
-}
-
-
-/**
- * rasqal_query_set_warning_handler:
- * @query: the query
- * @user_data: user data to pass to function
- * @handler: pointer to the function
- *
- * Set the query warning handling function.
- * 
- * The function will receive callbacks when the query gives a warning.
- * 
- **/
-void
-rasqal_query_set_warning_handler(rasqal_query* query, void *user_data,
-                                 raptor_message_handler handler)
-{
-#ifdef HAVE_RAPTOR2_API
-#else
-  raptor_error_handlers* error_handlers;
-
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
-
-  if(!query->world)
-    return;
-
-  error_handlers = &query->world->error_handlers;
-  
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].handler   = handler;
-#endif
+  return query->factory->desc.label;
 }
 
 
@@ -427,6 +312,11 @@ rasqal_query_set_feature(rasqal_query* query, rasqal_feature feature, int value)
 
   switch(feature) {
     case RASQAL_FEATURE_NO_NET:
+    case RASQAL_FEATURE_RAND_SEED:
+
+      if(feature == RASQAL_FEATURE_RAND_SEED)
+        query->user_set_rand = 1;
+      
       query->features[(int)feature] = value;
       break;
       
@@ -489,6 +379,7 @@ rasqal_query_get_feature(rasqal_query *query, rasqal_feature feature)
 
   switch(feature) {
     case RASQAL_FEATURE_NO_NET:
+    case RASQAL_FEATURE_RAND_SEED:
       result = (query->features[(int)feature] != 0);
       break;
 
@@ -620,7 +511,10 @@ rasqal_query_get_limit(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
 
-  return query->limit;
+  if(query->modifier)
+    return query->modifier->limit;
+  else
+    return -1;
 }
 
 
@@ -631,14 +525,17 @@ rasqal_query_get_limit(rasqal_query* query)
  *
  * Set the query-specified limit on results.
  *
- * This is the limit given in the query on the number of results allowed.
+ * This is the limit given in the query on the number of results
+ * allowed.  It is only guaranteed to work after the query is
+ * prepared and before it is executed.
  **/
 void
 rasqal_query_set_limit(rasqal_query* query, int limit)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
 
-  query->limit = limit;
+  if(query->modifier)
+    query->modifier->limit = limit;
 }
 
 
@@ -648,7 +545,9 @@ rasqal_query_set_limit(rasqal_query* query, int limit)
  *
  * Get the query-specified offset on results.
  *
- * This is the offset given in the query on the number of results allowed.
+ * This is the offset given in the query on the number of results
+ * allowed.  It is only guaranteed to work after the query is
+ * prepared and before it is executed.
  *
  * Return value: integer >=0 if a offset is given, otherwise <0
  **/
@@ -657,7 +556,10 @@ rasqal_query_get_offset(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
 
-  return query->offset;
+  if(query->modifier)
+    return query->modifier->offset;
+  else
+    return -1;
 }
 
 
@@ -675,47 +577,60 @@ rasqal_query_set_offset(rasqal_query* query, int offset)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
 
-  query->offset = offset;
+  if(query->modifier)
+    query->modifier->offset = offset;
 }
 
 
 /**
  * rasqal_query_add_data_graph:
  * @query: #rasqal_query query object
- * @uri: #raptor_uri source uri for retrieval
- * @name_uri: #raptor_uri name uri (or NULL)
- * @flags: RASQAL_DATA_GRAPH_NAMED or RASQAL_DATA_GRAPH_BACKGROUND
+ * @data_graph: data graph
  *
  * Add a data graph to the query.
  *
- * named_uri must be given if flags RASQAL_DATA_GRAPH_NAMED is set.
- * It is the name of the graph and also used as the base URI
- * when resolving any relative URIs for the graph in uri.
- *
  * Return value: non-0 on failure
  **/
 int
-rasqal_query_add_data_graph(rasqal_query* query, 
-                            raptor_uri* uri, raptor_uri* name_uri,
-                            int flags)
+rasqal_query_add_data_graph(rasqal_query* query, rasqal_data_graph* data_graph)
 {
-  rasqal_data_graph *dg;
-
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, raptor_uri, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(data_graph, rasqal_data_graph, 1);
 
-  if((flags & RASQAL_DATA_GRAPH_NAMED) && !name_uri)
-    return 1;
-  
-  dg = rasqal_new_data_graph(query->world, uri, name_uri, flags);
-  if(!dg)
-    return 1;
-  if(raptor_sequence_push(query->data_graphs, (void*)dg))
+  if(raptor_sequence_push(query->data_graphs, (void*)data_graph))
     return 1;
   return 0;
 }
 
 
+/**
+ * rasqal_query_add_data_graphs:
+ * @query: #rasqal_query query object
+ * @data_graphs: sequence of #rasqal_data_graph
+ *
+ * Add a set of data graphs to the query.
+ *
+ * The objects in the passed-in @data_graphs sequence becomes owne by the query.
+ * The @data_graphs sequence itself is freed and must not be used after this call.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_query_add_data_graphs(rasqal_query* query,
+                             raptor_sequence* data_graphs)
+{
+  int rc;
+  
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(data_graphs, raptor_sequence, 1);
+
+  rc = raptor_sequence_join(query->data_graphs, data_graphs);
+  raptor_free_sequence(data_graphs);
+
+  return rc;
+}
+
+
 /**
  * rasqal_query_get_data_graph_sequence:
  * @query: #rasqal_query query object
@@ -792,7 +707,7 @@ rasqal_query_dataset_contains_named_graph(rasqal_query* query,
  *
  * Add a binding variable to the query.
  *
- * See also rasqal_query_set_variable which assigns or removes a value to
+ * See also rasqal_query_set_variable() which assigns or removes a value to
  * a previously added variable in the query.
  *
  * Return value: non-0 on failure
@@ -804,15 +719,13 @@ rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(var, rasqal_variable, 1);
 
   if(!query->selects) {
-#ifdef HAVE_RAPTOR2_API
-    query->selects = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
-#else
-    query->selects = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
-#endif
+    query->selects = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                         (raptor_data_print_handler)rasqal_variable_print);
     if(!query->selects)
       return 1;
   }
 
+  var = rasqal_new_variable_from_variable(var);
   return raptor_sequence_push(query->selects, (void*)var);
 }
 
@@ -1083,11 +996,7 @@ rasqal_query_add_prefix(rasqal_query* query, rasqal_prefix* prefix)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(prefix, rasqal_prefix, 1);
 
   if(!query->prefixes) {
-#ifdef HAVE_RAPTOR2_API
     query->prefixes = raptor_new_sequence((raptor_data_free_handler)rasqal_free_prefix, (raptor_data_print_handler)rasqal_prefix_print);
-#else
-    query->prefixes = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
-#endif
     if(!query->prefixes)
       return 1;
   } else {
@@ -1273,6 +1182,8 @@ rasqal_query_prepare(rasqal_query* query,
     return 0;
   query->prepared = 1;
 
+  query->store_results = 0;
+
   if(query_string) {
     /* flex lexers require two NULs at the end of the lexed buffer.
      * Add them here instead of parser to allow resource cleanup on error.
@@ -1292,9 +1203,9 @@ rasqal_query_prepare(rasqal_query* query,
       query->failed = 1;
       return 1;
     }
-    strcpy((char*)query_string_copy, (const char*)query_string);
-    query_string_copy[len-3] = ' ';
-    query_string_copy[len-2] = query_string_copy[len-1] = '\0';
+    memcpy(query_string_copy, query_string, len - 3);
+    query_string_copy[len - 3] = ' ';
+    query_string_copy[len - 2] = query_string_copy[len - 1] = '\0';
     query->query_string = query_string_copy;
     query->query_string_length = len;
   }
@@ -1311,6 +1222,24 @@ rasqal_query_prepare(rasqal_query* query,
   rasqal_query_set_base_uri(query, base_uri);
   query->locator.line = query->locator.column = query->locator.byte = -1;
 
+  /* set evaluaton context with latest copies of query fields */
+  query->eval_context->flags = query->compare_flags;
+  rasqal_evaluation_context_set_base_uri(query->eval_context, query->base_uri);
+
+  /* set random seed */
+  if(1) {
+    unsigned int seed;
+
+    /* get seed either from user or system sources */
+    if(query->user_set_rand)
+      seed = (unsigned int)query->features[(int)RASQAL_FEATURE_RAND_SEED];
+    else
+      seed = rasqal_random_get_system_seed(query->world);
+    
+    rasqal_evaluation_context_set_rand_seed(query->eval_context, seed);
+  }
+  
+
   rc = query->factory->prepare(query);
   if(rc) {
     query->failed = 1;
@@ -1378,16 +1307,32 @@ rasqal_query_execute_with_engine(rasqal_query* query,
                                  const rasqal_query_execution_factory* engine)
 {
   rasqal_query_results *query_results = NULL;
+  rasqal_query_results_type type;
   
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
   if(query->failed)
     return NULL;
 
+  type = rasqal_query_get_result_type(query);
+  if(type == RASQAL_QUERY_RESULTS_UNKNOWN)
+    return NULL;
+  
+  query_results = rasqal_new_query_results(query->world, query, type,
+                                           query->vars_table);
+  if(!query_results)
+    return NULL;
+
   if(!engine)
     engine = rasqal_query_get_engine_by_name(NULL);
 
-  query_results = rasqal_query_results_execute_with_engine(query, engine);
+  if(rasqal_query_results_execute_with_engine(query_results, engine,
+                                              query->store_results)) {
+    rasqal_free_query_results(query_results);
+    query_results = NULL;      
+  }
+    
+      
   if(query_results && rasqal_query_add_query_result(query, query_results)) {
     rasqal_free_query_results(query_results);
     query_results = NULL;      
@@ -1469,11 +1414,14 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
             (query->distinct == 1 ? "distinct" : "reduced"));
   if(query->explain)
     fputs("query results explain: yes\n", fh);
-  if(query->limit >= 0)
-    fprintf(fh, "query results limit: %d\n", query->limit);
-  if(query->offset >= 0)
-    fprintf(fh, "query results offset: %d\n", query->offset);
 
+  if(query->modifier) {
+    if(query->modifier->limit >= 0)
+      fprintf(fh, "query results limit: %d\n", query->modifier->limit);
+    if(query->modifier->offset >= 0)
+      fprintf(fh, "query results offset: %d\n", query->modifier->offset);
+  }
+  
   fputs("data graphs: ", fh);
   if(query->data_graphs)
     raptor_sequence_print(query->data_graphs, fh);
@@ -1515,22 +1463,30 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
     fputs("\nquery graph pattern: ", fh);
     rasqal_graph_pattern_print(query->query_graph_pattern, fh);
   }
-  if(query->order_conditions_sequence) {
-    fputs("\nquery order conditions: ", fh);
-    raptor_sequence_print(query->order_conditions_sequence, fh);
-  }
-  if(query->group_conditions_sequence) {
-    fputs("\nquery group conditions: ", fh);
-    raptor_sequence_print(query->group_conditions_sequence, fh);
-  }
-  if(query->having_conditions_sequence) {
-    fputs("\nquery having conditions: ", fh);
-    raptor_sequence_print(query->having_conditions_sequence, fh);
+
+  if(query->modifier) {
+    if(query->modifier->order_conditions) {
+      fputs("\nquery order conditions: ", fh);
+      raptor_sequence_print(query->modifier->order_conditions, fh);
+    }
+    if(query->modifier->group_conditions) {
+      fputs("\nquery group conditions: ", fh);
+      raptor_sequence_print(query->modifier->group_conditions, fh);
+    }
+    if(query->modifier->having_conditions) {
+      fputs("\nquery having conditions: ", fh);
+      raptor_sequence_print(query->modifier->having_conditions, fh);
+    }
   }
+  
   if(query->updates) {
     fputs("\nupdate operations: ", fh);
     raptor_sequence_print(query->updates, fh);
   }
+  if(query->bindings) {
+    fputs("\nbindings: ", fh);
+    rasqal_bindings_print(query->bindings, fh);
+  }
   fputc('\n', fh);
 
   return 0;
@@ -1589,7 +1545,7 @@ rasqal_query_remove_query_result(rasqal_query* query,
  *
  * Get query user data.
  * 
- * Return value: user data as set by rasqal_query_set_user_data
+ * Return value: user data as set by rasqal_query_set_user_data()
  **/
 void*
 rasqal_query_get_user_data(rasqal_query* query)
@@ -1664,7 +1620,10 @@ rasqal_query_get_order_conditions_sequence(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->order_conditions_sequence;
+  if(query->modifier)
+    return query->modifier->order_conditions;
+  else
+    return NULL;
 }
 
 
@@ -1682,10 +1641,10 @@ rasqal_query_get_order_condition(rasqal_query* query, int idx)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  if(!query->order_conditions_sequence)
+  if(!query->modifier || !query->modifier->order_conditions)
     return NULL;
   
-  return (rasqal_expression*)raptor_sequence_get_at(query->order_conditions_sequence, idx);
+  return (rasqal_expression*)raptor_sequence_get_at(query->modifier->order_conditions, idx);
 }
 
 
@@ -1702,7 +1661,10 @@ rasqal_query_get_group_conditions_sequence(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->group_conditions_sequence;
+  if(query->modifier)
+    return query->modifier->group_conditions;
+  else
+    return NULL;
 }
 
 
@@ -1720,10 +1682,10 @@ rasqal_query_get_group_condition(rasqal_query* query, int idx)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  if(!query->group_conditions_sequence)
+   if(!query->modifier || !query->modifier->group_conditions)
     return NULL;
   
-  return (rasqal_expression*)raptor_sequence_get_at(query->group_conditions_sequence, idx);
+  return (rasqal_expression*)raptor_sequence_get_at(query->modifier->group_conditions, idx);
 }
 
 
@@ -1740,7 +1702,10 @@ rasqal_query_get_having_conditions_sequence(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->having_conditions_sequence;
+  if(query->modifier)
+    return query->modifier->having_conditions;
+  else
+    return NULL;
 }
 
 
@@ -1758,10 +1723,10 @@ rasqal_query_get_having_condition(rasqal_query* query, int idx)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  if(!query->having_conditions_sequence)
+   if(!query->modifier || !query->modifier->having_conditions)
     return NULL;
   
-  return (rasqal_expression*)raptor_sequence_get_at(query->having_conditions_sequence, idx);
+  return (rasqal_expression*)raptor_sequence_get_at(query->modifier->having_conditions, idx);
 }
 
 
@@ -1889,14 +1854,9 @@ rasqal_query_escape_counted_string(rasqal_query* query,
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, char*, NULL);
 
-#ifdef HAVE_RAPTOR2_API
   iostr = raptor_new_iostream_to_string(query->world->raptor_world_ptr,
                                         &output_string, output_len_p,
                                         rasqal_alloc_memory);
-#else
-  iostr = raptor_new_iostream_to_string(&output_string, output_len_p,
-                                        rasqal_alloc_memory);
-#endif
   if(!iostr)
     return NULL;
   rc = rasqal_query_iostream_write_escaped_counted_string(query, iostr,
@@ -1952,12 +1912,34 @@ rasqal_query_set_base_uri(rasqal_query* query, raptor_uri* base_uri)
 }
 
 
-void
+/**
+ * rasqal_query_set_store_results:
+ * @query: the #rasqal_query object
+ * @store_results: store results flag
+ *
+ * Request that query results are stored during execution
+ * 
+ * When called after a rasqal_query_prepare(), this tells
+ * rasqal_query_execute() to execute the entire query immediately
+ * rather than generate them lazily, and store all the results in
+ * memory.  The results will then be available for reading multiple
+ * times using rasqal_query_results_rewind() to move back to the
+ * start of the result object.  If called after preparation, returns
+ * failure.
+ *
+ * Return value: non-0 on failure.
+ **/
+int
 rasqal_query_set_store_results(rasqal_query* query, int store_results)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
+  if(!query->prepared)
+    return 1;
 
   query->store_results = store_results;
+
+  return 0;
 }
 
 
@@ -2009,11 +1991,7 @@ rasqal_query_add_update_operation(rasqal_query* query,
     return 1;
 
   if(!query->updates) {
-#ifdef HAVE_RAPTOR2_API
     query->updates = raptor_new_sequence((raptor_data_free_handler)rasqal_free_update_operation, (raptor_data_print_handler)rasqal_update_operation_print);
-#else
-    query->updates = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_update_operation, (raptor_sequence_print_handler*)rasqal_update_operation_print);
-#endif
     if(!query->updates) {
       rasqal_free_update_operation(update);
       return 1;
@@ -2064,85 +2042,6 @@ rasqal_query_get_update_operation(rasqal_query* query, int idx)
 }
 
 
-/**
- * rasqal_query_set_default_generate_bnodeid_parameters:
- * @rdf_query: #rasqal_query object
- * @prefix: prefix string
- * @base: integer base identifier
- *
- * Set default bnodeid generation parameters
- *
- * Sets the parameters for the default algorithm used to generate
- * blank node IDs.  The default algorithm uses both @prefix and @base
- * to generate a new identifier.  The exact identifier generated is
- * not guaranteed to be a strict concatenation of @prefix and @base
- * but will use both parts.
- *
- * For finer control of the generated identifiers, use
- * rasqal_world_set_generate_bnodeid_handler() on the world class (or
- * the deprecated rasqal_query_set_generate_bnodeid_handler() on the
- * query class).
- *
- * If prefix is NULL, the default prefix is used (currently "bnodeid")
- * If base is less than 1, it is initialised to 1.
- * 
- * This calls rasqal_world_set_default_generate_bnodeid_parameters()
- * method on the world class; there is no separate configuration per
- * query object.
- *  
- * @Deprecated: Use
- * rasqal_world_set_default_generate_bnodeid_parameters() on the
- * world class.
- *
- **/
-void
-rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, 
-                                                     char *prefix, int base)
-{
-
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rdf_query, rasqal_query);
-
-  rasqal_world_set_default_generate_bnodeid_parameters(rdf_query->world, 
-                                                       prefix, base);
-}
-
-
-/**
- * rasqal_query_set_generate_bnodeid_handler:
- * @query: #rasqal_query query object
- * @user_data: user data pointer for callback
- * @handler: generate blank ID callback function
- *
- * Set the generate blank node ID handler function for the query.
- *
- * Sets the function to generate blank node IDs for the query.
- * The handler is called with a pointer to the rasqal_query, the
- * @user_data pointer and a user_bnodeid which is the value of
- * a user-provided blank node identifier (may be NULL).
- * It can either be returned directly as the generated value when present or
- * modified.  The passed in value must be free()d if it is not used.
- *
- * If handler is NULL, the default method is used
- *
- * Any hander set by rasqal_world_set_generate_bnodeid_handler()
- * overrides this.
- *  
- * @Deprecated: Use
- * rasqal_world_set_generate_bnodeid_handler() on the world class.
- * 
- **/
-void
-rasqal_query_set_generate_bnodeid_handler(rasqal_query* query,
-                                          void *user_data,
-                                          rasqal_generate_bnodeid_handler handler)
-{
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
-
-  query->generate_bnodeid_handler_user_data = user_data;
-  query->generate_bnodeid_handler = handler;
-}
-
-
 /*
  * rasqal_query_generate_bnodeid - Default generate id - internal
  */
@@ -2154,10 +2053,158 @@ rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
   if(rdf_query->world->generate_bnodeid_handler)
     return rasqal_world_generate_bnodeid(rdf_query->world, user_bnodeid);
 
-  if(rdf_query->generate_bnodeid_handler)
-    return rdf_query->generate_bnodeid_handler(rdf_query, 
-                                               rdf_query->generate_bnodeid_handler_user_data, user_bnodeid);
-  else
     return rasqal_world_default_generate_bnodeid_handler(rdf_query->world,
                                                          user_bnodeid);
 }
+
+
+/**
+ * rasqal_query_get_bindings_variables_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of BINDINGS block variables
+ *
+ * Return value: a #raptor_sequence of #raptor_variable pointers
+ **/
+raptor_sequence*
+rasqal_query_get_bindings_variables_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(query->bindings)
+    return query->bindings->variables;
+  else
+    return NULL;
+  
+}
+
+
+/**
+ * rasqal_query_get_bindings_variable:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a rasqal_variable* in the sequence of BINDINGS block variables
+ *
+ * Return value: a #raptor_sequence of #raptor_variable pointers
+ **/
+rasqal_variable*
+rasqal_query_get_bindings_variable(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(!query->bindings || !query->bindings->variables)
+    return NULL;
+  
+  return (rasqal_variable*)raptor_sequence_get_at(query->bindings->variables, idx);
+}
+
+
+/**
+ * rasqal_query_get_bindings_rows_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of BINDINGS block result rows
+ *
+ * Return value: a #raptor_sequence of #raptor_row pointers
+ **/
+raptor_sequence*
+rasqal_query_get_bindings_rows_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(query->bindings)
+    return query->bindings->rows;
+  else
+    return NULL;
+  
+}
+
+
+/**
+ * rasqal_query_get_bindings_row:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a rasqal_row* in the sequence of BINDINGS block result rows
+ *
+ * Return value: a #raptor_sequence of #raptor_row pointers
+ **/
+rasqal_row*
+rasqal_query_get_bindings_row(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(!query->bindings || !query->bindings->rows)
+    return NULL;
+  
+  return (rasqal_row*)raptor_sequence_get_at(query->bindings->rows, idx);
+}
+
+
+/*
+ * rasqal_query_get_bound_in_column_for_variable:
+ * @query: #rasqal_query query object
+ * @variable: variable
+ * 
+ * INTERNAL - Get the column number [triple] that a variable is bound in
+ *
+ * Return value: column or < 0 if not bound
+ */
+int
+rasqal_query_get_bound_in_column_for_variable(rasqal_query* query,
+                                              rasqal_variable* v)
+{
+  return query->variables_bound_in[v->offset];
+}
+
+
+/**
+ * rasqal_query_get_result_type:
+ * @query: #rasqal_query query object
+ *
+ * Get the result type expected from executing the query.
+ *
+ * This function is only valid after rasqal_query_prepare() has been
+ * run on the query and will return #RASQAL_QUERY_RESULTS_UNKNOWN if
+ * called before preparation.
+ *
+ * Return value: result type or #RASQAL_QUERY_RESULTS_UNKNOWN if not known or on error
+ **/
+rasqal_query_results_type
+rasqal_query_get_result_type(rasqal_query* query)
+{
+  rasqal_query_results_type type = RASQAL_QUERY_RESULTS_BINDINGS;
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, RASQAL_QUERY_RESULTS_UNKNOWN);
+
+  if(!query->prepared)
+    return RASQAL_QUERY_RESULTS_UNKNOWN;
+
+  if(query->query_results_formatter_name)
+    type = RASQAL_QUERY_RESULTS_SYNTAX;
+  else
+    switch(query->verb) {
+      case RASQAL_QUERY_VERB_SELECT:
+        type = RASQAL_QUERY_RESULTS_BINDINGS;
+        break;
+
+      case RASQAL_QUERY_VERB_ASK:
+        type = RASQAL_QUERY_RESULTS_BOOLEAN;
+        break;
+
+      case RASQAL_QUERY_VERB_CONSTRUCT:
+      case RASQAL_QUERY_VERB_DESCRIBE:
+        type = RASQAL_QUERY_RESULTS_GRAPH;
+        break;
+        
+      case RASQAL_QUERY_VERB_UNKNOWN:
+      case RASQAL_QUERY_VERB_DELETE:
+      case RASQAL_QUERY_VERB_INSERT:
+      case RASQAL_QUERY_VERB_UPDATE:
+      default:
+        type = RASQAL_QUERY_RESULTS_UNKNOWN;
+    }
+
+  return type;
+}
