@@ -2,7 +2,7 @@
  *
  * rasqal_query_transform.c - Rasqal query transformations
  *
- * Copyright (C) 2004-2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2004-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -53,6 +53,7 @@ static int rasqal_query_build_variables_use_map(rasqal_query* query);
 static void rasqal_query_graph_build_variables_use_map_in_internal(rasqal_query* query, short *use_map, rasqal_literal *origin);
 static void rasqal_query_expression_build_variables_use_map(short *use_map, rasqal_expression* e);
 static void rasqal_query_let_build_variables_use_map_in_internal(rasqal_query* query, short *use_map, rasqal_variable *var, rasqal_expression* e);
+static void rasqal_query_select_build_variables_use_map_in_internal(rasqal_query* query, short *use_map);
 
 
 int
@@ -174,13 +175,13 @@ rasqal_query_expand_query_constraints_qnames(rasqal_query *rq)
 
 static int
 rasqal_query_convert_blank_node_to_anonymous_variable(rasqal_query *rq,
-                                                       rasqal_literal *l)
+                                                      rasqal_literal *l)
 {
   rasqal_variable* v;
   
-  v = rasqal_new_variable_typed(rq, 
-                                RASQAL_VARIABLE_TYPE_ANONYMOUS,
-                                (unsigned char*)l->string, NULL);
+  v = rasqal_variables_table_add(rq->vars_table,
+                                 RASQAL_VARIABLE_TYPE_ANONYMOUS,
+                                 (unsigned char*)l->string, NULL);
   /* rasqal_new_variable_typed took ownership of the l->string name.
    * Set to NULL to prevent double delete. */
   l->string = NULL;
@@ -216,12 +217,15 @@ rasqal_query_build_anonymous_variables(rasqal_query* rq)
   
   for(i = 0; i < raptor_sequence_size(s); i++) {
     rasqal_triple* t = (rasqal_triple*)raptor_sequence_get_at(s, i);
+
     if(t->subject->type == RASQAL_LITERAL_BLANK &&
        rasqal_query_convert_blank_node_to_anonymous_variable(rq, t->subject))
       goto done;
+
     if(t->predicate->type == RASQAL_LITERAL_BLANK &&
        rasqal_query_convert_blank_node_to_anonymous_variable(rq, t->predicate))
       goto done;
+
     if(t->object->type == RASQAL_LITERAL_BLANK &&
        rasqal_query_convert_blank_node_to_anonymous_variable(rq, t->object))
       goto done;
@@ -252,17 +256,16 @@ rasqal_query_expand_wildcards(rasqal_query* rq)
     return 0;
   
   /* If 'SELECT *' was given, make the selects be a list of all variables */
-#ifdef HAVE_RAPTOR2_API
-  rq->selects = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
-#else
-  rq->selects = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
-#endif
+  rq->selects = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                    (raptor_data_print_handler)rasqal_variable_print);
   if(!rq->selects)
     return 1;
   
   size = rasqal_variables_table_get_named_variables_count(rq->vars_table);
   for(i = 0; i < size; i++) {
     rasqal_variable* v = rasqal_variables_table_get(rq->vars_table, i);
+
+    v = rasqal_new_variable_from_variable(v);
     if(raptor_sequence_push(rq->selects, v))
       return 1;
   }
@@ -299,11 +302,8 @@ rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
   if(!size)
     return 0;
 
-#ifdef HAVE_RAPTOR2_API
-  new_seq = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
-#else
-  new_seq = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
-#endif
+  new_seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                (raptor_data_print_handler)rasqal_variable_print);
   if(!new_seq)
     return 1;
   
@@ -338,6 +338,7 @@ rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
       }
     }
     if(!warned) {
+      v = rasqal_new_variable_from_variable(v);
       raptor_sequence_push(new_seq, v);
       modified = 1;
     }
@@ -384,29 +385,32 @@ rasqal_query_triples_build_bound_in_internal(rasqal_query* query,
     t = (rasqal_triple*)raptor_sequence_get_at(query->triples, col);
 
     if((v = rasqal_literal_as_variable(t->subject))) {
-      if(bound_in[v->offset] < 0) {
+      if(bound_in[v->offset] == BOUND_IN_UNBOUND) {
         RASQAL_DEBUG4("Triple %d binds subject to variable %s (var #%d)\n",
                       col, v->name, v->offset);
         bound_in[v->offset] = col;
       }
     }
+
     if((v = rasqal_literal_as_variable(t->predicate))) {
-      if(bound_in[v->offset] < 0) {
+      if(bound_in[v->offset] == BOUND_IN_UNBOUND) {
         RASQAL_DEBUG4("Triple %d binds predicate to variable %s (var #%d)\n",
                       col, v->name, v->offset);
         bound_in[v->offset] = col;
       }
     }
+
     if((v = rasqal_literal_as_variable(t->object))) {
-      if(bound_in[v->offset] < 0) {
+      if(bound_in[v->offset] == BOUND_IN_UNBOUND) {
         RASQAL_DEBUG4("Triple %d binds object to variable %s (var #%d)\n",
                       col, v->name, v->offset);
         bound_in[v->offset] = col;
       }
     }
+
     if(t->origin) {
       if((v = rasqal_literal_as_variable(t->origin))) {
-        if(bound_in[v->offset] < 0) {
+        if(bound_in[v->offset] == BOUND_IN_UNBOUND) {
           RASQAL_DEBUG4("Triple %d binds graph to variable %s (var #%d)\n",
                         col, v->name, v->offset);
           bound_in[v->offset] = col;
@@ -471,6 +475,7 @@ rasqal_query_graph_pattern_build_bound_in(rasqal_query* query,
 
     for(i = 0; i < raptor_sequence_size(gp->graph_patterns); i++) {
       rasqal_graph_pattern *sgp;
+
       sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
       if(rasqal_query_graph_pattern_build_bound_in(query, bound_in, sgp))
         return 1;
@@ -479,6 +484,7 @@ rasqal_query_graph_pattern_build_bound_in(rasqal_query* query,
 
   if(gp->op == RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH && gp->origin) {
     rasqal_variable* v;
+
     v = rasqal_literal_as_variable(gp->origin);
     if(v)
       bound_in[v->offset] = BOUND_IN_ELSEWHERE;
@@ -513,6 +519,9 @@ rasqal_query_build_bound_in(rasqal_query* query)
   int size;
 
   size = rasqal_variables_table_get_total_variables_count(query->vars_table);
+
+  if(query->variables_bound_in)
+    RASQAL_FREE(intarray, query->variables_bound_in);
   
   query->variables_bound_in = (int*)RASQAL_CALLOC(intarray, size+1, sizeof(int));
   if(!query->variables_bound_in)
@@ -524,7 +533,9 @@ rasqal_query_build_bound_in(rasqal_query* query)
   size = rasqal_variables_table_get_named_variables_count(query->vars_table);
   for(i = 0; i < size; i++) {
     rasqal_variable *v;
+
     v = rasqal_variables_table_get(query->vars_table, i);
+
     if(v->expression)
       query->variables_bound_in[v->offset] = BOUND_IN_ELSEWHERE;
   }
@@ -641,6 +652,7 @@ rasqal_query_merge_triple_patterns(rasqal_query* query,
     /* find first basic graph pattern starting at offset */
     for(i= offset; i < size; i++) {
       rasqal_graph_pattern *sgp;
+
       sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
 
       if(sgp->op == RASQAL_GRAPH_PATTERN_OPERATOR_BASIC) {
@@ -661,12 +673,15 @@ rasqal_query_merge_triple_patterns(rasqal_query* query,
     dest_bgp = NULL; /* destination graph pattern */
     for(j = i; j < size; j++) {
       rasqal_graph_pattern *sgp;
+
       sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, j);
 
       if(sgp->op == RASQAL_GRAPH_PATTERN_OPERATOR_BASIC) {
         bgp_count++;
+
         if(!dest_bgp)
           dest_bgp = sgp;
+
         last = j;
       } else
         break;
@@ -684,16 +699,16 @@ rasqal_query_merge_triple_patterns(rasqal_query* query,
 
     RASQAL_DEBUG3("Initial columns %d to %d\n", gp->start_column, gp->end_column);
   #endif
-#ifdef HAVE_RAPTOR2_API
+
     seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
-#else
-    seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-#endif
     if(!seq)
       return 1;
+
     for(i = 0; raptor_sequence_size(gp->graph_patterns) > 0; i++) {
       rasqal_graph_pattern *sgp;
+
       sgp = (rasqal_graph_pattern*)raptor_sequence_unshift(gp->graph_patterns);
+
       if(i >= first && i <= last) {
         if(sgp != dest_bgp) {
           if(rasqal_graph_patterns_join(dest_bgp, sgp)) {
@@ -751,6 +766,7 @@ rasqal_graph_pattern_move_constraints(rasqal_graph_pattern* dest_gp,
   fs = dest_gp->filter_expression;
   if(fs)
     e = rasqal_new_2op_expression(e->world, RASQAL_EXPR_AND, fs, e);
+
   dest_gp->filter_expression = e;
 
   return rc;
@@ -788,7 +804,9 @@ rasqal_query_remove_empty_group_graph_patterns(rasqal_query* query,
 
   for(i = 0; i < raptor_sequence_size(gp->graph_patterns); i++) {
     rasqal_graph_pattern *sgp;
+
     sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+
     if(sgp->graph_patterns && !raptor_sequence_size(sgp->graph_patterns)) {
       /* One is enough to know we need to rewrite */
       saw_empty_gp = 1;
@@ -803,24 +821,24 @@ rasqal_query_remove_empty_group_graph_patterns(rasqal_query* query,
     return 0;
   }
   
-#ifdef HAVE_RAPTOR2_API
   seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
-#else
-  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-#endif
   if(!seq) {
     RASQAL_DEBUG1("Cannot create new gp sequence\n");
     *modified = -1;
     return 1;
   }
+
   while(raptor_sequence_size(gp->graph_patterns) > 0) {
     rasqal_graph_pattern *sgp;
+
     sgp = (rasqal_graph_pattern*)raptor_sequence_unshift(gp->graph_patterns);
+
     if(sgp->graph_patterns && !raptor_sequence_size(sgp->graph_patterns)) {
       rasqal_graph_pattern_move_constraints(gp, sgp);
       rasqal_free_graph_pattern(sgp);
       continue;
     }
+
     raptor_sequence_push(seq, sgp);
   }
   raptor_free_sequence(gp->graph_patterns);
@@ -899,9 +917,12 @@ rasqal_query_merge_graph_patterns(rasqal_query* query,
 #endif
   op = RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN;
   all_gp_op_same = 1;
+
   for(i = 0; i < size; i++) {
     rasqal_graph_pattern *sgp;
+
     sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+
     if(op == RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN) {
       op = sgp->op;
     } else {
@@ -915,6 +936,7 @@ rasqal_query_merge_graph_patterns(rasqal_query* query,
       }
     }
   }
+
 #if RASQAL_DEBUG > 1
   RASQAL_DEBUG2("Sub-graph patterns of %d done\n", gp->gp_index);
 #endif
@@ -937,6 +959,7 @@ rasqal_query_merge_graph_patterns(rasqal_query* query,
    */
   for(i = 0; i < size; i++) {
     rasqal_graph_pattern *sgp;
+
     sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
     
     if(sgp->op != RASQAL_GRAPH_PATTERN_OPERATOR_BASIC) {
@@ -989,6 +1012,7 @@ rasqal_query_merge_graph_patterns(rasqal_query* query,
     
     while(raptor_sequence_size(seq) > 0) {
       rasqal_graph_pattern *sgp;
+
       sgp = (rasqal_graph_pattern*)raptor_sequence_unshift(seq);
 
       /* fake this so that the join happens */
@@ -997,6 +1021,7 @@ rasqal_query_merge_graph_patterns(rasqal_query* query,
         RASQAL_DEBUG1("Cannot join graph patterns\n");
         *modified = -1; /* error flag */
       }
+
       rasqal_free_graph_pattern(sgp);
     }
 
@@ -1041,7 +1066,8 @@ rasqal_expression_foreach_fold(void *user_data, rasqal_expression *e)
   struct folding_state *st = (struct folding_state*)user_data;
   rasqal_query* query;  
   rasqal_literal* l;
-
+  int error = 0;
+  
   /* skip if already a  literal or this expression tree is not constant */
   if(e->op == RASQAL_EXPR_LITERAL || !rasqal_expression_is_constant(e))
     return 0;
@@ -1053,9 +1079,8 @@ rasqal_expression_foreach_fold(void *user_data, rasqal_expression *e)
 #endif
   
   query = st->query;
-  l = rasqal_expression_evaluate(query->world, &query->locator,
-                                 e, query->compare_flags);
-  if(!l) {
+  l = rasqal_expression_evaluate2(e, query->eval_context, &error);
+  if(error) {
     st->failed++;
     return 1;
   }
@@ -1108,7 +1133,9 @@ rasqal_graph_pattern_fold_expressions(rasqal_query* rq,
     
     for(i = 0; i < raptor_sequence_size(gp->graph_patterns); i++) {
       rasqal_graph_pattern *sgp;
+
       sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+
       if(rasqal_graph_pattern_fold_expressions(rq, sgp))
         return 1;
     }
@@ -1126,20 +1153,22 @@ rasqal_query_fold_expressions(rasqal_query* rq)
 {
   rasqal_graph_pattern *gp = rq->query_graph_pattern;
   int order_size;
+  raptor_sequence *order_seq = rasqal_query_get_order_conditions_sequence(rq);
 
   if(gp)
     rasqal_graph_pattern_fold_expressions(rq, gp);
 
-  if(!rq->order_conditions_sequence)
+  if(!order_seq)
     return 0;
   
-  order_size = raptor_sequence_size(rq->order_conditions_sequence);
+  order_size = raptor_sequence_size(order_seq);
   if(order_size) {
     int i;
     
     for(i = 0; i < order_size; i++) {
       rasqal_expression* e;
-      e = (rasqal_expression*)raptor_sequence_get_at(rq->order_conditions_sequence, i);
+
+      e = (rasqal_expression*)raptor_sequence_get_at(order_seq, i);
       rasqal_query_expression_fold(rq, e);
     }
   }
@@ -1164,6 +1193,43 @@ rasqal_query_prepare_count_graph_patterns(rasqal_query* query,
 }
 
 
+/**
+ * rasqal_query_build_variables_use:
+ * @query: query
+ *
+ * INTERNAL - build structures recording variable use in the query
+ *
+ * Should be called if the query variable usage is modified such as
+ * a variable added during query planning.
+ *
+ */
+int
+rasqal_query_build_variables_use(rasqal_query* query)
+{
+  int rc;
+  
+  /* create query->variables_use_map that marks where a variable is 
+   * mentioned, bound or used in a graph pattern.
+   */
+  rc = rasqal_query_build_variables_use_map(query); 
+  if(rc)
+    return rc;
+  
+  /* create query->variables_bound_in to find triples where a variable
+   * is bound and look for variables selected that are not used
+   * in the execution order (graph pattern tree walk order).
+   *
+   * The query->variables_bound_in array is used in
+   * rasqal_engine_graph_pattern_init() when trying to figure out
+   * which parts of a triple pattern need to bind to a variable:
+   * only the first reference to it.
+   */
+  rc = rasqal_query_build_bound_in(query);
+  
+  return rc;
+}
+
+
 /**
  * rasqal_query_prepare_common:
  * @query: query
@@ -1243,7 +1309,7 @@ rasqal_query_prepare_common(rasqal_query *query)
       fputs("\n", DEBUG_FH);
 #endif
 
-    } while(modified>0);
+    } while(modified > 0);
 
     rc = modified; /* error if modified<0, success if modified==0 */
     if(rc)
@@ -1269,26 +1335,9 @@ rasqal_query_prepare_common(rasqal_query *query)
                                      rasqal_query_prepare_count_graph_patterns,
                                      query->graph_patterns_sequence);
 
-    /* create query->variables_use_map that marks where a variable is 
-     * mentioned, bound or used in a graph pattern.
-     */
-    rc = rasqal_query_build_variables_use_map(query); 
-    if(rc)
+    if(rasqal_query_build_variables_use(query))
       goto done;
-
-    /* create query->variables_bound_in to find triples where a variable
-     * is bound and look for variables selected that are not used
-     * in the execution order (graph pattern tree walk order).
-     *
-     * The query->variables_bound_in array is used in
-     * rasqal_engine_graph_pattern_init() when trying to figure out
-     * which parts of a triple pattern need to bind to a variable:
-     * only the first reference to it.
-     */
-    rc = rasqal_query_build_bound_in(query);
-    if(rc)
-      goto done;
-
+    
     /* warn if any of the selected named variables are not in a triple */
     rc = rasqal_query_check_unused_variables(query, query->variables_bound_in);
     if(rc)
@@ -1341,13 +1390,8 @@ rasqal_graph_patterns_join(rasqal_graph_pattern *dest_gp,
 
   if(src_gp->graph_patterns) {
     if(!dest_gp->graph_patterns) {
-#ifdef HAVE_RAPTOR2_API
       dest_gp->graph_patterns = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern,
                                                     (raptor_data_print_handler)rasqal_graph_pattern_print);
-#else
-      dest_gp->graph_patterns = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern,
-                                                    (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-#endif
       if(!dest_gp->graph_patterns)
         return -1;
     }
@@ -1383,6 +1427,11 @@ rasqal_graph_patterns_join(rasqal_graph_pattern *dest_gp,
     src_gp->origin = NULL;
   }
 
+  if(src_gp->var) {
+    dest_gp->var = src_gp->var;
+    src_gp->var = NULL;
+  }
+
 #if RASQAL_DEBUG > 1
   RASQAL_DEBUG2("Result graph pattern #%d\n  ", dest_gp->gp_index);
   rasqal_graph_pattern_print(dest_gp, stdout);
@@ -1420,10 +1469,13 @@ rasqal_query_triples_build_variables_use_map_row(raptor_sequence *triples,
 
     if((v = rasqal_literal_as_variable(t->subject)))
       use_map_row[v->offset] |= RASQAL_VAR_USE_MENTIONED_HERE;
+
     if((v = rasqal_literal_as_variable(t->predicate)))
       use_map_row[v->offset] |= RASQAL_VAR_USE_MENTIONED_HERE;
+
     if((v = rasqal_literal_as_variable(t->object)))
       use_map_row[v->offset] |= RASQAL_VAR_USE_MENTIONED_HERE;
+
     if(t->origin) {
       if((v = rasqal_literal_as_variable(t->origin)))
         use_map_row[v->offset] |= RASQAL_VAR_USE_MENTIONED_HERE;
@@ -1494,9 +1546,16 @@ rasqal_query_graph_pattern_build_variables_use_map(rasqal_query* query,
                                                            gp->filter_expression);
       break;
       
+    case RASQAL_GRAPH_PATTERN_OPERATOR_SELECT:
+      rasqal_query_select_build_variables_use_map_in_internal(query, 
+                                                              &use_map[offset]);
+      break;
+      
     case RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL:
     case RASQAL_GRAPH_PATTERN_OPERATOR_UNION:
     case RASQAL_GRAPH_PATTERN_OPERATOR_GROUP:
+    case RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE:
+    case RASQAL_GRAPH_PATTERN_OPERATOR_MINUS:
     case RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN:
       break;
   }
@@ -1541,10 +1600,12 @@ rasqal_query_print_variables_use_map(FILE* fh, rasqal_query* query)
   
   fprintf(fh, "Query variables-use map (B=bound, M=mentioned, U=used):\n");
   fputs("GP#  Type      ", fh);
+
   for(i = 0; i < width; i++) {
     rasqal_variable* v = rasqal_variables_table_get(query->vars_table, i);
     fprintf(fh, "%-10s ", v->name);
   }
+
   fputc('\n', fh);
 
   for(row_index = 0; row_index < height; row_index++) {
@@ -1562,6 +1623,7 @@ rasqal_query_print_variables_use_map(FILE* fh, rasqal_query* query)
     
     for(i = 0; i < width; i++) {
       int flag_index = row[i];
+
       /* Turn unknown flags into "???" */
       if(flag_index > N_MAP_FLAGS_LABELS - 1)
         flag_index = N_MAP_FLAGS_LABELS;
@@ -1580,7 +1642,9 @@ rasqal_query_expression_build_variables_use_map_row(short *use_map_row,
 {
   if(e->literal) {
     rasqal_variable* v;
+
     v = rasqal_literal_as_variable(e->literal);
+
     if(v) 
       use_map_row[v->offset] |= RASQAL_VAR_USE_MENTIONED_HERE;
   }
@@ -1710,6 +1774,7 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   int rc = 0;
   short *use_map;
   raptor_sequence* seq;
+  short *use_map_row;
   
   width = rasqal_variables_table_get_total_variables_count(query->vars_table);
   height = RASQAL_VAR_USE_MAP_OFFSET_LAST + 1 + query->graph_pattern_count;
@@ -1718,19 +1783,24 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   if(!use_map)
     return 1;
 
+  if(query->variables_use_map)
+    RASQAL_FREE(intarray, query->variables_use_map);
+
   query->variables_use_map = use_map;
 
+  use_map_row = &use_map[RASQAL_VAR_USE_MAP_OFFSET_VERBS * width];
+  
   /* record variable use for 1) Query verbs */
   switch(query->verb) {
     case RASQAL_QUERY_VERB_SELECT:
       /* This also handles 1a) select/project expressions */
-      rc = rasqal_query_build_variables_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_VERBS],
+      rc = rasqal_query_build_variables_sequence_use_map_row(use_map_row,
                                                              query->selects);
       break;
   
     case RASQAL_QUERY_VERB_DESCRIBE:
       /* This is a list of rasqal_literal not rasqal_variable */
-      rc = rasqal_query_build_literals_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_VERBS],
+      rc = rasqal_query_build_literals_sequence_use_map_row(use_map_row,
                                                             query->describes);
       break;
       
@@ -1739,7 +1809,7 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
         int last_column = raptor_sequence_size(query->constructs)-1;
         
         rc = rasqal_query_triples_build_variables_use_map_row(query->constructs, 
-                                                              &use_map[RASQAL_VAR_USE_MAP_OFFSET_VERBS],
+                                                              use_map_row,
                                                               0,
                                                               last_column);
       }
@@ -1762,29 +1832,29 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   
 
   /* Record variable use for 2) GROUP BY expressions (SPARQL 1.1) */
-  seq = query->group_conditions_sequence;
+  seq = rasqal_query_get_group_conditions_sequence(query);
   if(seq) {
-    rc = rasqal_query_build_expressions_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_GROUP_BY],
-                                                             seq);
+    use_map_row = &use_map[RASQAL_VAR_USE_MAP_OFFSET_GROUP_BY * width];
+    rc = rasqal_query_build_expressions_sequence_use_map_row(use_map_row, seq);
     if(rc)
       goto done;
   }
   
 
   /* Record variable use for 3) HAVING expr (SPARQL 1.1) */
-  seq = query->having_conditions_sequence;
+  seq = rasqal_query_get_having_conditions_sequence(query);
   if(seq) {
-    rc = rasqal_query_build_expressions_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_HAVING],
-                                                             seq);
+    use_map_row = &use_map[RASQAL_VAR_USE_MAP_OFFSET_HAVING * width];
+    rc = rasqal_query_build_expressions_sequence_use_map_row(use_map_row, seq);
     if(rc)
       goto done;
   }
 
   /* record variable use for 4) ORDER list-of-expr (SPARQL 1.0) */
-  seq = query->order_conditions_sequence;
+  seq = rasqal_query_get_order_conditions_sequence(query);
   if(seq) {
-    rc = rasqal_query_build_expressions_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_ORDER_BY],
-                                                             seq);
+    use_map_row = &use_map[RASQAL_VAR_USE_MAP_OFFSET_ORDER_BY * width];
+    rc = rasqal_query_build_expressions_sequence_use_map_row(use_map_row, seq);
     if(rc)
       goto done;
   }
@@ -1869,3 +1939,21 @@ rasqal_query_let_build_variables_use_map_in_internal(rasqal_query* query,
                           use_map_row);
 }
 
+
+
+/**
+ * rasqal_query_select_build_variables_use_map_in_internal:
+ * @query: the #rasqal_query to find the variables in
+ * @use_map_row: 1D array of size num. variables to write
+ *
+ * INTERNAL - Mark variables mentioned in a sub-SELECT graph pattern
+ * 
+ **/
+static void
+rasqal_query_select_build_variables_use_map_in_internal(rasqal_query* query,
+                                                        short *use_map_row)
+{
+  rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR, NULL,
+                          "Evaluation of Sub SELECT is not implemented.");
+}
+
