@@ -84,11 +84,7 @@ rasqal_query_write_sparql_uri(sparql_writer_context *wc,
     const raptor_namespace* nspace = raptor_qname_get_namespace(qname);
     if(!raptor_namespace_get_prefix(nspace))
       raptor_iostream_write_byte(':', iostr);
-#ifdef HAVE_RAPTOR2_API
     raptor_qname_write(qname, iostr);
-#else
-    raptor_iostream_write_qname(iostr, qname);
-#endif
     raptor_free_qname(qname);
     return;
   }
@@ -267,7 +263,34 @@ static const char* const rasqal_sparql_op_labels[RASQAL_EXPR_LAST+1] = {
   "GROUP_CONCAT",
   "SAMPLE",
   "IN",
-  "NOT IN"
+  "NOT IN",
+  "isNUMERIC",
+  "YEAR",
+  "MONTH",
+  "DAY",
+  "HOURS",
+  "MINUTES",
+  "SECONDS",
+  "TIMEZONE",
+  "CURRENT_DATETIME",
+  "NOW",
+  "FROM_UNIXTIME",
+  "TO_UNIXTIME",
+  "CONCAT",
+  "STRLEN",
+  "SUBSTR",
+  "UCASE",
+  "LCASE",
+  "STRSTARTS",
+  "STRENDS",
+  "CONTAINS",
+  "ENCODE_FOR_URI",
+  "TZ",
+  "RAND",
+  "ABS",
+  "ROUND",
+  "CEIL",
+  "FLOOR"
 };
 
 
@@ -299,6 +322,13 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
   int count;
 
   switch(e->op) {
+    case RASQAL_EXPR_CURRENT_DATETIME:
+    case RASQAL_EXPR_NOW:
+    case RASQAL_EXPR_RAND:
+      rasqal_query_write_sparql_expression_op(wc, iostr, e);
+      raptor_iostream_counted_string_write("()", 2, iostr);
+      break;
+
     case RASQAL_EXPR_AND:
     case RASQAL_EXPR_OR:
     case RASQAL_EXPR_EQ:
@@ -316,6 +346,9 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_STR_NEQ:
     case RASQAL_EXPR_STRLANG:
     case RASQAL_EXPR_STRDT:
+    case RASQAL_EXPR_STRSTARTS:
+    case RASQAL_EXPR_STRENDS:
+    case RASQAL_EXPR_CONTAINS:
       raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
       raptor_iostream_write_byte(' ', iostr);
@@ -346,6 +379,25 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_IRI:
     case RASQAL_EXPR_BNODE:
     case RASQAL_EXPR_SAMPLE:
+    case RASQAL_EXPR_ISNUMERIC:
+    case RASQAL_EXPR_YEAR:
+    case RASQAL_EXPR_MONTH:
+    case RASQAL_EXPR_DAY:
+    case RASQAL_EXPR_HOURS:
+    case RASQAL_EXPR_MINUTES:
+    case RASQAL_EXPR_SECONDS:
+    case RASQAL_EXPR_TIMEZONE:
+    case RASQAL_EXPR_FROM_UNIXTIME:
+    case RASQAL_EXPR_TO_UNIXTIME:
+    case RASQAL_EXPR_STRLEN:
+    case RASQAL_EXPR_UCASE:
+    case RASQAL_EXPR_LCASE:
+    case RASQAL_EXPR_ENCODE_FOR_URI:
+    case RASQAL_EXPR_TZ:
+    case RASQAL_EXPR_ABS:
+    case RASQAL_EXPR_ROUND:
+    case RASQAL_EXPR_CEIL:
+    case RASQAL_EXPR_FLOOR:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
       raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
@@ -355,12 +407,14 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_REGEX:
     case RASQAL_EXPR_IF:
+    case RASQAL_EXPR_SUBSTR:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
       raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
       raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg2);
-      if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF) && e->arg3) {
+      if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF ||
+          e->op == RASQAL_EXPR_SUBSTR) && e->arg3) {
         raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_query_write_sparql_expression(wc, iostr, e->arg3);
       }
@@ -408,7 +462,9 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
       break;
       
     case RASQAL_EXPR_COALESCE:
-      raptor_iostream_counted_string_write("COALESCE( ", 10, iostr);
+    case RASQAL_EXPR_CONCAT:
+      rasqal_query_write_sparql_expression_op(wc, iostr, e);
+      raptor_iostream_counted_string_write("( ", 2, iostr);
       count = raptor_sequence_size(e->args);
       for(i = 0; i < count ; i++) {
         rasqal_expression* e2;
@@ -471,45 +527,60 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
 static void
 rasqal_query_write_sparql_triple_data(sparql_writer_context *wc,
                                       raptor_iostream* iostr,
-                                      raptor_sequence *triple_gps,
+                                      raptor_sequence *triples,
                                       int indent)
 {
-  int gp = 0;
+  int triple_index = 0;
   
-  for(gp = 0; 1; gp++) {
-    rasqal_graph_pattern* tgp;
-    int triple_index = 0;
+  raptor_iostream_counted_string_write("{\n", 2, iostr);
+  indent += 2;
   
-    tgp = (rasqal_graph_pattern*)raptor_sequence_get_at(triple_gps, gp);
-    if(!tgp)
+  /* look for triples */
+  while(1) {
+    rasqal_triple* t = raptor_sequence_get_at(triples, triple_index);
+    if(!t)
       break;
-
-    if(gp > 0)
-      raptor_iostream_write_byte('\n', iostr);
-
-    raptor_iostream_counted_string_write("{\n", 2, iostr);
-    indent += 2;
-    
-    /* look for triples */
-    while(1) {
-      rasqal_triple* t = rasqal_graph_pattern_get_triple(tgp, triple_index);
-      if(!t)
-        break;
-      
-      rasqal_query_write_indent(iostr, indent);
-      rasqal_query_write_sparql_triple(wc, iostr, t);
-      raptor_iostream_write_byte('\n', iostr);
-      
-      triple_index++;
-    }
-    
-    indent -= 2;
     
     rasqal_query_write_indent(iostr, indent);
-    raptor_iostream_write_byte('}', iostr);
+    if(t->origin) {
+      raptor_iostream_counted_string_write("GRAPH ", 6, iostr);
+      rasqal_query_write_sparql_literal(wc, iostr, t->origin);
+      raptor_iostream_counted_string_write(" { ", 3, iostr);
+    }
+    
+    rasqal_query_write_sparql_triple(wc, iostr, t);
 
+    if(t->origin)
+      raptor_iostream_counted_string_write(" }", 2, iostr);
+
+    raptor_iostream_write_byte('\n', iostr);
+    
+    triple_index++;
+  }
+  
+  indent -= 2;
+  
+  rasqal_query_write_indent(iostr, indent);
+  raptor_iostream_write_byte('}', iostr);
+  
+}
+
+
+static int
+rasqal_query_write_sparql_select(sparql_writer_context *wc,
+                                 raptor_iostream *iostr, 
+                                 raptor_sequence* vars_seq)
+{
+  int count = raptor_sequence_size(vars_seq);
+  int i;
+  
+  for(i = 0; i < count; i++) {
+    rasqal_variable* v = (rasqal_variable*)raptor_sequence_get_at(vars_seq, i);
+    raptor_iostream_write_byte(' ', iostr);
+    rasqal_query_write_sparql_variable(wc, iostr, v);
   }
 
+  return 0;
 }
 
 
@@ -526,7 +597,23 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
   int want_braces = 1;
   
   op = rasqal_graph_pattern_get_operator(gp);
-  
+
+  if(op == RASQAL_GRAPH_PATTERN_OPERATOR_SELECT) {
+    raptor_sequence* vars_seq;
+    rasqal_graph_pattern* where_gp;
+    
+    raptor_iostream_counted_string_write("SELECT", 6, iostr);
+    vars_seq = rasqal_projection_get_variables_sequence(gp->projection);
+    rasqal_query_write_sparql_select(wc, iostr, vars_seq);
+    raptor_iostream_write_byte('\n', iostr);
+    rasqal_query_write_indent(iostr, indent);
+    raptor_iostream_counted_string_write("WHERE ", 6, iostr);
+    where_gp = rasqal_graph_pattern_get_sub_graph_pattern(gp, 0);
+    rasqal_query_write_sparql_graph_pattern(wc, iostr, where_gp, 0, indent);
+    /* FIXME - not implemented: modifiers */
+    return;
+  }
+
   if(op == RASQAL_GRAPH_PATTERN_OPERATOR_LET) {
     /* LAQRS */
     raptor_iostream_counted_string_write("LET (", 5, iostr);
@@ -537,6 +624,18 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
     return;
   }
   
+  if(op == RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE) {
+    rasqal_graph_pattern* service_gp;
+
+    /* LAQRS */
+    raptor_iostream_counted_string_write("SERVICE ", 8, iostr);
+    rasqal_query_write_sparql_literal(wc, iostr, gp->origin);
+    raptor_iostream_counted_string_write(" ", 1, iostr);
+    service_gp = rasqal_graph_pattern_get_sub_graph_pattern(gp, 0);
+    rasqal_query_write_sparql_graph_pattern(wc, iostr, service_gp, 0, indent);
+    return;
+  }
+  
   if(op == RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL ||
      op == RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH) {
     /* prefix verbs */
@@ -576,7 +675,9 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
   seq = rasqal_graph_pattern_get_sub_graph_pattern_sequence(gp);
   if(seq && raptor_sequence_size(seq) > 0) {
     for(gp_index = 0; 1; gp_index++) {
-      rasqal_graph_pattern* sgp = rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+      rasqal_graph_pattern* sgp;
+
+      sgp = rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
       if(!sgp)
         break;
 
@@ -638,32 +739,132 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
 
 
     
+static void
+rasqal_query_write_data_format_comment(sparql_writer_context* wc,
+                                       raptor_iostream *iostr,
+                                       rasqal_data_graph* dg) 
+{
+  if(dg->format_type || dg->format_name || dg->format_uri) {
+    raptor_iostream_counted_string_write("# format ", 9, iostr);
+    if(dg->format_type) {
+      raptor_iostream_counted_string_write("type ", 5, iostr);
+      raptor_iostream_string_write(dg->format_type, iostr);
+    }
+    if(dg->format_type) {
+      raptor_iostream_counted_string_write("name ", 5, iostr);
+      raptor_iostream_string_write(dg->format_name, iostr);
+    }
+    if(dg->format_type) {
+      raptor_iostream_counted_string_write("uri ", 4, iostr);
+      rasqal_query_write_sparql_uri(wc, iostr, dg->format_uri);
+    }
+  }
+}
+  
+
+
+static int
+rasqal_write_sparql_row(sparql_writer_context* wc,
+                        raptor_iostream* iostr,
+                        rasqal_row* row)
+{
+  int i;
+  
+  raptor_iostream_counted_string_write("( ", 2, iostr);
+  for(i = 0; i < row->size; i++) {
+    rasqal_literal* value = row->values[i];
+    if(i > 0)
+      raptor_iostream_counted_string_write(" ", 1, iostr);
+
+    if(value)
+      rasqal_query_write_sparql_literal(wc, iostr, value);
+    else
+      raptor_iostream_counted_string_write("UNDEF", 5, iostr);
+  }
+  raptor_iostream_counted_string_write(" )", 2, iostr);
+
+  return 0;
+}
+
+
+static int
+rasqal_write_sparql_bindings(sparql_writer_context* wc,
+                             raptor_iostream* iostr,
+                             rasqal_bindings* bindings)
+{
+  raptor_iostream_counted_string_write("BINDINGS", 8, iostr);
+  rasqal_query_write_sparql_select(wc, iostr, bindings->variables);
+  raptor_iostream_counted_string_write(" {\n", 3, iostr);
+
+  if(bindings->rows) {
+    int i;
+  
+    for(i = 0; i < raptor_sequence_size(bindings->rows); i++) {
+      rasqal_row* row;
+      row = raptor_sequence_get_at(bindings->rows, i);
+      raptor_iostream_counted_string_write("  ", 2, iostr);
+      rasqal_write_sparql_row(wc, iostr, row);
+      raptor_iostream_counted_string_write("\n", 1, iostr);
+    }
+  }
+
+  raptor_iostream_counted_string_write("}\n", 2, iostr);
+
+  return 0;
+}
+
+
+static int
+rasqal_query_write_graphref(sparql_writer_context* wc,
+                            raptor_iostream *iostr, 
+                            raptor_uri* uri,
+                            rasqal_update_graph_applies applies)
+{
+  switch(applies) {
+    case RASQAL_UPDATE_GRAPH_ONE:
+      if(uri) {
+        raptor_iostream_counted_string_write(" GRAPH ", 7, iostr);
+        rasqal_query_write_sparql_uri(wc, iostr, uri);
+        break;
+      }
+      /* FALLTHROUGH */
+      
+    case RASQAL_UPDATE_GRAPH_DEFAULT:
+      raptor_iostream_counted_string_write(" DEFAULT", 8, iostr);
+      break;
+      
+    case RASQAL_UPDATE_GRAPH_NAMED:
+      raptor_iostream_counted_string_write(" NAMED", 6, iostr);
+      break;
+      
+    case RASQAL_UPDATE_GRAPH_ALL:
+      raptor_iostream_counted_string_write(" ALL", 4, iostr);
+      break;
+      
+    default:
+      break;
+  }
+
+  return 0;
+}
+
+
+
 int
 rasqal_query_write_sparql_20060406(raptor_iostream *iostr, 
                                    rasqal_query* query, raptor_uri *base_uri)
 {
   int i;
   sparql_writer_context wc;
-#ifndef HAVE_RAPTOR2_API
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-#endif
+  int limit, offset;
+  rasqal_query_verb verb;
 
   wc.world = query->world;
   wc.base_uri = NULL;
 
-#ifdef HAVE_RAPTOR2_API
   wc.type_uri = raptor_new_uri_for_rdf_concept(query->world->raptor_world_ptr,
                                                (const unsigned char*)"type");
   wc.nstack = raptor_new_namespaces(query->world->raptor_world_ptr, 1);
-#else
-  wc.type_uri = raptor_new_uri_for_rdf_concept("type");
-  raptor_uri_get_handler(&uri_handler, &uri_context);
-  wc.nstack = raptor_new_namespaces(uri_handler, uri_context,
-                                    (raptor_simple_message_handler)rasqal_query_simple_error,
-                                    query,
-                                    1);
-#endif
 
   if(base_uri) {
     raptor_iostream_counted_string_write("BASE ", 5, iostr);
@@ -696,22 +897,24 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
   if(query->explain)
     raptor_iostream_counted_string_write("EXPLAIN ", 8, iostr);
 
-  /* Experimental LAQRS INSERT and DELETE (RASQAL_QUERY_VERB_INSERT
-   * and RASQAL_QUERY_VERB_DELETE) are handled in generic code
-   * below */
+  verb = query->verb;
+  
+  /* These terms are deprecated */
   if(query->verb == RASQAL_QUERY_VERB_INSERT ||
      query->verb == RASQAL_QUERY_VERB_DELETE) {
-    raptor_iostream_string_write("\n# Writing deprecated LAQRS INSERT/DELETE\n",
-                                 iostr);
+    verb = RASQAL_QUERY_VERB_UPDATE;
   }
 
   
   /* Write SPARQL 1.1 (Draft) Update forms */
-  if(query->verb == RASQAL_QUERY_VERB_UPDATE) {
+  if(verb == RASQAL_QUERY_VERB_UPDATE) {
     rasqal_update_operation* update;
     /* Write SPARQL Update */
 
     for(i = 0; (update = rasqal_query_get_update_operation(query, i)); i++) {
+      int is_always_2_args = (update->type >= RASQAL_UPDATE_TYPE_ADD &&
+                              update->type <= RASQAL_UPDATE_TYPE_COPY);
+      
       if(update->type == RASQAL_UPDATE_TYPE_UPDATE) {
         /* update operations:
          * WITH ... INSERT { template } DELETE { template } WHERE { template }
@@ -750,28 +953,56 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
         }
       } else {
         /* admin operations:
-         * CLEAR / CLEAR GRAPH graph-uri
-         * CREATE (SILENT) GRAPH graph-uri
+         * CLEAR GRAPH graph-uri | DEFAULT | NAMED | ALL
+         * CREATE (SILENT) GRAPH graph-uri | DEFAULT | NAMED | ALL
          * DROP (SILENT) GRAPH graph-uri
-         * LOAD GRAPH graph-uri / LOAD doc-uri INTO graph-uri
+         * LOAD (SILENT) doc-uri / LOAD (SILENT) doc-uri INTO GRAPH graph-uri
+         * ADD (SILENT) GraphOrDefault TO GraphOrDefault
+         * MOVE (SILENT) GraphOrDefault TO GraphOrDefault
+         * COPY (SILENT) GraphOrDefault TO GraphOrDefault
          */
         raptor_iostream_string_write(rasqal_update_type_label(update->type),
                                      iostr);
         if(update->flags & RASQAL_UPDATE_FLAGS_SILENT)
           raptor_iostream_counted_string_write(" SILENT", 7, iostr);
-        if(update->document_uri) {
+
+        if(is_always_2_args) {
+          /* ADD, MOVE, COPY are always 2-arg admin operations */
+          rasqal_query_write_graphref(&wc, iostr, 
+                                      update->graph_uri,
+                                      RASQAL_UPDATE_GRAPH_ONE);
+
+          raptor_iostream_counted_string_write(" TO", 3, iostr);
+        
+          rasqal_query_write_graphref(&wc, iostr, 
+                                      update->document_uri,
+                                      RASQAL_UPDATE_GRAPH_ONE);
+
+        } else if(update->type == RASQAL_UPDATE_TYPE_LOAD) {
+          /* LOAD is 1 or 2 URIs and first one never has a GRAPH prefix */
+
           raptor_iostream_write_byte(' ', iostr);
+
           rasqal_query_write_sparql_uri(&wc, iostr, update->document_uri);
+
+          if(update->graph_uri) {
+            raptor_iostream_counted_string_write(" INTO", 5, iostr);
+            
+            rasqal_query_write_graphref(&wc, iostr, 
+                                        update->graph_uri,
+                                        RASQAL_UPDATE_GRAPH_ONE);
+          }
+        } else {
+          /* everything else is defined by update->applies; only
+          * CLEAR and DROP may apply to >1 graph
+          */
+          rasqal_query_write_graphref(&wc, iostr, 
+                                      update->graph_uri,
+                                      update->applies);
         }
-        if(update->graph_uri) {
-          if(update->type == RASQAL_UPDATE_TYPE_LOAD &&
-             update->document_uri)
-            raptor_iostream_counted_string_write(" INTO ", 6, iostr);
-          else
-            raptor_iostream_counted_string_write(" GRAPH ", 7, iostr);
-          rasqal_query_write_sparql_uri(&wc, iostr, update->graph_uri);
-          raptor_iostream_write_byte('\n', iostr);
-        }
+        
+        raptor_iostream_write_byte('\n', iostr);
+
       }
     }
 
@@ -779,7 +1010,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
   }
 
 
-  if(query->verb != RASQAL_QUERY_VERB_CONSTRUCT)
+  if(verb != RASQAL_QUERY_VERB_CONSTRUCT)
     raptor_iostream_string_write(rasqal_query_verb_as_string(query->verb),
                                  iostr);
 
@@ -792,7 +1023,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
 
   if(query->wildcard)
     raptor_iostream_counted_string_write(" *", 2, iostr);
-  else if(query->verb == RASQAL_QUERY_VERB_DESCRIBE) {
+  else if(verb == RASQAL_QUERY_VERB_DESCRIBE) {
     raptor_sequence *lit_seq = query->describes;
     int count = raptor_sequence_size(lit_seq);
 
@@ -801,15 +1032,8 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
       raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_literal(&wc, iostr, l);
     }
-  } else if(query->verb == RASQAL_QUERY_VERB_SELECT) {
-    raptor_sequence *var_seq = query->selects;
-    int count = raptor_sequence_size(var_seq);
-
-    for(i = 0; i < count; i++) {
-      rasqal_variable* v = (rasqal_variable*)raptor_sequence_get_at(var_seq, i);
-      raptor_iostream_write_byte(' ', iostr);
-      rasqal_query_write_sparql_variable(&wc, iostr, v);
-    }
+  } else if(verb == RASQAL_QUERY_VERB_SELECT) {
+    rasqal_query_write_sparql_select(&wc, iostr, query->selects);
   }
   raptor_iostream_write_byte('\n', iostr);
 
@@ -822,6 +1046,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
       if(dg->flags & RASQAL_DATA_GRAPH_NAMED)
         continue;
       
+      rasqal_query_write_data_format_comment(&wc, iostr, dg);
       raptor_iostream_counted_string_write("FROM ", 5, iostr);
       rasqal_query_write_sparql_uri(&wc, iostr, dg->uri);
       raptor_iostream_counted_string_write("\n", 1, iostr);
@@ -835,6 +1060,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
       if(!(dg->flags & RASQAL_DATA_GRAPH_NAMED))
         continue;
       
+      rasqal_query_write_data_format_comment(&wc, iostr, dg);
       raptor_iostream_counted_string_write("FROM NAMED ", 11, iostr);
       rasqal_query_write_sparql_uri(&wc, iostr, dg->name_uri);
       raptor_iostream_write_byte('\n', iostr);
@@ -863,7 +1089,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
     raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->group_conditions_sequence) {
+  if(rasqal_query_get_group_conditions_sequence(query)) {
     raptor_iostream_counted_string_write("GROUP BY ", 9, iostr);
     for(i = 0; 1; i++) {
       rasqal_expression* expr = rasqal_query_get_group_condition(query, i);
@@ -877,7 +1103,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
     raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->having_conditions_sequence) {
+  if(rasqal_query_get_having_conditions_sequence(query)) {
     raptor_iostream_counted_string_write("HAVING ", 7, iostr);
     for(i = 0; 1; i++) {
       rasqal_expression* expr = rasqal_query_get_having_condition(query, i);
@@ -891,7 +1117,7 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
     raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->order_conditions_sequence) {
+  if(rasqal_query_get_order_conditions_sequence(query)) {
     raptor_iostream_counted_string_write("ORDER BY ", 9, iostr);
     for(i = 0; 1; i++) {
       rasqal_expression* expr = rasqal_query_get_order_condition(query, i);
@@ -905,20 +1131,24 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
     raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->limit >= 0 || query->offset >= 0) {
-    if(query->limit >= 0) {
+  limit = rasqal_query_get_limit(query);
+  offset = rasqal_query_get_offset(query);
+  if(limit >= 0 || offset >= 0) {
+    if(limit >= 0) {
       raptor_iostream_counted_string_write("LIMIT ", 7, iostr);
-      raptor_iostream_decimal_write(query->limit, iostr);
+      raptor_iostream_decimal_write(limit, iostr);
     }
-    if(query->offset >= 0) {
-      if(query->limit)
+    if(offset >= 0) {
+      if(limit)
         raptor_iostream_write_byte(' ', iostr);
       raptor_iostream_counted_string_write("OFFSET ", 8, iostr);
-      raptor_iostream_decimal_write(query->offset, iostr);
+      raptor_iostream_decimal_write(offset, iostr);
     }
     raptor_iostream_write_byte('\n', iostr);
   }
 
+  if(query->bindings)
+    rasqal_write_sparql_bindings(&wc, iostr, query->bindings);
 
   tidy:
   raptor_free_uri(wc.type_uri);
