@@ -81,33 +81,7 @@ static int rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_d
 static void rasqal_raptor_free_triples_source(void *user_data);
 
 
-#ifndef HAVE_RAPTOR2_API
-static raptor_uri* 
-ordinal_as_uri(rasqal_world* world, int ordinal) 
-{
-  int t = ordinal;
-  size_t len; 
-  unsigned char *buffer;
-  raptor_uri* uri;
-  
-  len = raptor_rdf_namespace_uri_len + 1 + 1; /* 1 for min-length, 1 for '_' */
-  while(t /= 10)
-    len++;
-  buffer = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
-  if(!buffer)
-    return NULL;
-  
-  sprintf((char*)buffer, "%s_%d", raptor_rdf_namespace_uri, ordinal);
-  uri = raptor_new_uri(world->raptor_world_ptr, buffer);
-  RASQAL_FREE(cstring, buffer);
-
-  return uri;
-}
-#endif
-
-
-#ifdef HAVE_RAPTOR2_API
-static rasqal_triple*
+rasqal_triple*
 raptor_statement_as_rasqal_triple(rasqal_world* world,
                                   const raptor_statement *statement)
 {
@@ -163,108 +137,10 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   return rasqal_new_triple(s, p, o);
 }
-#else /* ifdef HAVE_RAPTOR_API */
-static rasqal_triple*
-raptor_statement_as_rasqal_triple(rasqal_world* world,
-                                  const raptor_statement *statement)
-{
-  rasqal_literal *s, *p, *o;
-
-  if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
-    unsigned char *new_blank;
-    size_t blank_len;
-
-    blank_len = strlen((char*)statement->subject);
-    new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    memcpy(new_blank, statement->subject, blank_len + 1);
-    s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
-  } else if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    raptor_uri* uri;
-
-    uri = ordinal_as_uri(world, *((int*)statement->subject));
-    if(!uri)
-      return NULL;
-    s = rasqal_new_uri_literal(world, uri);
-  } else {
-    raptor_uri *uri;
-
-    uri = raptor_uri_copy((raptor_uri*)statement->subject);
-    s = rasqal_new_uri_literal(world, uri);
-  }
-
-  if(statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    raptor_uri* uri = ordinal_as_uri(world, *((int*)statement->predicate));
-
-    if(!uri)
-      return NULL;
-    p = rasqal_new_uri_literal(world, uri);
-  } else {
-    raptor_uri *uri;
-
-    uri = raptor_uri_copy((raptor_uri*)statement->predicate);
-    p = rasqal_new_uri_literal(world, uri);
-  }
-  
-  if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL || 
-     statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
-    unsigned char *string;
-    char *language = NULL;
-    raptor_uri *uri = NULL;
-    size_t literal_len;
-
-    literal_len = strlen((char*)statement->object);
-    string = (unsigned char*)RASQAL_MALLOC(cstring, literal_len + 1);
-    memcpy(string, statement->object, literal_len + 1);
-
-    if(statement->object_literal_language) {
-      size_t language_len;
-      language_len = strlen((const char*)statement->object_literal_language);
-      language = (char*)RASQAL_MALLOC(cstring, language_len + 1);
-      memcpy(language, statement->object_literal_language, language_len + 1);
-    }
-
-    if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
-      const unsigned char* dt_uri_string;
-
-      dt_uri_string = (const unsigned char*)raptor_xml_literal_datatype_uri_string;
-      uri = raptor_new_uri(world->raptor_world_ptr, dt_uri_string);
-    } else if(statement->object_literal_datatype) {
-      raptor_uri *dt_uri = (raptor_uri*)statement->object_literal_datatype;
-
-      uri = raptor_uri_copy(dt_uri);
-    }
-    o = rasqal_new_string_literal(world, string, language, uri, NULL);
-  } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
-    char *blank = (char*)statement->object;
-    unsigned char *new_blank;
-    size_t blank_len;
-
-    blank_len = strlen(blank);
-    new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    memcpy(new_blank, blank, blank_len + 1);
-    o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
-  } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    raptor_uri* uri = ordinal_as_uri(world, *((int*)statement->object));
-
-    if(!uri)
-      return NULL;
-    o = rasqal_new_uri_literal(world, uri);
-  } else {
-    raptor_uri *uri;
-    uri = raptor_uri_copy((raptor_uri*)statement->object);
-    o = rasqal_new_uri_literal(world, uri);
-  }
-
-  return rasqal_new_triple(s, p, o);
-}
-#endif /* ifdef HAVE_RAPTOR2_API ... else */
 
 
 static void
 rasqal_raptor_statement_handler(void *user_data,
-#ifndef HAVE_RAPTOR2_API
-                                const
-#endif
                                 raptor_statement *statement)
 {
   rasqal_raptor_triples_source_user_data* rtsc;
@@ -293,48 +169,8 @@ rasqal_raptor_statement_handler(void *user_data,
 }
 
 
-#ifndef HAVE_RAPTOR2_API
-static void
-rasqal_raptor_error_handler(void *user_data, 
-                            raptor_locator* locator, const char *message)
-{
-  rasqal_query* query = (rasqal_query*)user_data;
-  int locator_len;
-
-  query->failed = 1;
-
-  if(locator &&
-#ifdef HAVE_RAPTOR2_API
-     (locator_len = raptor_locator_format(NULL, 0, locator)) > 0
-#else
-     (locator_len = raptor_format_locator(NULL, 0, locator)) > 0
-#endif
-    ) {
-    char *buffer = (char*)RASQAL_MALLOC(cstring, locator_len + 1);
-#ifdef HAVE_RAPTOR2_API
-    raptor_locator_format(buffer, locator_len, locator);
-#else
-    raptor_format_locator(buffer, locator_len, locator);
-#endif
-
-    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                            &query->locator,
-                            "Failed to parse %s - %s", buffer, message);
-    RASQAL_FREE(cstring, buffer);
-  } else
-    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                            &query->locator,
-                            "Failed to parse - %s", message);
-}
-#endif
-
-
 static unsigned char*
 rasqal_raptor_generate_id_handler(void *user_data,
-#ifdef HAVE_RAPTOR2_API
-#else
-                                  raptor_genid_type type,
-#endif
                                   unsigned char *user_bnodeid) 
 {
   rasqal_raptor_triples_source_user_data* rtsc;
@@ -362,10 +198,27 @@ rasqal_raptor_generate_id_handler(void *user_data,
 
 
 static int
-rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
-                                 void *factory_user_data,
-                                 void *user_data,
-                                 rasqal_triples_source *rts)
+rasqal_raptor_support_feature(void *user_data,
+                              rasqal_triples_source_feature feature)
+{
+  switch(feature) {
+    case RASQAL_TRIPLES_SOURCE_FEATURE_IOSTREAM_DATA_GRAPH:
+      return 1;
+      break;
+      
+    default:
+    case RASQAL_TRIPLES_SOURCE_FEATURE_NONE:
+      return 0;
+  }
+}
+
+
+static int
+rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
+                                  void *factory_user_data,
+                                  void *user_data,
+                                  rasqal_triples_source *rts,
+                                  rasqal_triples_error_handler handler)
 {
   rasqal_raptor_triples_source_user_data* rtsc;
   raptor_parser *parser;
@@ -373,20 +226,27 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
 
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
-  if(!rdf_query->data_graphs)
+  if(!rdf_query->data_graphs) {
+    if(handler)
+      handler(rdf_query, /* locator */ NULL, "Query has no data graphs.");
     return -1;  /* no data */
+  }
 
   /* Max API version this triples source generates */
-  rts->version = 1;
+  rts->version = 2;
   
   rts->init_triples_match = rasqal_raptor_init_triples_match;
   rts->triple_present = rasqal_raptor_triple_present;
   rts->free_triples_source = rasqal_raptor_free_triples_source;
+  rts->support_feature = rasqal_raptor_support_feature;
 
   rtsc->sources_count = raptor_sequence_size(rdf_query->data_graphs);
   /* no default triple source possible */
-  if(!rtsc->sources_count)
+  if(!rtsc->sources_count) {
+    if(handler)
+      handler(rdf_query, /* locator */ NULL, "Query has no data graphs.");
     return -1;  /* no data */
+  }
 
   rtsc->source_literals = (rasqal_literal**)RASQAL_CALLOC(rasqal_literal_ptr,
                                                           rtsc->sources_count,
@@ -396,22 +256,26 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
 
   for(i = 0; i < rtsc->sources_count; i++) {
     rasqal_data_graph *dg;
-    raptor_uri* uri;
+    raptor_uri* uri = NULL;
     raptor_uri* name_uri;
     int free_name_uri = 0;
-
+    const char* parser_name;
+    raptor_iostream* iostr = NULL;
+    
     dg = (rasqal_data_graph*)raptor_sequence_get_at(rdf_query->data_graphs, i);
     uri = dg->uri;
     name_uri = dg->name_uri;
+    iostr = dg->iostr;
 
     rtsc->source_index = i;
-    rtsc->source_uri = raptor_uri_copy(uri);
+    if(uri)
+      rtsc->source_uri = raptor_uri_copy(uri);
 
     if(name_uri)
       rtsc->source_literals[i] = rasqal_new_uri_literal(rdf_query->world, 
                                                         raptor_uri_copy(name_uri)
                                                         );
-    else {
+    else if(uri) {
       name_uri = raptor_uri_copy(uri);
       free_name_uri = 1;
     }
@@ -421,19 +285,23 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
                                                   i);
     rtsc->mapped_id_base_len = strlen((const char*)rtsc->mapped_id_base);
 
-#ifdef HAVE_RAPTOR2_API
-    parser = raptor_new_parser(rdf_query->world->raptor_world_ptr, "guess");
+    parser_name = dg->format_name;
+    if(parser_name) {
+      if(!raptor_world_is_parser_name(rdf_query->world->raptor_world_ptr,
+                                      parser_name)) {
+        handler(rdf_query, /* locator */ NULL,
+                "Invalid data graph parser name ignored");
+        parser_name = NULL;
+      }
+    }
+    if(!parser_name)
+      parser_name = "guess";
+    
+    parser = raptor_new_parser(rdf_query->world->raptor_world_ptr, parser_name);
     raptor_parser_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
     raptor_world_set_generate_bnodeid_handler(rdf_query->world->raptor_world_ptr,
                                               rtsc,
                                               rasqal_raptor_generate_id_handler);
-#else
-    parser = raptor_new_parser("guess");
-    raptor_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
-    raptor_set_error_handler(parser, rdf_query, rasqal_raptor_error_handler);
-    raptor_set_generate_id_handler(parser, rtsc,
-                                   rasqal_raptor_generate_id_handler);
-#endif
 
 #ifdef RAPTOR_FEATURE_NO_NET
     if(rdf_query->features[RASQAL_FEATURE_NO_NET])
@@ -441,11 +309,12 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
                          rdf_query->features[RASQAL_FEATURE_NO_NET]);
 #endif
 
-#ifdef HAVE_RAPTOR2_API
-    raptor_parser_parse_uri(parser, uri, name_uri);
-#else
-    raptor_parse_uri(parser, uri, name_uri);
-#endif
+    if(iostr) {
+      raptor_parser_parse_iostream(parser, iostr, dg->base_uri);
+    } else {
+      raptor_parser_parse_uri(parser, uri, name_uri);
+    }
+    
     raptor_free_parser(parser);
 
     raptor_free_uri(rtsc->source_uri);
@@ -467,6 +336,18 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
 }
 
 
+static int
+rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
+                                 void *factory_user_data,
+                                 void *user_data,
+                                 rasqal_triples_source *rts)
+{
+  return rasqal_raptor_init_triples_source(rdf_query, factory_user_data,
+                                           user_data, rts,
+                                           rasqal_triples_source_error_handler);
+}
+
+
 /**
  * rasqal_raptor_triple_match:
  * @world: rasqal_world object
@@ -480,7 +361,7 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
  * 
  * Return value: non-0 on match
  **/
-static int
+int
 rasqal_raptor_triple_match(rasqal_world* world,
                            rasqal_triple *triple,
                            rasqal_triple *match,
@@ -602,9 +483,12 @@ rasqal_raptor_free_triples_source(void *user_data)
 static int
 rasqal_raptor_register_triples_source_factory(rasqal_triples_source_factory *factory) 
 {
-  factory->version = 1;
+  factory->version = 2;
   factory->user_data_size = sizeof(rasqal_raptor_triples_source_user_data);
+  /* V1 */
   factory->new_triples_source = rasqal_raptor_new_triples_source;
+  /* V2 */
+  factory->init_triples_source = rasqal_raptor_init_triples_source;
 
   return 0;
 }
