@@ -124,6 +124,9 @@ rasqal_project_rowsource_finish(rasqal_rowsource* rowsource, void *user_data)
   if(con->rowsource)
     rasqal_free_rowsource(con->rowsource);
   
+  if(con->projection_variables)
+    raptor_free_sequence(con->projection_variables);
+  
   if(con->projection)
     RASQAL_FREE(int, con->projection);
   
@@ -138,45 +141,64 @@ rasqal_project_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
 {
   rasqal_project_rowsource_context *con;
   rasqal_row *row = NULL;
+  rasqal_row* nrow = NULL;
   
   con = (rasqal_project_rowsource_context*)user_data;
 
   row = rasqal_rowsource_read_row(con->rowsource);
   if(row) {
-    rasqal_row* nrow;
     int i;
     
     nrow = rasqal_new_row_for_size(rowsource->world, rowsource->size);
-    if(!nrow) {
-      rasqal_free_row(row);
-      row = NULL;
-    } else {
-      nrow->rowsource = rowsource;
-      nrow->offset = row->offset;
+    if(!nrow)
+      goto failed;
+
+    nrow->rowsource = rowsource;
+    nrow->offset = row->offset;
       
-      for(i = 0; i < rowsource->size; i++) {
-        int offset = con->projection[i];
-        if(offset >= 0)
-          nrow->values[i] = rasqal_new_literal_from_literal(row->values[offset]);
-        else {
-          rasqal_variable* v;
-          rasqal_query *query = rowsource->query;
+    for(i = 0; i < rowsource->size; i++) {
+      int offset = con->projection[i];
+      if(offset >= 0)
+        nrow->values[i] = rasqal_new_literal_from_literal(row->values[offset]);
+      else {
+        rasqal_variable* v;
+        rasqal_query *query = rowsource->query;
+        
+        v = (rasqal_variable*)raptor_sequence_get_at(con->projection_variables, i);
+        if(v && v->expression) {
+          int error = 0;
+
+          if(v->value)
+            rasqal_free_literal(v->value);
           
-          v = (rasqal_variable*)raptor_sequence_get_at(con->projection_variables, i);
-          if(v && v->expression) {
-            nrow->values[i] = rasqal_expression_evaluate(rowsource->world,
-                                                         &query->locator,
-                                                         v->expression,
-                                                         query->compare_flags);
-          }
+          v->value = rasqal_expression_evaluate2(v->expression,
+                                                 query->eval_context,
+                                                 &error);
+          if(error) {
+            /* FIXME: Errors are ignored - check this */
+#if 0
+            goto failed;
+#endif
+          } else
+            nrow->values[i] = rasqal_new_literal_from_literal(v->value);
+
         }
       }
-      rasqal_free_row(row);
-      row = nrow;
     }
+
+    rasqal_free_row(row);
+    row = nrow;
   }
   
   return row;
+
+  failed:
+  if(nrow)
+    rasqal_free_row(nrow);
+  if(row)
+    rasqal_free_row(row);
+  
+  return NULL;
 }
 
 
@@ -229,6 +251,19 @@ static const rasqal_rowsource_handler rasqal_project_rowsource_handler = {
 };
 
 
+/**
+ * rasqal_new_project_rowsource:
+ * @world: query world
+ * @query: query results object
+ * @rowsource: input rowsource
+ * @projection_variables: input sequence of #rasqal_variable
+ *
+ * INTERNAL - create a PROJECTion over input rowsource
+ *
+ * The @rowsource becomes owned by the new rowsource.
+ *
+ * Return value: new rowsource or NULL on failure
+ */
 rasqal_rowsource*
 rasqal_new_project_rowsource(rasqal_world *world,
                              rasqal_query *query,
@@ -246,7 +281,7 @@ rasqal_new_project_rowsource(rasqal_world *world,
     goto fail;
 
   con->rowsource = rowsource;
-  con->projection_variables = projection_variables;
+  con->projection_variables = rasqal_variable_copy_variable_sequence(projection_variables);
 
   return rasqal_new_rowsource_from_handler(world, query,
                                            con,
@@ -338,13 +373,16 @@ main(int argc, char *argv[])
   /* vars_seq and seq are now owned by input_rs */
   vars_seq = seq = NULL;
   
-  projection_seq = raptor_new_sequence(NULL, NULL);
+  projection_seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                       (raptor_data_print_handler)rasqal_variable_print);
   for(i = 0 ; i < EXPECTED_COLUMNS_COUNT; i++) {
     rasqal_variable* v;
     unsigned const char* name=(unsigned const char*)project_1_var_names[i];
     v = rasqal_variables_table_get_by_name(vt, name);
-    if(v)
+    if(v) {
+      v = rasqal_new_variable_from_variable(v);
       raptor_sequence_push(projection_seq, v);
+    }
   }
 
   rowsource = rasqal_new_project_rowsource(world, query, input_rs, projection_seq);
