@@ -123,7 +123,7 @@ rasqal_xsd_check_dateTime_format(const unsigned char* string, int flags)
   /* This should be correct according to 
    * http://www.w3.org/TR/xmlschema-2/#dateTime
    */
-  return rasqal_xsd_datetime_check(string);
+  return rasqal_xsd_datetime_check((const char*)string);
 }
 
 
@@ -331,7 +331,7 @@ unsigned char*
 rasqal_xsd_format_double(double d, size_t *len_p)
 {
   size_t e_index = 0;
-  size_t trailing_zero_start = 0;
+  int trailing_zero_start = -1;
   size_t exponent_start;
   size_t len = 0;
   unsigned char* buf = NULL;
@@ -365,32 +365,39 @@ rasqal_xsd_format_double(double d, size_t *len_p)
       break;
   }
 
-  if(buf[trailing_zero_start-1] == '.')
-    ++trailing_zero_start;
+  if(trailing_zero_start >= 0) {
+    if(buf[trailing_zero_start-1] == '.')
+      ++trailing_zero_start;
 
-  /* write an 'e' where the trailing zeros started */
-  buf[trailing_zero_start] = 'e';
-  if(buf[e_index + 1] == '-') {
-    buf[trailing_zero_start + 1] = '-';
-    ++trailing_zero_start;
+    /* write an 'E' where the trailing zeros started */
+    buf[trailing_zero_start] = 'E';
+    if(buf[e_index + 1] == '-') {
+      buf[trailing_zero_start + 1] = '-';
+      ++trailing_zero_start;
+    }
+  } else {
+    buf[e_index] = 'E';
+    trailing_zero_start = e_index + 1;
   }
-
+  
   exponent_start = e_index+2;
   while(buf[exponent_start] == '0')
     ++exponent_start;
 
-  len = strlen((const char*)buf);
-  if(exponent_start == len) {
-    len=trailing_zero_start+2;
-    buf[len-1] = '0';
-    buf[len] = '\0';
-  } else {
-    /* copy the exponent (minus leading zeros) after the new E */
-    memmove(buf+trailing_zero_start + 1, buf+exponent_start,
-            len-trailing_zero_start);
+  if(trailing_zero_start >= 0) {
     len = strlen((const char*)buf);
+    if(exponent_start == len) {
+      len = trailing_zero_start + 2;
+      buf[len-1] = '0';
+      buf[len] = '\0';
+    } else {
+      /* copy the exponent (minus leading zeros) after the new E */
+      memmove(buf + trailing_zero_start + 1, buf + exponent_start,
+              len - trailing_zero_start);
+      len = strlen((const char*)buf);
+    }
   }
-
+  
   if(len_p)
     *len_p = len;
 
