@@ -580,14 +580,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   198,   198,   202,   231,   236,   241,   254,   258,   265,
-     269,   274,   283,   288,   293,   306,   310,   314,   318,   329,
-     334,   339,   343,   347,   355,   360,   365,   370,   375,   383,
-     387,   393,   398,   404,   408,   412,   416,   420,   426,   430,
-     434,   440,   444,   448,   452,   456,   462,   469,   473,   477,
-     483,   487,   491,   495,   501,   505,   509,   515,   519,   523,
-     528,   532,   538,   542,   546,   552,   556,   562,   568,   574,
-     578,   582,   586,   590,   594,   597,   604,   609
+       0,   198,   198,   202,   237,   242,   247,   256,   260,   267,
+     271,   276,   285,   290,   295,   308,   312,   316,   320,   331,
+     336,   341,   345,   349,   357,   362,   367,   372,   377,   385,
+     389,   395,   400,   406,   410,   414,   418,   422,   428,   432,
+     436,   442,   446,   450,   454,   458,   464,   471,   475,   479,
+     485,   489,   493,   497,   503,   507,   511,   517,   521,   525,
+     530,   534,   540,   544,   548,   554,   558,   564,   571,   577,
+     581,   585,   589,   593,   597,   600,   607,   612
 };
 #endif
 
@@ -1701,8 +1701,14 @@ yyreduce:
     int i;
     
     for(i=0; i < raptor_sequence_size((yyvsp[(3) - (7)].seq)); i++) {
-      raptor_uri* uri=(raptor_uri*)raptor_sequence_get_at((yyvsp[(3) - (7)].seq), i);
-      rasqal_query_add_data_graph((rasqal_query*)rq, uri, NULL, RASQAL_DATA_GRAPH_BACKGROUND);
+      raptor_uri* uri = (raptor_uri*)raptor_sequence_get_at((yyvsp[(3) - (7)].seq), i);
+      rasqal_data_graph* dg;
+      
+      dg = rasqal_new_data_graph_from_uri(((rasqal_query*)rq)->world, uri, NULL,
+                                          RASQAL_DATA_GRAPH_BACKGROUND,
+                                          NULL, NULL, NULL);
+      if(dg)
+        rasqal_query_add_data_graph((rasqal_query*)rq, dg);
     }
     raptor_free_sequence((yyvsp[(3) - (7)].seq));
   }
@@ -1724,7 +1730,7 @@ yyreduce:
   case 4:
 
 /* Line 1455 of yacc.c  */
-#line 232 "./rdql_parser.y"
+#line 238 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].variable));
@@ -1734,7 +1740,7 @@ yyreduce:
   case 5:
 
 /* Line 1455 of yacc.c  */
-#line 237 "./rdql_parser.y"
+#line 243 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].variable));
@@ -1744,14 +1750,10 @@ yyreduce:
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 242 "./rdql_parser.y"
+#line 248 "./rdql_parser.y"
     {
-  /* The variables are freed from the rasqal_query field variables */
-#ifdef HAVE_RAPTOR2_API
-  (yyval.seq) = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
-#else
-  (yyval.seq) = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
-#endif
+  (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                           (raptor_data_print_handler)rasqal_variable_print);
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].variable));
 }
     break;
@@ -1759,7 +1761,7 @@ yyreduce:
   case 7:
 
 /* Line 1455 of yacc.c  */
-#line 255 "./rdql_parser.y"
+#line 257 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (1)].seq);
 }
@@ -1768,7 +1770,7 @@ yyreduce:
   case 8:
 
 /* Line 1455 of yacc.c  */
-#line 259 "./rdql_parser.y"
+#line 261 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
   ((rasqal_query*)rq)->wildcard=1;
@@ -1778,7 +1780,7 @@ yyreduce:
   case 9:
 
 /* Line 1455 of yacc.c  */
-#line 266 "./rdql_parser.y"
+#line 268 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1787,7 +1789,7 @@ yyreduce:
   case 10:
 
 /* Line 1455 of yacc.c  */
-#line 270 "./rdql_parser.y"
+#line 272 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1796,7 +1798,7 @@ yyreduce:
   case 11:
 
 /* Line 1455 of yacc.c  */
-#line 274 "./rdql_parser.y"
+#line 276 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
 }
@@ -1805,7 +1807,7 @@ yyreduce:
   case 12:
 
 /* Line 1455 of yacc.c  */
-#line 284 "./rdql_parser.y"
+#line 286 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].triple));
@@ -1815,7 +1817,7 @@ yyreduce:
   case 13:
 
 /* Line 1455 of yacc.c  */
-#line 289 "./rdql_parser.y"
+#line 291 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].triple));
@@ -1825,7 +1827,7 @@ yyreduce:
   case 14:
 
 /* Line 1455 of yacc.c  */
-#line 294 "./rdql_parser.y"
+#line 296 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->triples;
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].triple));
@@ -1835,7 +1837,7 @@ yyreduce:
   case 15:
 
 /* Line 1455 of yacc.c  */
-#line 307 "./rdql_parser.y"
+#line 309 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (7)].literal), (yyvsp[(4) - (7)].literal), (yyvsp[(6) - (7)].literal));
 }
@@ -1844,7 +1846,7 @@ yyreduce:
   case 16:
 
 /* Line 1455 of yacc.c  */
-#line 311 "./rdql_parser.y"
+#line 313 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (6)].literal), (yyvsp[(3) - (6)].literal), (yyvsp[(5) - (6)].literal));
 }
@@ -1853,7 +1855,7 @@ yyreduce:
   case 17:
 
 /* Line 1455 of yacc.c  */
-#line 315 "./rdql_parser.y"
+#line 317 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (6)].literal), (yyvsp[(4) - (6)].literal), (yyvsp[(5) - (6)].literal));
 }
@@ -1862,7 +1864,7 @@ yyreduce:
   case 18:
 
 /* Line 1455 of yacc.c  */
-#line 319 "./rdql_parser.y"
+#line 321 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (5)].literal), (yyvsp[(3) - (5)].literal), (yyvsp[(4) - (5)].literal));
 }
@@ -1871,7 +1873,7 @@ yyreduce:
   case 19:
 
 /* Line 1455 of yacc.c  */
-#line 330 "./rdql_parser.y"
+#line 332 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (2)].expr);
 }
@@ -1880,7 +1882,7 @@ yyreduce:
   case 20:
 
 /* Line 1455 of yacc.c  */
-#line 334 "./rdql_parser.y"
+#line 336 "./rdql_parser.y"
     {
   (yyval.expr)=NULL;
 }
@@ -1889,7 +1891,7 @@ yyreduce:
   case 21:
 
 /* Line 1455 of yacc.c  */
-#line 340 "./rdql_parser.y"
+#line 342 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1898,7 +1900,7 @@ yyreduce:
   case 22:
 
 /* Line 1455 of yacc.c  */
-#line 344 "./rdql_parser.y"
+#line 346 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1907,7 +1909,7 @@ yyreduce:
   case 23:
 
 /* Line 1455 of yacc.c  */
-#line 348 "./rdql_parser.y"
+#line 350 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -1916,7 +1918,7 @@ yyreduce:
   case 24:
 
 /* Line 1455 of yacc.c  */
-#line 356 "./rdql_parser.y"
+#line 358 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1925,7 +1927,7 @@ yyreduce:
   case 25:
 
 /* Line 1455 of yacc.c  */
-#line 360 "./rdql_parser.y"
+#line 362 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
 }
@@ -1934,7 +1936,7 @@ yyreduce:
   case 26:
 
 /* Line 1455 of yacc.c  */
-#line 366 "./rdql_parser.y"
+#line 368 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (5)].name), (yyvsp[(3) - (5)].uri)));
@@ -1944,7 +1946,7 @@ yyreduce:
   case 27:
 
 /* Line 1455 of yacc.c  */
-#line 371 "./rdql_parser.y"
+#line 373 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (4)].name), (yyvsp[(3) - (4)].uri)));
@@ -1954,7 +1956,7 @@ yyreduce:
   case 28:
 
 /* Line 1455 of yacc.c  */
-#line 376 "./rdql_parser.y"
+#line 378 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_push((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (3)].name), (yyvsp[(3) - (3)].uri)));
@@ -1964,7 +1966,7 @@ yyreduce:
   case 29:
 
 /* Line 1455 of yacc.c  */
-#line 384 "./rdql_parser.y"
+#line 386 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_OR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1973,7 +1975,7 @@ yyreduce:
   case 30:
 
 /* Line 1455 of yacc.c  */
-#line 388 "./rdql_parser.y"
+#line 390 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -1982,7 +1984,7 @@ yyreduce:
   case 31:
 
 /* Line 1455 of yacc.c  */
-#line 394 "./rdql_parser.y"
+#line 396 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 ;
@@ -1992,7 +1994,7 @@ yyreduce:
   case 32:
 
 /* Line 1455 of yacc.c  */
-#line 399 "./rdql_parser.y"
+#line 401 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2001,7 +2003,7 @@ yyreduce:
   case 33:
 
 /* Line 1455 of yacc.c  */
-#line 405 "./rdql_parser.y"
+#line 407 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2010,7 +2012,7 @@ yyreduce:
   case 34:
 
 /* Line 1455 of yacc.c  */
-#line 409 "./rdql_parser.y"
+#line 411 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_NEQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2019,7 +2021,7 @@ yyreduce:
   case 35:
 
 /* Line 1455 of yacc.c  */
-#line 413 "./rdql_parser.y"
+#line 415 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_string_op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_MATCH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].literal));
 }
@@ -2028,7 +2030,7 @@ yyreduce:
   case 36:
 
 /* Line 1455 of yacc.c  */
-#line 417 "./rdql_parser.y"
+#line 419 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_string_op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_NMATCH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].literal));
 }
@@ -2037,7 +2039,7 @@ yyreduce:
   case 37:
 
 /* Line 1455 of yacc.c  */
-#line 421 "./rdql_parser.y"
+#line 423 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2046,7 +2048,7 @@ yyreduce:
   case 38:
 
 /* Line 1455 of yacc.c  */
-#line 427 "./rdql_parser.y"
+#line 429 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2055,7 +2057,7 @@ yyreduce:
   case 39:
 
 /* Line 1455 of yacc.c  */
-#line 431 "./rdql_parser.y"
+#line 433 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_NEQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2064,7 +2066,7 @@ yyreduce:
   case 40:
 
 /* Line 1455 of yacc.c  */
-#line 435 "./rdql_parser.y"
+#line 437 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2073,7 +2075,7 @@ yyreduce:
   case 41:
 
 /* Line 1455 of yacc.c  */
-#line 441 "./rdql_parser.y"
+#line 443 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_LT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2082,7 +2084,7 @@ yyreduce:
   case 42:
 
 /* Line 1455 of yacc.c  */
-#line 445 "./rdql_parser.y"
+#line 447 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_GT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2091,7 +2093,7 @@ yyreduce:
   case 43:
 
 /* Line 1455 of yacc.c  */
-#line 449 "./rdql_parser.y"
+#line 451 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_LE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2100,7 +2102,7 @@ yyreduce:
   case 44:
 
 /* Line 1455 of yacc.c  */
-#line 453 "./rdql_parser.y"
+#line 455 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_GE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2109,7 +2111,7 @@ yyreduce:
   case 45:
 
 /* Line 1455 of yacc.c  */
-#line 457 "./rdql_parser.y"
+#line 459 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2118,7 +2120,7 @@ yyreduce:
   case 46:
 
 /* Line 1455 of yacc.c  */
-#line 463 "./rdql_parser.y"
+#line 465 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2127,7 +2129,7 @@ yyreduce:
   case 47:
 
 /* Line 1455 of yacc.c  */
-#line 470 "./rdql_parser.y"
+#line 472 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_PLUS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2136,7 +2138,7 @@ yyreduce:
   case 48:
 
 /* Line 1455 of yacc.c  */
-#line 474 "./rdql_parser.y"
+#line 476 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_MINUS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2145,7 +2147,7 @@ yyreduce:
   case 49:
 
 /* Line 1455 of yacc.c  */
-#line 478 "./rdql_parser.y"
+#line 480 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2154,7 +2156,7 @@ yyreduce:
   case 50:
 
 /* Line 1455 of yacc.c  */
-#line 484 "./rdql_parser.y"
+#line 486 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STAR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2163,7 +2165,7 @@ yyreduce:
   case 51:
 
 /* Line 1455 of yacc.c  */
-#line 488 "./rdql_parser.y"
+#line 490 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_SLASH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2172,7 +2174,7 @@ yyreduce:
   case 52:
 
 /* Line 1455 of yacc.c  */
-#line 492 "./rdql_parser.y"
+#line 494 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_REM, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2181,7 +2183,7 @@ yyreduce:
   case 53:
 
 /* Line 1455 of yacc.c  */
-#line 496 "./rdql_parser.y"
+#line 498 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2190,7 +2192,7 @@ yyreduce:
   case 54:
 
 /* Line 1455 of yacc.c  */
-#line 502 "./rdql_parser.y"
+#line 504 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (2)].expr);
 }
@@ -2199,7 +2201,7 @@ yyreduce:
   case 55:
 
 /* Line 1455 of yacc.c  */
-#line 506 "./rdql_parser.y"
+#line 508 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_UMINUS, (yyvsp[(2) - (2)].expr));
 }
@@ -2208,7 +2210,7 @@ yyreduce:
   case 56:
 
 /* Line 1455 of yacc.c  */
-#line 510 "./rdql_parser.y"
+#line 512 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2217,7 +2219,7 @@ yyreduce:
   case 57:
 
 /* Line 1455 of yacc.c  */
-#line 516 "./rdql_parser.y"
+#line 518 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_TILDE, (yyvsp[(2) - (2)].expr));
 }
@@ -2226,7 +2228,7 @@ yyreduce:
   case 58:
 
 /* Line 1455 of yacc.c  */
-#line 520 "./rdql_parser.y"
+#line 522 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_BANG, (yyvsp[(2) - (2)].expr));
 }
@@ -2235,7 +2237,7 @@ yyreduce:
   case 59:
 
 /* Line 1455 of yacc.c  */
-#line 524 "./rdql_parser.y"
+#line 526 "./rdql_parser.y"
     {
   rasqal_literal *l=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
   (yyval.expr)=rasqal_new_literal_expression(((rasqal_query*)rq)->world, l);
@@ -2245,7 +2247,7 @@ yyreduce:
   case 60:
 
 /* Line 1455 of yacc.c  */
-#line 529 "./rdql_parser.y"
+#line 531 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_literal_expression(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].literal));
 }
@@ -2254,7 +2256,7 @@ yyreduce:
   case 61:
 
 /* Line 1455 of yacc.c  */
-#line 533 "./rdql_parser.y"
+#line 535 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (3)].expr);
 }
@@ -2263,7 +2265,7 @@ yyreduce:
   case 62:
 
 /* Line 1455 of yacc.c  */
-#line 539 "./rdql_parser.y"
+#line 541 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
 }
@@ -2272,7 +2274,7 @@ yyreduce:
   case 63:
 
 /* Line 1455 of yacc.c  */
-#line 543 "./rdql_parser.y"
+#line 545 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_uri_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].uri));
 }
@@ -2281,7 +2283,7 @@ yyreduce:
   case 64:
 
 /* Line 1455 of yacc.c  */
-#line 547 "./rdql_parser.y"
+#line 549 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_simple_literal(((rasqal_query*)rq)->world, RASQAL_LITERAL_QNAME, (yyvsp[(1) - (1)].name));
 }
@@ -2290,7 +2292,7 @@ yyreduce:
   case 65:
 
 /* Line 1455 of yacc.c  */
-#line 553 "./rdql_parser.y"
+#line 555 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
 }
@@ -2299,7 +2301,7 @@ yyreduce:
   case 66:
 
 /* Line 1455 of yacc.c  */
-#line 557 "./rdql_parser.y"
+#line 559 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2308,16 +2310,17 @@ yyreduce:
   case 67:
 
 /* Line 1455 of yacc.c  */
-#line 563 "./rdql_parser.y"
+#line 565 "./rdql_parser.y"
     {
-  (yyval.variable)=rasqal_new_variable((rasqal_query*)rq, (yyvsp[(2) - (2)].name), NULL);
+  (yyval.variable) = rasqal_variables_table_add(((rasqal_query*)rq)->vars_table,
+                                  RASQAL_VARIABLE_TYPE_NORMAL, (yyvsp[(2) - (2)].name), NULL);
 }
     break;
 
   case 68:
 
 /* Line 1455 of yacc.c  */
-#line 569 "./rdql_parser.y"
+#line 572 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2326,7 +2329,7 @@ yyreduce:
   case 69:
 
 /* Line 1455 of yacc.c  */
-#line 575 "./rdql_parser.y"
+#line 578 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_uri_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].uri));
 }
@@ -2335,7 +2338,7 @@ yyreduce:
   case 70:
 
 /* Line 1455 of yacc.c  */
-#line 579 "./rdql_parser.y"
+#line 582 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2344,7 +2347,7 @@ yyreduce:
   case 71:
 
 /* Line 1455 of yacc.c  */
-#line 583 "./rdql_parser.y"
+#line 586 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2353,7 +2356,7 @@ yyreduce:
   case 72:
 
 /* Line 1455 of yacc.c  */
-#line 587 "./rdql_parser.y"
+#line 590 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2362,7 +2365,7 @@ yyreduce:
   case 73:
 
 /* Line 1455 of yacc.c  */
-#line 591 "./rdql_parser.y"
+#line 594 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2371,7 +2374,7 @@ yyreduce:
   case 74:
 
 /* Line 1455 of yacc.c  */
-#line 595 "./rdql_parser.y"
+#line 598 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2380,7 +2383,7 @@ yyreduce:
   case 75:
 
 /* Line 1455 of yacc.c  */
-#line 598 "./rdql_parser.y"
+#line 601 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_simple_literal(((rasqal_query*)rq)->world, RASQAL_LITERAL_QNAME, (yyvsp[(1) - (1)].name));
 }
@@ -2389,7 +2392,7 @@ yyreduce:
   case 76:
 
 /* Line 1455 of yacc.c  */
-#line 605 "./rdql_parser.y"
+#line 608 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(3) - (3)].seq);
   raptor_sequence_shift((yyval.seq), (yyvsp[(1) - (3)].uri));
@@ -2399,15 +2402,10 @@ yyreduce:
   case 77:
 
 /* Line 1455 of yacc.c  */
-#line 610 "./rdql_parser.y"
+#line 613 "./rdql_parser.y"
     {
-#ifdef HAVE_RAPTOR2_API
   (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)raptor_free_uri,
                            (raptor_data_print_handler)raptor_uri_print);
-#else
-  (yyval.seq) = raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri,
-                           (raptor_sequence_print_handler*)raptor_uri_print);
-#endif
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].uri));
 }
     break;
@@ -2415,7 +2413,7 @@ yyreduce:
 
 
 /* Line 1455 of yacc.c  */
-#line 2419 "rdql_parser.c"
+#line 2417 "rdql_parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2627,7 +2625,7 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 622 "./rdql_parser.y"
+#line 620 "./rdql_parser.y"
 
 
 
@@ -2841,27 +2839,47 @@ rdql_syntax_warning(rasqal_query *rq, const char *message, ...)
 }
 
 
-static void
+static const char* const rdql_names[] = { "rdql", NULL};
+
+static const char* const rdql_uri_strings[] = {
+  "http://jena.hpl.hp.com/2003/07/query/RDQL",
+  NULL
+};
+
+static const raptor_type_q rdql_types[] = {
+  { "application/x-rdql", 18, 10}, 
+  { NULL, 0, 0}
+};
+
+static int
 rasqal_rdql_query_language_register_factory(rasqal_query_language_factory *factory)
 {
+  int rc = 0;
+
+  factory->desc.names = rdql_names;
+
+  factory->desc.mime_types = rdql_types;
+
+  factory->desc.label = "RDF Data Query Language (RDQL)";
+
+  /* http://www.w3.org/Submission/2004/SUBM-RDQL-20040109/ */
+  factory->desc.uri_strings = rdql_uri_strings;
+
   factory->context_length = sizeof(rasqal_rdql_query_language);
 
   factory->init      = rasqal_rdql_query_language_init;
   factory->terminate = rasqal_rdql_query_language_terminate;
   factory->prepare   = rasqal_rdql_query_language_prepare;
+
+  return rc;
 }
 
 
 int
-rasqal_init_query_language_rdql(rasqal_world* world) {
-  /* http://www.w3.org/Submission/2004/SUBM-RDQL-20040109/ */
-
-  return rasqal_query_language_register_factory(world,
-                                                "rdql", 
-                                                "RDF Data Query Language (RDQL)",
-                                                NULL,
-                                                (const unsigned char*)"http://jena.hpl.hp.com/2003/07/query/RDQL",
-                                                &rasqal_rdql_query_language_register_factory);
+rasqal_init_query_language_rdql(rasqal_world* world)
+{
+  return !rasqal_query_language_register_factory(world,
+                                                 &rasqal_rdql_query_language_register_factory);
 }
 
 
